@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -33,20 +33,32 @@ class MKAutomationError(Exception):
 
 def do_automation(cmd, args):
     try:
-        if cmd == "try-inventory":
-            result = automation_try_inventory(args)
-        elif cmd == "inventory":
-            result = automation_inventory(args)
-        elif cmd == "get-autochecks":
-            result = automation_get_autochecks(args)
-        elif cmd == "set-autochecks":
-            result = automation_set_autochecks(args)
+        if cmd == "get-configuration":
+            read_config_files(with_autochecks=False, with_conf_d=False)
+            result = automation_get_configuration()
+        elif cmd == "get-check-information":
+            result = automation_get_check_information()
         elif cmd == "delete-host":
+            read_config_files(with_autochecks=False)
             result = automation_delete_host(args)
-        elif cmd == "restart":
-	    result = automation_restart()
         else:
-            raise MKAutomationError("Automation command '%s' is not implemented." % cmd)
+            read_config_files()
+            if cmd == "try-inventory":
+                result = automation_try_inventory(args)
+            elif cmd == "inventory":
+                result = automation_inventory(args)
+            elif cmd == "get-autochecks":
+                result = automation_get_autochecks(args)
+            elif cmd == "set-autochecks":
+                result = automation_set_autochecks(args)
+            elif cmd == "reload":
+                result = automation_restart("reload")
+            elif cmd == "restart":
+                result = automation_restart("restart")
+            elif cmd == "scan-parents":
+                result = automation_scan_parents(args)
+            else:
+                raise MKAutomationError("Automation command '%s' is not implemented." % cmd)
 
     except MKAutomationError, e:
         sys.stderr.write("%s\n" % e)
@@ -95,8 +107,8 @@ def automation_inventory(args):
     # Create new list of checks
     new_items = []
     for entry in table:
-        state_type, ct, item, paramstring = entry[:4]
-        if state_type in [ "legacy", "manual", "ignored" ]:
+        state_type, ct, checkgroup, item, paramstring = entry[:5]
+        if state_type in [ "legacy", "active", "manual", "ignored" ]:
             continue # this is not an autocheck or ignored and currently not checked
 
         if state_type == "new":
@@ -108,7 +120,7 @@ def automation_inventory(args):
             # keep currently existing valid services in any case
             new_items.append((ct, item, paramstring))
             count_kept += 1
-        
+
         elif state_type in [ "obsolete", "vanished" ]:
             # keep item, if we are currently only looking for new services
             # otherwise fix it: remove ignored and non-longer existing services
@@ -123,39 +135,93 @@ def automation_inventory(args):
 
 
 def automation_try_inventory(args):
+    global opt_use_cachefile, inventory_max_cachefile_age, check_max_cachefile_age
     if args[0] == '--cache':
-        global opt_use_cachefile, inventory_max_cachefile_age, check_max_cachefile_age
         opt_use_cachefile = True
         check_max_cachefile_age = 1000000000
         inventory_max_cachefile_age = 1000000000
         args = args[1:]
 
     hostname = args[0]
+
+    # hostname might be a cluster. In that case we compute the clustered
+    # services of that cluster.
+    services = []
+    if is_cluster(hostname):
+        already_added = set([])
+        for node in nodes_of(hostname):
+            new_services = automation_try_inventory_node(node)
+            for entry in new_services:
+                if host_of_clustered_service(node, entry[6]) == hostname:
+                    # 1: check, 6: Service description
+                    if (entry[1], entry[6]) not in already_added:
+                        services.append(entry)
+                        already_added.add((entry[1], entry[6])) # make it unique
+
+    else:
+        new_services = automation_try_inventory_node(hostname)
+        for entry in new_services:
+            if host_of_clustered_service(hostname, entry[6]) == hostname:
+                services.append(entry)
+
+    return services
+
+
+
+def automation_try_inventory_node(hostname):
+    global opt_use_cachefile, opt_no_tcp, opt_dont_submit
+
     try:
         ipaddress = lookup_ipaddress(hostname)
     except:
         raise MKAutomationError("Cannot lookup IP address of host %s" % hostname)
 
-    f = []
+    found_services = []
+
+    dual_host = is_snmp_host(hostname) and is_tcp_host(hostname)
 
     # if we are using cache files, then we restrict us to existing
     # check types. SNMP scan is only done without the --cache option
+    snmp_error = None
     if is_snmp_host(hostname):
-        if opt_use_cachefile:
-            existing_checks = set([ cn for (cn, item) in get_check_table(hostname) ])
-            for cn in inventorable_checktypes("snmp"):
-                if cn in existing_checks:
-                    f += make_inventory(cn, [hostname], True, True)
-        else:
-            f = do_snmp_scan([hostname], True, True)
+        try:
+            if opt_use_cachefile:
+                existing_checks = set([ cn for (cn, item) in get_check_table(hostname) ])
+                for cn in inventorable_checktypes("snmp"):
+                    if cn in existing_checks:
+                        found_services += make_inventory(cn, [hostname], True, True)
+            else:
+                sys_descr = get_single_oid(hostname, ipaddress, ".1.3.6.1.2.1.1.1.0")
+                if sys_descr != None:
+                    found_services = do_snmp_scan([hostname], True, True)
+                else:
+                    raise MKSNMPError("Cannot get system description via SNMP. "
+                                      "SNMP agent is not responding. Probably wrong "
+                                      "community or wrong SNMP version.")
 
+        except Exception, e:
+            if not dual_host:
+                raise
+            snmp_error = str(e)
+
+    tcp_error = None
     if is_tcp_host(hostname):
-        for cn in inventorable_checktypes("tcp"):
-            f += make_inventory(cn, [hostname], True, True)
+        try:
+            for cn in inventorable_checktypes("tcp"):
+                found_services += make_inventory(cn, [hostname], True, True)
+        except Exception, e:
+            if not dual_host:
+                raise
+            tcp_error = str(e)
+
+    # raise MKAutomationError("%s/%s/%s" % (dual_host, snmp_error, tcp_error))
+    if dual_host and snmp_error and tcp_error:
+        raise MKAutomationError("Error using TCP (%s)\nand SNMP (%s)" %
+                (tcp_error, snmp_error))
 
     found = {}
-    for hn, ct, item, paramstring, state_type in f:
-       found[(ct, item)] = ( state_type, paramstring ) 
+    for hn, ct, item, paramstring, state_type in found_services:
+       found[(ct, item)] = ( state_type, paramstring )
 
     # Check if already in autochecks (but not found anymore)
     for hn, ct, item, params in autochecks:
@@ -167,34 +233,26 @@ def automation_try_inventory(args):
     for (ct, item), (params, descr, deps) in existing.items():
         if (ct, item) not in found:
             found[(ct, item)] = ('manual', repr(params) )
-        
-    # Add legacy checks with artificial type 'legacy'
+
+    # Add legacy checks and active checks with artificial type 'legacy'
     legchecks = host_extra_conf(hostname, legacy_checks)
     for cmd, descr, perf in legchecks:
         found[('legacy', descr)] = ( 'legacy', 'None' )
 
+    # Similar for 'active_checks', but here we have parameters
+    for acttype, rules in active_checks.items():
+        act_info = active_check_info[acttype]
+        entries = host_extra_conf(hostname, rules)
+        for params in entries:
+            descr = act_info["service_description"](params)
+            found[(acttype, descr)] = ( 'active', repr(params) )
+
+
+    # Collect current status information about all existing checks
     table = []
     for (ct, item), (state_type, paramstring) in found.items():
-        if state_type != 'legacy':
-            descr = service_description(ct, item)
-            infotype = ct.split('.')[0]
-            opt_use_cachefile = True
-            opt_no_tcp = True
-            try:
-                info = get_host_info(hostname, ipaddress, infotype)
-            # Handle cases where agent does not output data
-            except MKAgentError, e:
-                if str(e) == "":
-                    continue
-                else:
-                    raise
-            except MKSNMPError, e:
-                if str(e) == "":
-                    continue
-                else:
-                    raise
-            
-            check_function = check_info[ct][0]
+        params = None
+        if state_type not in [ 'legacy', 'active' ]:
             # apply check_parameters
             try:
                 if type(paramstring) == str:
@@ -203,28 +261,83 @@ def automation_try_inventory(args):
                     params = paramstring
             except:
                 raise MKAutomationError("Invalid check parameter string '%s'" % paramstring)
-            if state_type != 'manual':
-                params = compute_check_parameters(hostname, ct, item, params)
+
+
+            descr = service_description(ct, item)
+            infotype = ct.split('.')[0]
+            opt_use_cachefile = True
+            opt_no_tcp = True
+            opt_dont_submit = True
 
             try:
-                result = check_function(item, params, info)
-            except MKCounterWrapped, e:
-                result = (None, "WAITING - Counter based check, cannot be done offline")
+                exitcode = None
+                perfdata = []
+                info = get_host_info(hostname, ipaddress, infotype)
+            # Handle cases where agent does not output data
+            except MKAgentError, e:
+                exitcode = 3
+                output = "Error getting data from agent"
+                if str(e):
+                    output += ": %s" % e
+                tcp_error = output
+
+            except MKSNMPError, e:
+                exitcode = 3
+                output = "Error getting data from agent for %s via SNMP" % infotype
+                if str(e):
+                    output += ": %s" % e
+                snmp_error = output
+
             except Exception, e:
-                result = (3, "UNKNOWN - invalid output from agent or error in check implementation")
-            if len(result) == 2:
-                result = (result[0], result[1], [])
-            exitcode, output, perfdata = result
+                exitcode = 3
+                output = "Error getting data for %s: %s" % (infotype, e)
+                if check_uses_snmp(ct):
+                    snmp_error = output
+                else:
+                    tcp_error = output
+
+            if exitcode == None:
+                check_function = check_info[ct]["check_function"]
+                if state_type != 'manual':
+                    params = compute_check_parameters(hostname, ct, item, params)
+
+                try:
+                    result = check_function(item, params, info)
+                except MKCounterWrapped, e:
+                    result = (None, "WAITING - Counter based check, cannot be done offline")
+                except Exception, e:
+                    result = (3, "UNKNOWN - invalid output from agent or error in check implementation")
+                if len(result) == 2:
+                    result = (result[0], result[1], [])
+                exitcode, output, perfdata = result
         else:
             descr = item
             exitcode = None
             output = "WAITING - Legacy check, cannot be done offline"
             perfdata = []
-        table.append((state_type, ct, item, paramstring, params, descr, exitcode, output, perfdata))
+
+        if state_type == "active":
+            params = eval(paramstring)
+
+        if state_type in [ "legacy", "active" ]:
+            checkgroup = None
+        else:
+            checkgroup = check_info[ct]["group"]
+        table.append((state_type, ct, checkgroup, item, paramstring, params, descr, exitcode, output, perfdata))
+
+    if not table and (tcp_error or snmp_error):
+        error = ""
+        if snmp_error:
+            error = "Error getting data via SNMP: %s" % snmp_error
+        if tcp_error:
+            if error:
+                error += ", "
+            error += "Error getting data from Check_MK agent: %s" % tcp_error
+        raise MKAutomationError(error)
 
     return table
 
-# Set the new list of autochecks. This list is specified by a 
+# Set the new list of autochecks. This list is specified by a
 # table of (checktype, item). No parameters are specified. Those
 # are either (1) kept from existing autochecks or (2) computed
 # from a new inventory. Note: we must never convert check parameters
@@ -305,7 +418,7 @@ def automation_parse_autochecks_file(hostname):
         lineno += 1
         try:
             line = line.strip()
-            if not line.startswith("("): 
+            if not line.startswith("("):
                 continue
 
             # drop everything after potential '#' (from older versions)
@@ -342,7 +455,7 @@ def automation_delete_host(args):
         "%s/%s.*"  % (tcp_cache_dir, hostname)]:
         os.system("rm -rf '%s'" % path)
 
-def automation_restart():
+def automation_restart(job="restart"):
     # make sure, Nagios does not inherit any open
     # filedescriptors. This really happens, e.g. if
     # check_mk is called by WATO via Apache. Nagios inherits
@@ -366,6 +479,10 @@ def automation_restart():
     sys.stdout = null_file()
 
     try:
+        backup_path = None
+        if not lock_nagios_objects_file():
+            raise MKAutomationError("Cannot activate changes. "
+                  "Another activation process is currently in progresss")
         if os.path.exists(nagios_objects_file):
             backup_path = nagios_objects_file + ".save"
             os.rename(nagios_objects_file, backup_path)
@@ -383,7 +500,10 @@ def automation_restart():
             if backup_path:
                 os.remove(backup_path)
             do_precompile_hostchecks()
-            do_restart_nagios(False)
+            if job == 'restart':
+                do_restart_nagios(False)
+            elif job == 'reload':
+                do_restart_nagios(True)
         else:
             if backup_path:
                 os.rename(backup_path, nagios_objects_file)
@@ -398,4 +518,46 @@ def automation_restart():
 
     sys.stdout = old_stdout
 
+def automation_get_configuration():
+    # We read the list of variable names from stdin since
+    # that could be too much for the command line
+    variable_names = eval(sys.stdin.read())
+    result = {}
+    for varname in variable_names:
+        if varname in globals():
+            result[varname] = globals()[varname]
+    return result
+
+def automation_get_check_information():
+    manuals = all_manuals()
+    checks = {}
+    for check_type, check in check_info.items():
+        manfile = manuals.get(check_type)
+        if manfile:
+            title = file(manfile).readline().strip().split(":", 1)[1].strip()
+        else:
+            title = check_type
+        checks[check_type] = { "title" : title }
+        if check["group"]:
+            checks[check_type]["group"] = check["group"]
+    return checks
+
+def automation_scan_parents(args):
+    settings = {
+        "timeout"     : int(args[0]),
+        "probes"      : int(args[1]),
+        "max_ttl"     : int(args[2]),
+        "ping_probes" : int(args[3]),
+    }
+    hostnames = args[4:]
+    traceroute_prog = find_bin_in_path('traceroute')
+    if not traceroute_prog:
+        raise MKAutomationError("Cannot find binary <tt>traceroute</tt> in search path.")
+
+    try:
+        gateways = scan_parents_of(hostnames, silent=True, settings=settings)
+        return gateways
+    except Exception, e:
+        raise MKAutomationError(str(e))
+
 
