@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -75,7 +75,6 @@ else:
     def tty(fg=-1, bg=-1, attr=-1):
         return ''
 
-
 # global variables used to cache temporary values
 g_dns_cache                  = {}
 g_infocache                  = {} # In-memory cache of host info.
@@ -105,6 +104,11 @@ opt_use_snmp_walk            = False
 opt_cleanup_autochecks       = False
 fake_dns                     = False
 
+# register SIGINT handler for consistenct CTRL+C handling
+def interrupt_handler(signum, frame):
+    sys.stderr.write('<Interrupted>\n')
+    sys.exit(1)
+signal.signal(signal.SIGINT, interrupt_handler)
 
 class MKGeneralException(Exception):
     def __init__(self, reason):
@@ -249,8 +253,15 @@ def submit_check_mk_aggregation(hostname, status, output):
 # and thus do no network activity at all...
 
 def get_host_info(hostname, ipaddress, checkname):
-    nodes = nodes_of(hostname)
 
+    # If the check want's the node info, we add an additional
+    # column (as the first column) with the name of the node
+    # or None (in case of non-clustered nodes). On problem arises,
+    # if we deal with subchecks. We assume that all subchecks
+    # have the same setting here. If not, let's raise an exception.
+    add_nodeinfo = check_info.get(checkname, {}).get("node_info", False)
+
+    nodes = nodes_of(hostname)
     if nodes != None:
         info = []
         at_least_one_without_exception = False
@@ -263,7 +274,10 @@ def get_host_info(hostname, ipaddress, checkname):
             # try the other node.
             try:
                 ipaddress = lookup_ipaddress(node)
-                info += get_realhost_info(node, ipaddress, checkname, cluster_max_cachefile_age)
+                new_info = get_realhost_info(node, ipaddress, checkname, cluster_max_cachefile_age)
+                if add_nodeinfo:
+                    new_info = [ [node] + line for line in new_info ]
+                info += new_info
                 at_least_one_without_exception = True
             except MKAgentError, e:
 		if str(e) != "": # only first error contains text
@@ -281,7 +295,11 @@ def get_host_info(hostname, ipaddress, checkname):
                 raise MKAgentError(", ".join(exception_texts))
         return info
     else:
-        return get_realhost_info(hostname, ipaddress, checkname, check_max_cachefile_age)
+        info = get_realhost_info(hostname, ipaddress, checkname, check_max_cachefile_age)
+        if add_nodeinfo:
+            return [ [ None ] + line for line in info ]
+        else:
+            return info
 
 # Gets info from a real host (not a cluster). There are three possible
 # ways: TCP, SNMP and external command.  This function raises
@@ -295,15 +313,17 @@ def get_host_info(hostname, ipaddress, checkname):
 #
 # This function assumes, that each check type is queried
 # only once for each host.
-def get_realhost_info(hostname, ipaddress, checkname, max_cache_age):
+def get_realhost_info(hostname, ipaddress, check_type, max_cache_age):
     info = get_cached_hostinfo(hostname)
-    if info and info.has_key(checkname):
-        return info[checkname]
+    if info and info.has_key(check_type):
+        return info[check_type]
 
-    cache_relpath = hostname + "." + checkname
+    cache_relpath = hostname + "." + check_type
 
     # Is this an SNMP table check? Then snmp_info specifies the OID to fetch
-    oid_info = snmp_info.get(checkname)
+    # Please note, that if the check_type is foo.bar then we lookup the
+    # snmp info for "foo", not for "foo.bar".
+    oid_info = snmp_info.get(check_type.split(".")[0])
     if oid_info:
         content = read_cache_file(cache_relpath, max_cache_age)
         if content:
@@ -318,14 +338,21 @@ def get_realhost_info(hostname, ipaddress, checkname, max_cache_age):
         # of that list is interpreted as one real oid_info, fetches
         # a separate snmp table. The overall result is then the list
         # of these results.
-        if type(oid_info) == list:
-            table = [ get_snmp_table(hostname, ipaddress, entry) for entry in oid_info ]
-            # if at least one query fails, we discard the hole table
-            if None in table:
-                table = None
-        else:
-            table = get_snmp_table(hostname, ipaddress, oid_info)
-        store_cached_checkinfo(hostname, checkname, table)
+        try:
+            if type(oid_info) == list:
+                table = [ get_snmp_table(hostname, ipaddress, entry) for entry in oid_info ]
+                # if at least one query fails, we discard the hole table
+                if None in table:
+                    table = None
+            else:
+                table = get_snmp_table(hostname, ipaddress, oid_info)
+        except:
+            if opt_debug:
+                raise
+            else:
+                raise MKGeneralException("Incomplete or invalid response from SNMP agent")
+
+        store_cached_checkinfo(hostname, check_type, table)
         write_cache_file(cache_relpath, repr(table) + "\n")
         return table
 
@@ -347,7 +374,7 @@ def get_realhost_info(hostname, ipaddress, checkname, max_cache_age):
     lines = [ l.strip() for l in output.split('\n') ]
     info = parse_info(lines)
     store_cached_hostinfo(hostname, info)
-    return info.get(checkname, []) # return only data for specified check
+    return info.get(check_type, []) # return only data for specified check
 
 
 def read_cache_file(relpath, max_cache_age):
@@ -371,7 +398,8 @@ def read_cache_file(relpath, max_cache_age):
         raise MKGeneralException("Simulation mode and no cachefile present.")
 
     if opt_no_tcp:
-        raise MKGeneralException("Cache file '%s' missing or too old. TCP disallowed by you." % cachefile)
+        raise MKGeneralException("Host is unreachable")
+        #Cache file '%s' missing or too old. TCP disallowed by you." % cachefile)
 
 
 def write_cache_file(relpath, output):
@@ -462,7 +490,7 @@ def get_agent_info_tcp(hostname, ipaddress):
                 break
         s.close()
         if len(output) == 0: # may be caused by xinetd not allowing our address
-            raise MKAgentError("Empty output from agent at TCP port %d" % 
+            raise MKAgentError("Empty output from agent at TCP port %d" %
                   agent_port_of(hostname))
         return output
     except MKAgentError, e:
@@ -582,6 +610,8 @@ def get_counter(countername, this_time, this_val, allow_negative=False):
     last_time, last_val = g_counters.get(countername)
     timedif = this_time - last_time
     if timedif <= 0: # do not update counter
+        # Reset counter to a (hopefully) reasonable value
+        g_counters[countername] = (this_time, this_val)
         # Do not suppress this check on check_mk -nv
         if opt_dont_submit:
             return 1.0, 0.0
@@ -627,13 +657,13 @@ def get_average(itemname, this_time, this_val, backlog, initialize_zero = True):
     # we get one sample per minute. And that backlog is the number
     # of minutes we should average over. Then we want that the weight
     # of the values of the last average minutes have a fraction of W%
-    # in the result and the rest until infinity the rest (1-W%). 
+    # in the result and the rest until infinity the rest (1-W%).
     # Then the weight can be computed as backlog'th root of 1-W
     percentile = 0.50
 
     weight_per_minute = (1 - percentile) ** (1.0 / backlog)
 
-    # now let's compute the weight per second. This is done 
+    # now let's compute the weight per second. This is done
     weight = weight_per_minute ** (timedif / 60.0)
 
     new_val = last_val * weight + this_val * (1 - weight)
@@ -672,7 +702,8 @@ def save_counters(hostname):
 
 # This is the main check function - the central entry point to all and
 # everything
-def do_check(hostname, ipaddress):
+def do_check(hostname, ipaddress, only_check_types = None):
+
     if opt_verbose:
         sys.stderr.write("Check_mk version %s\n" % check_mk_version)
 
@@ -680,7 +711,7 @@ def do_check(hostname, ipaddress):
 
     try:
         load_counters(hostname)
-        agent_version, num_success, error_sections, problems = do_all_checks_on_host(hostname, ipaddress)
+        agent_version, num_success, error_sections, problems = do_all_checks_on_host(hostname, ipaddress, only_check_types)
         num_errors = len(error_sections)
         save_counters(hostname)
         if problems:
@@ -718,16 +749,86 @@ def do_check(hostname, ipaddress):
         close_checkresult_file()
 
     run_time = time.time() - start_time
-    output += "execution time %.1f sec|execution_time=%.3f\n" % (run_time, run_time)
+    if check_mk_perfdata_with_times:
+        times = os.times()
+        output += "execution time %.1f sec|execution_time=%.3f user_time=%.3f "\
+                  "system_time=%.3f children_user_time=%.3f children_system_time=%.3f\n" %\
+                (run_time, run_time, times[0], times[1], times[2], times[3])
+    else:
+        output += "execution time %.1f sec|execution_time=%.3f\n" % (run_time, run_time)
+
     sys.stdout.write(output)
     sys.exit(status)
 
 def check_unimplemented(checkname, params, info):
     return (3, 'UNKNOWN - Check not implemented')
 
+def convert_check_info():
+    for check_type, info in check_info.items():
+        basename = check_type.split(".")[0]
+
+        if type(info) != dict:
+            # Convert check declaration from old style to new API
+            check_function, service_description, has_perfdata, inventory_function = info
+            if inventory_function == no_inventory_possible:
+                inventory_function = None
+
+            check_info[check_type] = {
+                "check_function"          : check_function,
+                "service_description"     : service_description,
+                "has_perfdata"            : not not has_perfdata,
+                "inventory_function"      : inventory_function,
+                # Insert check name as group if no group is being defined
+                "group"                   : checkgroup_of.get(check_type, check_type),
+                "snmp_info"               : snmp_info.get(check_type),
+                # Sometimes the scan function is assigned to the check_type
+                # rather than to the base name.
+                "snmp_scan_function"      :
+                    snmp_scan_functions.get(check_type,
+                        snmp_scan_functions.get(basename)),
+                "default_levels_variable" : check_default_levels.get(check_type),
+                "node_info"               : False,
+            }
+        else:
+            # Check does already use new API. Make sure that all keys are present,
+            # extra check-specific information into file-specific variables.
+            info.setdefault("inventory_function", None)
+            info.setdefault("group", None)
+            info.setdefault("snmp_info", None)
+            info.setdefault("snmp_scan_function", None)
+            info.setdefault("default_levels_variable", None)
+            info.setdefault("node_info", False)
+
+            # Include files are related to the check file (= the basename),
+            # not to the (sub-)check. So we keep them in check_includes.
+            check_includes.setdefault(basename, [])
+            check_includes[basename] += info.get("includes", [])
+
+    # Make sure that setting for node_info of check and subcheck matches
+    for check_type, info in check_info.iteritems():
+        if "." in check_type:
+            base_check = check_type.split(".")[0]
+            if base_check not in check_info:
+                if info["node_info"]:
+                    raise MKGeneralException("Invalid check implementation: node_info for %s is True, but base check %s not defined" %
+                        (check_type, base_check))
+            elif check_info[base_check]["node_info"] != info["node_info"]:
+               raise MKGeneralException("Invalid check implementation: node_info for %s and %s are different." % (
+                   (base_check, check_type)))
+
+    # Now gather snmp_info and snmp_scan_function back to the
+    # original arrays. Note: these information is tied to a "agent section",
+    # not to a check. Several checks may use the same SNMP info and scan function.
+    for check_type, info in check_info.iteritems():
+        basename = check_type.split(".")[0]
+        if info["snmp_info"] and basename not in snmp_info:
+            snmp_info[basename] = info["snmp_info"]
+        if info["snmp_scan_function"] and basename not in snmp_scan_functions:
+            snmp_scan_functions[basename] = info["snmp_scan_function"]
+
 # Loops over all checks for a host, gets the data, calls the check
 # function that examines that data and sends the result to Nagios
-def do_all_checks_on_host(hostname, ipaddress):
+def do_all_checks_on_host(hostname, ipaddress, only_check_types = None):
     global g_aggregated_service_results
     g_aggregated_service_results = {}
     global g_hostname
@@ -738,6 +839,20 @@ def do_all_checks_on_host(hostname, ipaddress):
     problems = []
 
     for checkname, item, params, description, info in check_table:
+        if only_check_types != None and checkname not in only_check_types:
+            continue
+
+        # Skip checks that are not in their check period
+        period = check_period_of(hostname, description)
+        if period and not check_timeperiod(period):
+            if opt_debug:
+                sys.stderr.write("Skipping service %s: currently not in timeperiod %s.\n" %
+                        (description, period))
+            continue
+        elif period and opt_debug:
+            sys.stderr.write("Service %s: timeperiod %s is currently active.\n" %
+                    (description, period))
+
         # In case of a precompiled check table info is the aggrated
         # service name. In the non-precompiled version there are the dependencies
         if type(info) == str:
@@ -765,13 +880,13 @@ def do_all_checks_on_host(hostname, ipaddress):
         if info or info == []:
             num_success += 1
             try:
-                check_funktion = check_info[checkname][0]
+                check_function = check_info[checkname]["check_function"]
             except:
-                check_funktion = check_unimplemented
+                check_function = check_unimplemented
 
             try:
                 dont_submit = False
-                result = check_funktion(item, params, info)
+                result = check_function(item, params, info)
             # handle check implementations that do not yet support the
             # handling of wrapped counters via exception. Do not submit
             # any check result in that case:
@@ -780,7 +895,7 @@ def do_all_checks_on_host(hostname, ipaddress):
                     print "Counter wrapped, not handled by check, ignoring this check result: %s" % e
                 dont_submit = True
             except Exception, e:
-                result = (3, "UNKNOWN - invalid output from agent, invalid check parameters or error in implementation of check %s. Please set <tt>debug_log</tt> to a filename in <tt>main.mk</tt> for enabling exception logging." % checkname)
+                result = (3, "invalid output from agent, invalid check parameters or error in implementation of check %s. Please set <tt>debug_log</tt> to a filename in <tt>main.mk</tt> for enabling exception logging." % checkname)
                 if debug_log:
                     try:
                         import traceback, pprint
@@ -837,7 +952,7 @@ def open_checkresult_file():
             checkresult_file_fd, checkresult_file_path = \
                 tempfile.mkstemp('', 'c', check_result_path)
         except Exception, e:
-            raise MKGeneralException("Cannot create check result file in %s: %s" % 
+            raise MKGeneralException("Cannot create check result file in %s: %s" %
                     (check_result_path, e))
 
 
@@ -888,19 +1003,31 @@ def convert_perf_data(p):
 
 
 def submit_check_result(host, servicedesc, result, sa):
+    if len(result) >= 3:
+        state, infotext, perfdata = result[:3]
+    else:
+        state, infotext = result
+        perfdata = None
+
+    if not (
+        infotext.startswith("OK -") or
+        infotext.startswith("WARN -") or
+        infotext.startswith("CRIT -") or
+        infotext.startswith("UNKNOWN -")):
+        infotext = nagios_state_names[state] + " - " + infotext
+
     global nagios_command_pipe
     # [<timestamp>] PROCESS_SERVICE_CHECK_RESULT;<host_name>;<svc_description>;<return_code>;<plugin_output>
 
     # Aggregated service -> store for later
     if sa != "":
-        store_aggregated_service_result(host, servicedesc, sa, result[0], result[1])
+        store_aggregated_service_result(host, servicedesc, sa, state, infotext)
 
     # performance data - if any - is stored in the third part of the result
     perftexts = [];
     perftext = ""
 
-    if len(result) > 2:
-        perfdata = result[2]
+    if perfdata:
         # Check may append the name of the check command to the
         # list of perfdata. It is of type string. And it might be
         # needed by the graphing tool in order to choose the correct
@@ -920,15 +1047,15 @@ def submit_check_result(host, servicedesc, result, sa):
             perftext = "|" + (" ".join(perftexts))
 
     if not opt_dont_submit:
-        submit_to_nagios(host, servicedesc, result[0], result[1] + perftext)
+        submit_to_nagios(host, servicedesc, state, infotext + perftext)
 
     if opt_verbose:
         if opt_showperfdata:
             p = ' (%s)' % (" ".join(perftexts))
         else:
             p = ''
-        color = { 0: tty_green, 1: tty_yellow, 2: tty_red, 3: tty_magenta }[result[0]]
-        print "%-20s %s%s%-56s%s%s" % (servicedesc, tty_bold, color, result[1], tty_normal, p)
+        color = { 0: tty_green, 1: tty_yellow, 2: tty_red, 3: tty_magenta }[state]
+        print "%-20s %s%s%-56s%s%s" % (servicedesc, tty_bold, color, infotext, tty_normal, p)
 
 
 def submit_to_nagios(host, service, state, output):
@@ -944,7 +1071,7 @@ def submit_to_nagios(host, service, state, output):
         open_checkresult_file()
         if checkresult_file_fd:
             now = time.time()
-            os.write(checkresult_file_fd, 
+            os.write(checkresult_file_fd,
                 """host_name=%s
 service_description=%s
 check_type=1
@@ -1021,12 +1148,13 @@ def get_regex(pattern):
 # Names of texts usually output by checks
 nagios_state_names = ["OK", "WARN", "CRIT", "UNKNOWN"]
 
-# int() function that return 0 for empty strings
+# int() function that return 0 for strings the
+# cannot be converted to a number
 def saveint(i):
-    if i == '':
-        return 0
-    else:
+    try:
         return int(i)
+    except:
+        return 0
 
 def savefloat(f):
     try:
@@ -1036,24 +1164,42 @@ def savefloat(f):
 
 # Takes bytes as integer and returns a string which represents the bytes in a
 # more human readable form scaled to GB/MB/KB
-def get_bytes_human_readable(b, base=1024.0):
+# The unit parameter simply changes the returned string, but does not interfere
+# with any calcluations
+def get_bytes_human_readable(b, base=1024.0, bytefrac=True, unit="B"):
+    base = float(base)
     # Handle negative bytes correctly
     prefix = ''
     if b < 0:
         prefix = '-'
         b *= -1
 
-    if b >= base * base * base:
-        return '%s%.2fGB' % (prefix, b / base / base / base)
-    if b >= base * base:
-        return '%s%.2fMB' % (prefix, b / base / base)
+    if b >= base * base * base * base:
+        return '%s%.2fT%s' % (prefix, b / base / base / base / base, unit)
+    elif b >= base * base * base:
+        return '%s%.2fG%s' % (prefix, b / base / base / base, unit)
+    elif b >= base * base:
+        return '%s%.2fM%s' % (prefix, b / base / base, unit)
     elif b >= base:
-        return '%s%.2fKB' % (prefix, b / base)
+        return '%s%.2fk%s' % (prefix, b / base, unit)
+    elif bytefrac:
+        return '%s%.2f%s' % (prefix, b, unit)
+    else: # Omit byte fractions
+        return '%s%.0f%s' % (prefix, b, unit)
+
+# Similar to get_bytes_human_readable, but optimized for file
+# sizes
+def get_filesize_human_readable(size):
+    if size < 4 * 1024 * 1024:
+        return str(size)
+    elif size < 4 * 1024 * 1024 * 1024:
+        return "%.2fMB" % (float(size) / (1024 * 1024))
     else:
-        return '%s%.2fB' % (prefix, b)
+        return "%.2fGB" % (float(size) / (1024 * 1024 * 1024))
+
 
 def get_nic_speed_human_readable(speed):
-    try: 
+    try:
         speedi = int(speed)
         if speedi == 10000000:
             speed = "10MBit/s"
@@ -1077,4 +1223,44 @@ def get_nic_speed_human_readable(speed):
 def to_celsius(f):
     return round(float(f) - 32.0) * 5.0 / 9.0
 
+# Format time difference seconds into approximated
+# human readable value
+def get_age_human_readable(secs):
+    if secs < 240:
+        return "%d sec" % secs
+    mins = secs / 60
+    if mins < 120:
+        return "%d min" % mins
+    hours, mins = divmod(mins, 60)
+    if hours < 12:
+        return "%d hours, %d min" % (hours, mins)
+    if hours < 48:
+        return "%d hours" % hours
+    days, hours = divmod(hours, 24)
+    if days < 7:
+        return "%d days, %d hours" % (days, hours)
+    return "%d days" % days
+
+# Quote string for use as arguments on the shell
+def quote_shell_string(s):
+    return "'" + s.replace("'", "'\"'\"'") + "'"
+
+
+# Check if a timeperiod is currently active. We have no other way than
+# doing a Livestatus query. This is not really nice, but if you have a better
+# idea, please tell me...
+g_inactive_timerperiods = None
+def check_timeperiod(timeperiod):
+    global g_inactive_timerperiods
+    # Let exceptions happen, they will be handled upstream.
+    if g_inactive_timerperiods == None:
+        import socket
+        s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
+        s.connect(livestatus_unix_socket)
+        # We just get the currently inactive timeperiods. All others
+        # (also non-existing) are considered to be active
+        s.send("GET timeperiods\nColumns:name\nFilter: in = 0\n")
+        s.shutdown(socket.SHUT_WR)
+        g_inactive_timerperiods = s.recv(10000000).splitlines()
+    return timeperiod not in g_inactive_timerperiods
 
