@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -45,11 +45,12 @@ MAX = -1
 # These settings might go into the config module, sometime in future,
 # in order to allow the user to customize this.
 
-header_height   = 40     # Distance from top of the screen to the lower border of the heading
-screen_margin   = 5      # Distance from the left border of the main-frame to the dashboard area
-dashlet_padding = 8      # Margin between outer border of dashlet and its content
-title_height    = 20     # Height of dashlet title-box
-raster          = 32, 32 # Raster the dashlet choords are measured in
+header_height   = 60             # Distance from top of the screen to the lower border of the heading
+screen_margin   = 5              # Distance from the left border of the main-frame to the dashboard area
+dashlet_padding = 21, 5, 5, 0 # Margin (N, E, S, W) between outer border of dashlet and its content
+corner_overlap  = 22
+title_height    = 0             # Height of dashlet title-box
+raster          = 10, 10        # Raster the dashlet choords are measured in
 
 # Load plugins in web/plugins/dashboard and declare permissions,
 # note: these operations produce language-specific results and
@@ -59,7 +60,6 @@ def load_plugins():
     global loaded_with_language
     if loaded_with_language == current_language:
         return
-    loaded_with_language = current_language
 
     # Permissions are currently not being defined. That will be the
     # case as soon as dashboards become editable.
@@ -69,6 +69,11 @@ def load_plugins():
     # just may add custom dashboards by adding to builtin_dashboards.
     load_web_plugins("dashboard", globals())
 
+    # This must be set after plugin loading to make broken plugins raise
+    # exceptions all the time and not only the first time (when the plugins
+    # are loaded).
+    loaded_with_language = current_language
+
     # In future there will be user editable dashboards just like
     # views which will be loaded. Currently we only use the builtin
     # dashboads.
@@ -85,46 +90,58 @@ def page_dashboard():
 
     render_dashboard(name)
 
-def add_filename_to_url(url, filename):
-    if not filename:
+def add_wato_folder_to_url(url, wato_folder):
+    if not wato_folder:
         return url
     elif '/' in url:
-        return url # do not append filename to non-Check_MK-urls
+        return url # do not append wato_folder to non-Check_MK-urls
     elif '?' in url:
-        return url + "&filename=" + htmllib.urlencode(filename)
+        return url + "&wato_folder=" + htmllib.urlencode(wato_folder)
     else:
-        return url + "?filename=" + htmllib.urlencode(filename)
+        return url + "?wato_folder=" + htmllib.urlencode(wato_folder)
 
 
 # Actual rendering function
 def render_dashboard(name):
     board = dashboards[name]
 
-    # The dashboard may be called with "filename" set. In that case
+    # The dashboard may be called with "wato_folder" set. In that case
     # the dashboard is assumed to restrict the shown data to a specific
     # WATO subfolder or file. This could be a configurable feature in
     # future, but currently we assume, that *all* dashboards are filename
     # sensitive.
 
-    filename = html.var("filename")
-    if not filename or filename == "/": # ignore filename in case of root folder
-        filename = None
+    wato_folder = html.var("wato_folder")
 
-    # The title of the dashboard needs to be prefixed with the WATO path, 
+    # When an empty wato_folder attribute is given a user really wants
+    # to see only the hosts contained in the root folder. So don't ignore
+    # the root folder anymore.
+    #if not wato_folder: # ignore wato folder in case of root folder
+    #    wato_folder = None
+
+    # The title of the dashboard needs to be prefixed with the WATO path,
     # in order to make it clear to the user, that he is seeing only partial
     # data.
     title = board["title"]
-    if filename:
-        title = wato.api.get_folder_title(filename) + " - " + title
-    html.header(title)
 
-    html.javascript_file("dashboard")
-    html.write("<div id=dashboard>\n") # Container of all dashlets
+    global header_height
+    if title is None:
+        # If the title is none, hide the header line
+        html.set_render_headfoot(False)
+        header_height = 0
+        title = ''
+
+    elif wato_folder is not None:
+        title = wato.api.get_folder_title(wato_folder) + " - " + title
+
+    html.header(title, javascripts=["dashboard"], stylesheets=["pages", "dashboard", "status", "views"])
+
+    html.write("<div id=dashboard class=\"dashboard_%s\">\n" % name) # Container of all dashlets
 
     refresh_dashlets = [] # Dashlets with automatic refresh, for Javascript
     for nr, dashlet in enumerate(board["dashlets"]):
         # dashlets using the 'urlfunc' method will dynamically compute
-        # an url (using HTML context variables at their wish). 
+        # an url (using HTML context variables at their wish).
         if "urlfunc" in dashlet:
             dashlet["url"] = dashlet["urlfunc"]()
 
@@ -132,11 +149,11 @@ def render_dashboard(name):
         # dashlets using static content (such as an iframe) will not be
         # refreshed by us but need to do that themselves.
         if "url" in dashlet:
-            refresh_dashlets.append([nr, dashlet.get("refresh", 0), 
-              add_filename_to_url(dashlet["url"], filename)])
+            refresh_dashlets.append([nr, dashlet.get("refresh", 0),
+              str(add_wato_folder_to_url(dashlet["url"], wato_folder))])
 
         # Paint the dashlet's HTML code
-        render_dashlet(nr, dashlet, filename)
+        render_dashlet(nr, dashlet, wato_folder)
 
     html.write("</div>\n")
 
@@ -148,28 +165,31 @@ def render_dashboard(name):
 var header_height = %d;
 var screen_margin = %d;
 var title_height = %d;
-var dashlet_padding = %d;
+var dashlet_padding = Array%s;
+var corner_overlap = %d;
 var refresh_dashlets = %r;
 var dashboard_name = '%s';
 set_dashboard_size();
 window.onresize = function () { set_dashboard_size(); }
+window.onload = function () { set_dashboard_size(); }
 dashboard_scheduler(1);
-    """ % (header_height, screen_margin, title_height, dashlet_padding, refresh_dashlets, name))
+    """ % (header_height, screen_margin, title_height, dashlet_padding,
+           corner_overlap, refresh_dashlets, name))
 
-    html.footer()
+    html.body_end() # omit regular footer with status icons, etc.
 
 # Create the HTML code for one dashlet. Each dashlet has an id "dashlet_%d",
 # where %d is its index (in board["dashlets"]). Javascript uses that id
 # for the resizing. Within that div there is an inner div containing the
 # actual dashlet content. The margin between the inner and outer div is
 # used for stylish layout stuff (shadows, etc.)
-def render_dashlet(nr, dashlet, filename):
+def render_dashlet(nr, dashlet, wato_folder):
 
     html.write('<div class=dashlet id="dashlet_%d">' % nr)
     # render shadow
     if dashlet.get("shadow", True):
         for p in [ "nw", "ne", "sw", "se", "n", "s", "w", "e" ]:
-            html.write('<img id="dashadow_%s_%d" class="shadow %s" src="images/dashadow-%s.png">' % 
+            html.write('<img id="dashadow_%s_%d" class="shadow %s" src="images/dashadow-%s.png">' %
                 (p, nr, p, p))
 
     if dashlet.get("title"):
@@ -185,17 +205,24 @@ def render_dashlet(nr, dashlet, filename):
         bg = ""
     html.write('<div class="dashlet_inner%s" id="dashlet_inner_%d">' % (bg, nr))
 
+    # Optional way to render a dynamic iframe URL
+    if "iframefunc" in dashlet:
+        dashlet["iframe"] = dashlet["iframefunc"]()
+
     # The method "view" is a shortcut for "iframe" with a certain url
     if "view" in dashlet:
-        dashlet["iframe"] = "view.py?view_name=%s&display_options=HRSIXL&_body_class=dashlet" % dashlet["view"]
+        dashlet["iframe"] = "view.py?view_name=%s&_display_options=HRSIXL&_body_class=dashlet" % dashlet["view"]
 
     # The content is rendered only if it is fixed. In the
     # other cases the initial (re)-size will paint the content.
     if "content" in dashlet: # fixed content
         html.write(dashlet["content"])
     elif "iframe" in dashlet: # fixed content containing iframe
-        html.write('<iframe allowTransparency="true" frameborder="0" width="100%%" height="100%%" src="%s"></iframe>' % 
-           add_filename_to_url(dashlet["iframe"], filename))
+        # Fix of iPad >:-P
+        html.write('<div style="width: 100%; height: 100%; -webkit-overflow-scrolling:touch; overflow: auto;">')
+        html.write('<iframe allowTransparency="true" frameborder="0" width="100%%" height="100%%" src="%s"></iframe>' %
+           add_wato_folder_to_url(dashlet["iframe"], wato_folder))
+        html.write('</div>')
     html.write("</div></div>\n")
 
 # Here comes the brain stuff: An intelligent liquid layout algorithm.
@@ -303,7 +330,7 @@ def ajax_resize():
                         raise Exception()
                     used_matrix[(x,y)] = True
             # Helper variable for how to grow, both x and y in [-1, 0, 1]
-            grow_by = rel_position.compute_grow_by(size) 
+            grow_by = rel_position.compute_grow_by(size)
 
             positions.append((nr, True, left, top, right, bottom, grow_by))
         except:
@@ -412,25 +439,25 @@ def dashlet_mk_logo():
 
 def dashlet_hoststats():
     table = [
-       ( _("up"), "#0b3",
+       ( _("Up"), "#0b3",
         "searchhost&is_host_scheduled_downtime_depth=0&hst0=on",
         "Stats: state = 0\n" \
         "Stats: scheduled_downtime_depth = 0\n" \
         "StatsAnd: 2\n"),
 
-       ( _("down"), "#f00",
+       ( _("Down"), "#f00",
         "searchhost&is_host_scheduled_downtime_depth=0&hst1=on",
         "Stats: state = 1\n" \
         "Stats: scheduled_downtime_depth = 0\n" \
         "StatsAnd: 2\n"),
 
-       ( _("unreachable"), "#f80",
+       ( _("Unreachable"), "#f80",
         "searchhost&is_host_scheduled_downtime_depth=0&hst2=on",
         "Stats: state = 2\n" \
         "Stats: scheduled_downtime_depth = 0\n" \
         "StatsAnd: 2\n"),
 
-       ( _("in downtime"), "#0af",
+       ( _("In Downtime"), "#0af",
         "searchhost&search=1&is_host_scheduled_downtime_depth=1",
         "Stats: scheduled_downtime_depth > 0\n" \
        )
@@ -447,44 +474,48 @@ def dashlet_servicestats():
         "Stats: scheduled_downtime_depth = 0\n" \
         "Stats: host_scheduled_downtime_depth = 0\n" \
         "Stats: host_state = 0\n" \
-        "StatsAnd: 4\n"),
+        "Stats: host_has_been_checked = 1\n" \
+        "StatsAnd: 5\n"),
 
-       ( _("warning"), "#ff0",
-        "searchsvc&hst0=on&st1=on&is_in_downtime=0",
-        "Stats: state = 1\n" \
-        "Stats: scheduled_downtime_depth = 0\n" \
-        "Stats: host_scheduled_downtime_depth = 0\n" \
-        "Stats: host_state = 0\n" \
-        "StatsAnd: 4\n"),
-
-       ( _("unknown"), "#f80",
-        "searchsvc&hst0=on&st3=on&is_in_downtime=0",
-        "Stats: state = 3\n" \
-        "Stats: scheduled_downtime_depth = 0\n" \
-        "Stats: host_scheduled_downtime_depth = 0\n" \
-        "Stats: host_state = 0\n" \
-        "StatsAnd: 4\n"),
-
-       ( _("critical"), "#f00",
-        "searchsvc&hst0=on&st2=on&is_in_downtime=0",
-        "Stats: state = 2\n" \
-        "Stats: scheduled_downtime_depth = 0\n" \
-        "Stats: host_scheduled_downtime_depth = 0\n" \
-        "Stats: host_state = 0\n" \
-        "StatsAnd: 4\n"),
-
-       ( _("in downtime"), "#0af",
+       ( _("In Downtime"), "#0af",
         "searchsvc&is_in_downtime=1",
         "Stats: scheduled_downtime_depth > 0\n" \
         "Stats: host_scheduled_downtime_depth > 0\n" \
         "StatsOr: 2\n"),
 
-       ( _("on down host"), "#048",
+       ( _("On Down host"), "#048",
         "searchsvc&hst1=on&hst2=on&hstp=on&is_in_downtime=0",
         "Stats: scheduled_downtime_depth = 0\n" \
         "Stats: host_scheduled_downtime_depth = 0\n" \
         "Stats: host_state != 0\n" \
         "StatsAnd: 3\n"),
+
+       ( _("Warning"), "#ff0",
+        "searchsvc&hst0=on&st1=on&is_in_downtime=0",
+        "Stats: state = 1\n" \
+        "Stats: scheduled_downtime_depth = 0\n" \
+        "Stats: host_scheduled_downtime_depth = 0\n" \
+        "Stats: host_state = 0\n" \
+        "Stats: host_has_been_checked = 1\n" \
+        "StatsAnd: 5\n"),
+
+       ( _("Unknown"), "#f80",
+        "searchsvc&hst0=on&st3=on&is_in_downtime=0",
+        "Stats: state = 3\n" \
+        "Stats: scheduled_downtime_depth = 0\n" \
+        "Stats: host_scheduled_downtime_depth = 0\n" \
+        "Stats: host_state = 0\n" \
+        "Stats: host_has_been_checked = 1\n" \
+        "StatsAnd: 5\n"),
+
+       ( _("Critical"), "#f00",
+        "searchsvc&hst0=on&st2=on&is_in_downtime=0",
+        "Stats: state = 2\n" \
+        "Stats: scheduled_downtime_depth = 0\n" \
+        "Stats: host_scheduled_downtime_depth = 0\n" \
+        "Stats: host_state = 0\n" \
+        "Stats: host_has_been_checked = 1\n" \
+        "StatsAnd: 5\n"),
     ]
     filter = "Filter: host_custom_variable_names < _REALNAME\n"
 
@@ -492,12 +523,27 @@ def dashlet_servicestats():
 
 
 def render_statistics(pie_id, what, table, filter):
+    html.write("<div class=stats>")
+    pie_diameter     = 130
+    pie_left_aspect  = 0.5
+    pie_right_aspect = 0.8
+
     # Is the query restricted to a certain WATO-path?
-    filename = html.var("filename")
-    if filename and filename != "/":
+    wato_folder = html.var("wato_folder")
+    if wato_folder:
         # filter += "Filter: host_state = 0"
-        filter += "Filter: host_filename ~ ^%s\n" % filename.replace("\n", "")
-    
+        filter += "Filter: host_filename ~ ^/wato/%s/\n" % wato_folder.replace("\n", "")
+
+    # Is the query restricted to a host contact group?
+    host_contact_group = html.var("host_contact_group")
+    if host_contact_group:
+        filter += "Filter: host_contact_groups >= %s\n" % host_contact_group.replace("\n", "")
+
+    # Is the query restricted to a service contact group?
+    service_contact_group = html.var("service_contact_group")
+    if service_contact_group:
+        filter += "Filter: service_contact_groups >= %s\n" % service_contact_group.replace("\n", "")
+
     query = "GET %s\n" % what
     for entry in table:
         query += entry[3]
@@ -506,75 +552,110 @@ def render_statistics(pie_id, what, table, filter):
     result = html.live.query_summed_stats(query)
     pies = zip(table, result)
     total = sum([x[1] for x in pies])
-    pie_diameter = 136
 
-    html.write('<canvas class=pie width=%d height=%d id=%s_stats style="float: left"></canvas>' % 
+    html.write('<canvas class=pie width=%d height=%d id="%s_stats" style="float: left"></canvas>' %
             (pie_diameter, pie_diameter, pie_id))
-    
-    html.write('<table class="hoststats%s" style="float:left">' % ( 
-        len(pies) > 5 and " narrow" or ""))
-    for (name, color, viewurl, query), count in pies + [ ((_("Total"), "", "all%s" % what, ""), total) ]:
-        url = "view.py?view_name=" + viewurl + "&filled_in=filter&search=1&filename=" \
-              + htmllib.urlencode(html.var("filename", ""))
+    html.write('<img src="images/globe.png" class="globe">')
+
+    html.write('<table class="hoststats%s" style="float:left">' % (
+        len(pies) > 1 and " narrow" or ""))
+    table_entries = pies
+    while len(table_entries) < 6:
+        table_entries = table_entries + [ (("", "#95BBCD", "", ""), "&nbsp;") ]
+    table_entries.append(((_("Total"), "", "all%s" % what, ""), total))
+    for (name, color, viewurl, query), count in table_entries:
+        url = "view.py?view_name=" + viewurl + "&filled_in=filter&search=1&wato_folder=" \
+              + htmllib.urlencode(html.var("wato_folder", ""))
+        if host_contact_group:
+            url += '&opthost_contactgroup=' + host_contact_group
+        if service_contact_group:
+            url += '&optservice_contactgroup=' + service_contact_group
         html.write('<tr><th><a href="%s">%s</a></th>' % (url, name))
-        html.write('<td class=color style="background-color: %s">'
-                   '</td><td><a href="%s">%d</a></td></tr>' % (color, url, count))
+        style = ''
+        if color:
+            style = ' style="background-color: %s"' % color
+        html.write('<td class=color%s>'
+                   '</td><td><a href="%s">%s</a></td></tr>' % (style, url, count))
 
     html.write("</table>")
 
     r = 0.0
     pie_parts = []
     if total > 0:
-        for (name, color, viewurl, q), value in pies:
-            perc = 100.0 * value / total
-            pie_parts.append('chart_pie("%s", %f, %f, %r);' % (pie_id, r, r + perc, color))
-            r += perc
+        # Count number of non-empty classes
+        num_nonzero = 0
+        for info, value in pies:
+            if value > 0:
+                num_nonzero += 1
 
+        # Each non-zero class gets at least a view pixels of visible thickness.
+        # We reserve that space right now. All computations are done in percent
+        # of the radius.
+        separator = 0.02                                    # 3% of radius
+        remaining_separatorspace = num_nonzero * separator  # space for separators
+        remaining_radius = 1 - remaining_separatorspace     # remaining space
+        remaining_part = 1.0 # keep track of remaining part, 1.0 = 100%
+
+        # Loop over classes, begin with most outer sphere. Inner spheres show
+        # worse states and appear larger to the user (which is the reason we
+        # are doing all this stuff in the first place)
+        for (name, color, viewurl, q), value in pies[::1]:
+            if value > 0 and remaining_part > 0: # skip empty classes
+
+                # compute radius of this sphere *including all inner spheres!* The first
+                # sphere always gets a radius of 1.0, of course.
+                radius = remaining_separatorspace + remaining_radius * (remaining_part ** (1/3.0))
+                pie_parts.append('chart_pie("%s", %f, %f, %r, true);' % (pie_id, pie_right_aspect, radius, color))
+                pie_parts.append('chart_pie("%s", %f, %f, %r, false);' % (pie_id, pie_left_aspect, radius, color))
+
+                # compute relative part of this class
+                part = float(value) / total # ranges from 0 to 1
+                remaining_part           -= part
+                remaining_separatorspace -= separator
+
+
+    html.write("</div>")
     html.javascript("""
-function has_canvas_support() {
-    return document.createElement('canvas').getContext;
-}
-
-function chart_pie(pie_id, from, to, color) {
+function chart_pie(pie_id, x_scale, radius, color, right_side) {
     var context = document.getElementById(pie_id + "_stats").getContext('2d');
-    if(!context)
+    if (!context)
         return;
     var pie_x = %(x)f;
     var pie_y = %(y)f;
     var pie_d = %(d)f;
-    context.beginPath();
-    context.moveTo(pie_x, pie_y);
-    context.arc(pie_x, pie_y, pie_d / 2 - 2, rad(from), rad(to), false);
-    context.closePath();
     context.fillStyle = color;
-    context.shadowOffsetX = 5;
-    context.shadowOffsetY = 5;
-    context.shadowBlur = 10;
-    context.strokeStyle = "#ffffff";
+    context.save();
+    context.translate(pie_x, pie_y);
+    context.scale(x_scale, 1);
+    context.beginPath();
+    if(right_side)
+        context.arc(0, 0, (pie_d / 2) * radius, 1.5 * Math.PI, 0.5 * Math.PI, false);
+    else
+        context.arc(0, 0, (pie_d / 2) * radius, 0.5 * Math.PI, 1.5 * Math.PI, false);
+    context.closePath();
     context.fill();
-    context.stroke();
+    context.restore();
     context = null;
 }
 
-// convert percent to angle(rad)
-function rad(g) {
-    return (g * 360 / 100 * Math.PI) / 180;
-}
 
-if(has_canvas_support()) {
+if (has_canvas_support()) {
     %(p)s
 }
 """ % { "x" : pie_diameter / 2, "y": pie_diameter/2, "d" : pie_diameter, 'p': '\n'.join(pie_parts) })
 
 def dashlet_pnpgraph():
-    render_pnpgraph(html.var("site"), html.var("host"), html.var("service"), int(html.var("source", 0)))
+    render_pnpgraph(
+        html.var("site"), html.var("host"), html.var("service"),
+        int(html.var("source", 0)), int(html.var("view", 0)),
+    )
 
-def dashlet_nodata(): 
+def dashlet_nodata():
     html.write("<div class=nograph><div class=msg>")
     html.write(html.var("message", _("No data available.")))
     html.write("</div></div>")
 
-def render_pnpgraph(site, host, service=None, source=0):
+def render_pnpgraph(site, host, service = None, source = 0, view = 0):
     if not host:
         html.message("Invalid URL to this dashlet. Missing <tt>host</tt>")
         return;
@@ -582,15 +663,15 @@ def render_pnpgraph(site, host, service=None, source=0):
         service = "_HOST_"
 
     if not site:
-        base_url = defaults.url_prefix 
+        base_url = defaults.url_prefix
     else:
         base_url = html.site_status[site]["site"]["url_prefix"]
-    base_url += "/pnp4nagios/index.php/"
-    var_part = "?host=%s&srv=%s&view=0&source=%d&theme=multisite&_t=%d" % \
-            (pnp_cleanup(host), pnp_cleanup(service), source, int(time.time()))
+    base_url += "pnp4nagios/index.php/"
+    var_part = "?host=%s&srv=%s&view=0&source=%d&view=%d&theme=multisite&_t=%d" % \
+            (pnp_cleanup(host), pnp_cleanup(service), source, view, int(time.time()))
 
     pnp_url = base_url + "graph" + var_part
     img_url = base_url + "image" + var_part
     html.write('<a href="%s"><img border=0 src="%s"></a>' % (pnp_url, img_url))
 
-load_plugins()
+# load_plugins()
