@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -67,7 +67,7 @@
 #   +----------------------------------------------------------------------+
 
 def paint_icon_image(what, row, tags, custom_vars):
-    if row.get(what + '_icon_image'):
+    if row[what + '_icon_image']:
         return '<img class=icon src="images/icons/%s">' % row[what + '_icon_image']
 
 multisite_icons.append({
@@ -75,6 +75,46 @@ multisite_icons.append({
     'paint':           paint_icon_image,
 })
 
+
+#   +----------------------------------------------------------------------+
+#   |          ____                _              _       _                |
+#   |         |  _ \ ___  ___  ___| |__   ___  __| |_   _| | ___           |
+#   |         | |_) / _ \/ __|/ __| '_ \ / _ \/ _` | | | | |/ _ \          |
+#   |         |  _ <  __/\__ \ (__| | | |  __/ (_| | |_| | |  __/          |
+#   |         |_| \_\___||___/\___|_| |_|\___|\__,_|\__,_|_|\___|          |
+#   |                                                                      |
+#   +----------------------------------------------------------------------+
+
+def paint_reschedule(what, row, tags, custom_vars):
+    # Reschedule button
+    if (row[what + "_active_checks_enabled"] == 1
+        or row[what + '_check_command'].startswith('check_mk-')) \
+       and config.may('action.reschedule'):
+
+        servicedesc = ''
+        wait_svc    = ''
+        icon        = 'icon_reload'
+        txt         = _('Reschedule an immediate check of this %s') % _(what)
+
+        if what == 'service':
+            servicedesc = row['service_description'].replace("\\","\\\\")
+            wait_svc = servicedesc
+
+            # Use Check_MK service for cmk based services
+            if row[what + '_check_command'].startswith('check_mk-'):
+                servicedesc = 'Check_MK'
+                icon        = 'icon_reload_cmk'
+                txt         = _('Reschedule an immediate check of the \'Check_MK\' service')
+
+        return '<a href=\"javascript:void(0);\" ' \
+               'onclick="performAction(this, \'reschedule\', \'%s\', \'%s\', \'%s\', \'%s\');">' \
+               '<img class=icon title="%s" src="images/%s.gif" /></a>' % \
+                (row["site"], row["host_name"], htmllib.urlencode(servicedesc), htmllib.urlencode(wait_svc), txt, icon)
+
+multisite_icons.append({
+    'columns':         [ 'active_checks_enabled' ],
+    'paint':           paint_reschedule,
+})
 #   +----------------------------------------------------------------------+
 #   |       _        _                        _          _                 |
 #   |      / \   ___| | ___ __   _____      _| | ___  __| | __ _  ___      |
@@ -85,7 +125,7 @@ multisite_icons.append({
 #   +----------------------------------------------------------------------+
 
 def paint_ack_image(what, row, tags, custom_vars):
-    if row.get(what + "_acknowledged"):
+    if row[what + "_acknowledged"]:
         return '<img class=icon title="' + _('This problem has been acknowledged') \
              + '" src="images/icon_ack.gif">'
 
@@ -105,7 +145,7 @@ multisite_icons.append({
 
 def paint_realhost_link_image(what, row, tags, custom_vars):
     # Link to detail host if this is a summary host
-    if custom_vars and "_REALNAME" in custom_vars:
+    if "_REALNAME" in custom_vars:
         newrow = row.copy()
         newrow["host_name"] = custom_vars["_REALNAME"]
         return link_to_view("<img class=icon title='" + _("Detailed host infos")
@@ -133,8 +173,13 @@ def pnp_url(row, what, how = 'graph'):
     else:
         svc = pnp_cleanup(row["service_description"])
     site = html.site_status[sitename]["site"]
-    url = site["url_prefix"] + ("pnp4nagios/index.php/%s?host=%s&srv=%s" % \
+    if html.mobile:
+        url = site["url_prefix"] + ("pnp4nagios/index.php?kohana_uri=/mobile/%s/%s/%s" % \
             (how, htmllib.urlencode(host), htmllib.urlencode(svc)))
+    else:
+        url = site["url_prefix"] + ("pnp4nagios/index.php/%s?host=%s&srv=%s" % \
+            (how, htmllib.urlencode(host), htmllib.urlencode(svc)))
+
     if how == 'graph':
         url += "&theme=multisite&baseurl=%scheck_mk/" % \
                         htmllib.urlencode(site["url_prefix"])
@@ -148,12 +193,12 @@ def pnp_icon(row, what):
         url = pnp_url(row, what)
     else:
         url = ""
-    return '<a href="%s" onmouseover="displayHoverMenu(event, get_url_sync(\'%s\'))" ' \
+    return '<a href="%s" onmouseover="displayHoverMenu(event, pnp_hover_contents(\'%s\'))" ' \
            'onmouseout="hoverHide()"><img class=icon src="images/icon_pnp.png"></a>' % \
                                                         (url, pnp_popup_url(row, what))
 
 def paint_pnp_graph(what, row, tags, custom_vars):
-    pnpgraph_present = row.get(what + "_pnpgraph_present")
+    pnpgraph_present = row[what + "_pnpgraph_present"]
     if pnpgraph_present == 1:
         return pnp_icon(row, what)
 
@@ -174,8 +219,8 @@ multisite_icons.append({
 def paint_action(what, row, tags, custom_vars):
     if 'X' in html.display_options:
         # action_url (only, if not a PNP-URL and pnp_graph is working!)
-        action_url       = row.get(what + "_action_url_expanded")
-        pnpgraph_present = row.get(what + "_pnpgraph_present")
+        action_url       = row[what + "_action_url_expanded"]
+        pnpgraph_present = row[what + "_pnpgraph_present"]
         if action_url \
            and not ('/pnp4nagios/' in action_url and pnpgraph_present >= 0):
             return '<a href="%s"><img class=icon ' \
@@ -196,17 +241,25 @@ multisite_icons.append({
 #   |                                                                      |
 #   +----------------------------------------------------------------------+
 
+# Adds the url_prefix of the services site to the notes url configured in this site.
+# It also adds the master_url which will be used to link back to the source site
+# in multi site environments.
 def logwatch_url(sitename, notes_url):
-    i = notes_url.index("/check_mk/logwatch.py")
+    i = notes_url.index("check_mk/logwatch.py")
     site = html.site_status[sitename]["site"]
-    return site["url_prefix"] + notes_url[i:]
+
+    master_url = ''
+    if config.is_multisite():
+        master_url = '&master_url=' + defaults.url_prefix + 'check_mk/'
+
+    return site["url_prefix"] + notes_url[i:] + master_url
 
 def paint_notes(what, row, tags, custom_vars):
     if 'X' in html.display_options:
         # notes_url (only, if not a Check_MK logwatch check pointing to
         # logwatch.py. These is done by a special icon)
-        notes_url = row.get(what + "_notes_url_expanded")
-        check_command = row.get(what + "_check_command")
+        notes_url = row[what + "_notes_url_expanded"]
+        check_command = row[what + "_check_command"]
         if notes_url:
             # unmodified original logwatch link
             # -> translate into more intelligent icon
@@ -236,11 +289,21 @@ multisite_icons.append({
 def paint_downtimes(what, row, tags, custom_vars):
     # Currently we are in a downtime + link to list of downtimes
     # for this host / service
-    if row.get(what + "_scheduled_downtime_depth", 0) > 0:
-        return link_to_view('<img class=icon src="images/icon_downtime.gif">',
-                                                   row, 'downtimes_of_' + what)
+    if row[what + "_scheduled_downtime_depth"] > 0:
+        if what == "host":
+            icon = "hostdowntime"
+        else:
+            icon = "downtime"
+        return link_to_view('<img class=icon title="%s" src="images/icon_%s.png">' %
+        (_("Currently in downtime"), icon), row, 'downtimes_of_' + what)
+    elif what == "service" and row["host_scheduled_downtime_depth"] > 0:
+        return link_to_view('<img class=icon title="%s" src="images/icon_hostdowntime.png">' %
+        _("The host is currently in downtime"), row, 'downtimes_of_host')
+
+
 
 multisite_icons.append({
+    'host_columns':    [ 'scheduled_downtime_depth' ],
     'columns':         [ 'scheduled_downtime_depth' ],
     'paint':           paint_downtimes,
 })
@@ -255,17 +318,18 @@ multisite_icons.append({
 #   +----------------------------------------------------------------------+
 
 def paint_comments(what, row, tags, custom_vars):
-    comments = row.get(what+ "_comments_with_info", [])
+    comments = row[what+ "_comments_with_extra_info"]
     if len(comments) > 0:
         text = ""
-        for id, author, comment in comments:
-            text += "%s: \"%s\" \n" % (author, comment)
+        for c in comments:
+            id, author, comment, ty, timestamp = c
+            text += "%s %s: \"%s\" \n" % (paint_age(timestamp, True, 0, 'abs')[1], author, comment)
         return link_to_view('<img class=icon title=\'%s\' ' \
                             'src="images/icon_comment.gif">' %
                                  text, row, 'comments_of_' + what)
 
 multisite_icons.append({
-    'columns':         [ 'comments_with_info' ],
+    'columns':         [ 'comments_with_extra_info' ],
     'paint':           paint_comments,
 })
 
@@ -280,7 +344,7 @@ multisite_icons.append({
 
 def paint_notifications(what, row, tags, custom_vars):
     # Notifications disabled
-    if not row.get(what + "_notifications_enabled", True):
+    if not row[what + "_notifications_enabled"]:
         return '<img class=icon title="%s" src="images/icon_ndisabled.gif">' % \
                          _('Notifications are disabled for this %s') % what
 
@@ -299,9 +363,12 @@ multisite_icons.append({
 #   +----------------------------------------------------------------------+
 
 def paint_flapping(what, row, tags, custom_vars):
-    if row.get(what + "_is_flapping"):
-        return '<img class=icon title="%s" src="images/icon_flapping.gif">' % \
-                                          _('This %s is flapping') % what
+    if row[what + "_is_flapping"]:
+        if what == "host":
+            title = _("This host is flapping")
+        else:
+            title = _("This service is flapping")
+        return '<img class=icon title="%s" src="images/icon_flapping.gif">' % title
 
 multisite_icons.append({
     'columns':         [ 'is_flapping' ],
@@ -319,7 +386,7 @@ multisite_icons.append({
 
 def paint_active_checks(what, row, tags, custom_vars):
     # Setting of active checks modified by user
-    if "active_checks_enabled" in row.get(what + "_modified_attributes_list", []):
+    if "active_checks_enabled" in row[what + "_modified_attributes_list"]:
         if row[what + "_active_checks_enabled"] == 0:
             return '<img class=icon title="%s" src="images/icon_disabled.gif">' % \
                     _('Active checks have been manually disabled for this %s!') % what
@@ -344,7 +411,7 @@ multisite_icons.append({
 
 def paint_passive_checks(what, row, tags, custom_vars):
     # Passive checks disabled manually?
-    if "passive_checks_enabled" in row.get(what + "_modified_attributes_list", []):
+    if "passive_checks_enabled" in row[what + "_modified_attributes_list"]:
         if row[what + "_accept_passive_checks"] == 0:
             return '<img class=icon title="%s" src="images/icon_npassive.gif">' % \
                     _('Passive checks have been manually disabled for this %s!') % what
@@ -365,7 +432,7 @@ multisite_icons.append({
 #   +----------------------------------------------------------------------+
 
 def paint_notification_periods(what, row, tags, custom_vars):
-    if not row.get(what + "_in_notification_period", True):
+    if not row[what + "_in_notification_period"]:
         return '<img class=icon title="%s" src="images/icon_outofnot.gif">' % \
                                            _('Out of notification period')
 
@@ -384,9 +451,12 @@ multisite_icons.append({
 #   |           |___/ |___/          |___/                                 |
 #   +----------------------------------------------------------------------+
 
+# Link to aggregations of the host/service
+# When precompile on demand is enabled, this icon is displayed for all hosts/services
+# otherwise only for the hosts/services which are part of aggregations.
 def paint_aggregations(what, row, tags, custom_vars):
-    # Link to aggregations
-    if bi.is_part_of_aggregation(what, row["site"], row["host_name"],
+    if config.bi_precompile_on_demand \
+       or bi.is_part_of_aggregation(what, row["site"], row["host_name"],
                                  row.get("service_description")):
          return link_to_view('<img class=icon src="images/icon_aggr.gif" title="%s">' %
                   _('Aggregations containing this %s') % what, row, 'aggr_' + what)
@@ -394,79 +464,3 @@ def paint_aggregations(what, row, tags, custom_vars):
 multisite_icons.append({
     'paint':           paint_aggregations,
 })
-
-#   +----------------------------------------------------------------------+
-#   |                     __        ___  _____ ___                         |
-#   |                     \ \      / / \|_   _/ _ \                        |
-#   |                      \ \ /\ / / _ \ | || | | |                       |
-#   |                       \ V  V / ___ \| || |_| |                       |
-#   |                        \_/\_/_/   \_\_| \___/                        |
-#   |                                                                      |
-#   +----------------------------------------------------------------------+
-
-def wato_link(filename, site, hostname, where):
-    if 'X' in html.display_options:
-        prefix = config.site(site)["url_prefix"] + "check_mk/"
-        url = prefix + "wato.py?filename=%s&host=%s" % (htmllib.urlencode(filename),
-                                                        htmllib.urlencode(hostname))
-        if where == "inventory":
-            url += "&mode=inventory"
-            help = _("Edit services in WATO - the Check_MK Web Administration Tool")
-        else:
-            url += "&mode=edithost"
-            help = _("Open this host in WATO - the Check_MK Web Administration Tool")
-        return '<a href="%s"><img class=icon src="images/icon_wato.gif" ' \
-               'title="%s"></a>' % (url, help)
-    else:
-        return ""
-
-def paint_wato(what, row, tags, custom_vars):
-    if not config.may("use_wato") or not tags:
-        return
-
-    # Link to WATO for hosts
-    if "wato" in tags and what == "host":
-        for tag in tags:
-            if tag.endswith(".mk"):
-                wato_filename = tag
-                return wato_link(wato_filename, row["site"],
-                                 row["host_name"], "edithost")
-
-
-    # Link to WATO for Check_MK Inventory service
-    if what == "service":
-        wato_filename = custom_vars.get("WATO")
-        if wato_filename:
-            return wato_link(wato_filename, row["site"],
-                             row["host_name"], "inventory")
-
-multisite_icons.append({
-    'paint':           paint_wato,
-})
-
-#   +----------------------------------------------------------------------+
-#   |          ____                _              _       _                |
-#   |         |  _ \ ___  ___  ___| |__   ___  __| |_   _| | ___           |
-#   |         | |_) / _ \/ __|/ __| '_ \ / _ \/ _` | | | | |/ _ \          |
-#   |         |  _ <  __/\__ \ (__| | | |  __/ (_| | |_| | |  __/          |
-#   |         |_| \_\___||___/\___|_| |_|\___|\__,_|\__,_|_|\___|          |
-#   |                                                                      |
-#   +----------------------------------------------------------------------+
-
-def paint_reschedule(what, row, tags, custom_vars):
-    # Reschedule button
-    if row.get(what + "_active_checks_enabled") == 1 \
-       and config.may('action.reschedule'):
-        servicedesc = ''
-        if what == 'service':
-            servicedesc = row['service_description']
-        return '<a href=\"javascript:void(0);\" ' \
-               'onclick="performAction(this, \'reschedule\', \'%s\', \'%s\', \'%s\');">' \
-               '<img class=icon title="%s" src="images/icon_reload.gif" /></a>' % \
-                (row["site"], row["host_name"], servicedesc, 
-               (_('Reschedule an immediate check of this %s') % _(what)))
-
-multisite_icons.append({
-    'columns':         [ 'active_checks_enabled' ],
-    'paint':           paint_reschedule,
-})
