@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -22,12 +22,6 @@
 // to the Free Software Foundation, Inc., 51 Franklin St,  Fifth Floor,
 // Boston, MA 02110-1301 USA.
 
-// +------------------------------------------------------------------+
-// | This file has been contributed and is copyrighted by:            |
-// |                                                                  |
-// | Lars Michelsen <lm@mathias-kettner.de>            Copyright 2010 |
-// +------------------------------------------------------------------+
-
 var aSearchResults = [];
 var aSearchContents = '';
 var iCurrent = null;
@@ -45,8 +39,6 @@ function mkSearchAddField(field, targetFrame) {
         oField.onkeydown = function(e) { if (!e) e = window.event; return mkSearchKeyDown(e, oField); }
         oField.onkeyup   = function(e) { if (!e) e = window.event; return mkSearchKeyUp(e, oField);}
         oField.onclick   = function(e) { mkSearchClose(); return true; }
-        // keypress is needed for key-repeation for cursor up/down
-        oField.onkeypress  = function(e) { if (!e) e = window.event; return mkSearchKeyRepeat(e, oField);}
 
         // On doubleclick toggle the list
         oField.ondblclick  = function(e) { if (!e) e = window.event; mkSearchToggle(e, oField); }
@@ -69,12 +61,6 @@ function mkSearchKeyUp(e, oField) {
             e.returnValue = false;
             e.cancelBubble = true;
         break;
-        
-        // Up/Down
-        case 38:
-        case 40:
-            return false;
-        break;
 
         // Other keys
         default:
@@ -82,7 +68,7 @@ function mkSearchKeyUp(e, oField) {
                 e.returnValue = false;
                 e.cancelBubble = true;
                 mkSearchClose();
-            } 
+            }
             else {
                 mkSearch(e, oField);
             }
@@ -122,34 +108,13 @@ function mkSearchFindUrl(aSearchObjects, objType, oField) {
     return mkSearchGetUrl(objType, namepart, '', found);
 }
 
-function mkSearchKeyRepeat(e, oField) {
-    var keyCode = e.which || e.keyCode;
-
-    switch (keyCode) {
-            // Up arrow
-            case 38:
-                if(!mkSearchResultShown()) {
-                    mkSearch(e, oField);
-                }
-                
-                mkSearchMoveElement(-1);
-                return false;
-            break;
-            
-            // Down arrow
-            case 40:
-                if(!mkSearchResultShown()) {
-                    mkSearch(e, oField);
-                }
-                
-                mkSearchMoveElement(1);
-                return false;
-            break;
-    }
+// On key press down event handler
+function mkSearchButton() {
+    var oField = document.getElementById("mk_side_search_field");
+    var ev = { "which" : 0, "keyCode" : 13 }
+    return mkSearchKeyDown(ev, oField);
 }
 
-
-// On key press down event handler
 function mkSearchKeyDown(e, oField) {
     var keyCode = e.which || e.keyCode;
 
@@ -173,18 +138,18 @@ function mkSearchKeyDown(e, oField) {
                     top.frames[mkSearchTargetFrame].location.href = url;
                     mkSearchClose();
                 }
-                
+
                 e.returnValue = false;
                 e.cancelBubble = true;
             break;
-            
+
             // Escape
             case 27:
                 mkSearchClose();
                 e.returnValue = false;
                 e.cancelBubble = true;
             break;
-            
+
             // Tab
             case 9:
                 if(mkSearchResultShown()) {
@@ -192,6 +157,19 @@ function mkSearchKeyDown(e, oField) {
                 }
                 return;
             break;
+
+            // Up/Down arrow (Must not be handled in onkeyup since this does not fire repeated events)
+            case 38:
+            case 40:
+                if(!mkSearchResultShown()) {
+                    mkSearch(e, oField);
+                }
+
+                mkSearchMoveElement(keyCode == 38 ? -1 : 1);
+
+                e.preventDefault();
+                return false;
+            break;
     }
     oldValue = oField.value;
 }
@@ -212,7 +190,7 @@ function mkSearchMoveElement(step) {
 
     if(iCurrent < 0)
         iCurrent = aSearchResults.length-1;
-    
+
     if(iCurrent > aSearchResults.length-1)
         iCurrent = 0;
 
@@ -223,7 +201,7 @@ function mkSearchMoveElement(step) {
 
     var a = 0;
     for(var i = 0; i < oResults.length; i++) {
-        if(oResults[i].nodeName == 'A') {
+        if(oResults[i].tagName == 'A') {
             if(a == iCurrent) {
                 oResults[i].setAttribute('class', 'active');
                 oResults[i].setAttribute('className', 'active');
@@ -263,7 +241,7 @@ function mkSearchClose() {
         oContainer.parentNode.removeChild(oContainer);
         oContainer = null;
     }
-    
+
     aSearchResults = [];
     iCurrent = null;
 }
@@ -296,7 +274,15 @@ function mkSearchGetSearchObjects(objType) {
         return [];
 }
 
+function is_ipaddress(add, prefix) {
+    if (prefix)
+        return add.match(/^[0-9]{1,3}(\.[0-9]{1,3}){0,2}\.?$/);
+    else
+        return add.match(/^([0-9]{1,3}\.){3}[0-9]{1,3}$/);
+}
+
 function mkSearchGetUrl(objType, objName, objSite, numMatches) {
+    objName = objName.replace(/\*/g,"\.\*");
     if (numMatches == null)
         numMatches = 0;
 
@@ -305,8 +291,12 @@ function mkSearchGetUrl(objType, objName, objSite, numMatches) {
             return 'view.py?view_name=host&host=' + objName + '&site=' + objSite;
         else if(numMatches > 1)
             return 'view.py?view_name=hosts&host=' + objName;
+        else if (is_ipaddress(objName, true))
+            return 'view.py?view_name=searchsvc&search=Search&filled_in=filter&host_address_prefix=yes&host_address=' + objName;
+        else if (is_ipaddress(objName, false))
+            return 'view.py?view_name=searchsvc&search=Search&filled_in=filter&host_address_prefix=no&host_address=' + objName;
         else
-            return 'view.py?view_name=searchsvc&search=Search&filled_in=on&service=' + objName;
+            return 'view.py?view_name=searchsvc&search=Search&filled_in=filter&service=' + objName;
     else if(objType == 'hg')
         if(numMatches == 1)
             return 'view.py?view_name=hostgroup&hostgroup=' + objName + '&site=' + objSite;
@@ -347,13 +337,30 @@ function mkSearchAddSearchResults(aSearchObjects, objType, val) {
     var objName, objSite;
     aSearchContents = '';
     var numHits = 0;
-    for(var i = 0; i < aSearchObjects.length; i++) {
+
+    // First check, if all matched items have the same
+    // site. If not, we will display the site name in
+    // brackets after the item
+    var the_only_site = null;
+    var show_site = false;
+    for (var i = 0; i < aSearchObjects.length; i++) {
+        objName  = aSearchObjects[i][1];
+        if (mkSearchMatch(objName, val)) {
+            objSite  = aSearchObjects[i][0];
+            if (the_only_site == null)
+                the_only_site = objSite;
+            else if (the_only_site != objSite) {
+                show_site = true;
+                break;
+            }
+        }
+    }
+
+    for (var i = 0; i < aSearchObjects.length; i++) {
         objSite  = aSearchObjects[i][0];
         objName  = aSearchObjects[i][1];
 
-        // if(objName.match(oMatch)) {
-				// case insensitive search!
-        if(mkSearchMatch(objName, val)) {
+        if (mkSearchMatch(objName, val)) {
             var url = mkSearchGetUrl(objType, objName, objSite, 1);
             var oResult = {
                 'id': 'result_' + objName,
@@ -362,7 +369,7 @@ function mkSearchAddSearchResults(aSearchObjects, objType, val) {
                 'type': objType,
                 'url': url
             };
-            
+
             // limit the number of search hits
             numHits ++;
             if (numHits > aSearchLimit)
@@ -370,10 +377,13 @@ function mkSearchAddSearchResults(aSearchObjects, objType, val) {
 
             // Add id to search result array
             aSearchResults.push(oResult);
-            aSearchContents += '<a id="' + oResult.id + '" class="' + oResult.type 
-                + '" href="' + oResult.url 
-                + '" onclick="mkSearchClose()" target="' + mkSearchTargetFrame 
-                + '">'+ objName + "</a>\n";
+            aSearchContents += '<a id="' + oResult.id + '" class="' + oResult.type
+                + '" href="' + oResult.url
+                + '" onclick="mkSearchClose()" target="' + mkSearchTargetFrame
+                + '">' + objName
+            if (show_site)
+                aSearchContents += " (" + objSite + ")"
+            aSearchContents += "</a>\n";
         }
     }
 }
@@ -383,7 +393,7 @@ function mkSearch(e, oField) {
     if(oField == null) {
         return;
     }
-    
+
     var val = oField.value;
     if (val == oldValue)
         return;
@@ -400,7 +410,7 @@ function mkSearch(e, oField) {
         // alert("No objects to search for");
         return;
     }
-    
+
     mkSearchAddSearchResults(aSearchObjects, objType, val);
 
     if(aSearchContents != '') {
@@ -415,6 +425,6 @@ function mkSearch(e, oField) {
     } else {
         mkSearchClose();
     }
-    
+
     oField = null;
 }
