@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -84,21 +84,19 @@ function registerEdgeListeners(obj) {
             continue;
 
         if (window.addEventListener)
-            edges[i].addEventListener("mousemove", function(e) {
-                                                       stopDragScroll(e);
-                                                       snapinTerminateDrag(e);
-                                                       return false;
-                                                   }, false);
+            edges[i].addEventListener("mousemove", stop_snapin_dragging, false);
         else
-            edges[i].onmousemove = function(e) {
-                                       stopDragScroll(e);
-                                       snapinTerminateDrag(e);
-                                       return false;
-                                   };
+            edges[i].onmousemove = stop_snapin_dragging;
     }
     edges = null;
 }
 
+function stop_snapin_dragging(e) {
+    stopDragScroll(e);
+    snapinTerminateDrag(e);
+    return false;
+}
+
 /************************************************
  * snapin drag/drop code
  *************************************************/
@@ -131,15 +129,15 @@ function snapinStartDrag(event) {
   if (event.stopPropagation)
     event.stopPropagation();
   event.cancelBubble = true;
-  
+
   snapinDragging = target.parentNode;
 
   // Save relative offset of the mouse to the snapin title to prevent flipping on drag start
-  snapinOffset   = [ event.clientY - target.parentNode.offsetTop, 
+  snapinOffset   = [ event.clientY - target.parentNode.offsetTop,
                      event.clientX - target.parentNode.offsetLeft ];
   snapinStartPos = [ event.clientY, event.clientX ];
   snapinScrollTop = document.getElementById('side_content').scrollTop;
-  
+
   // Disable the default events for all the different browsers
   if (event.preventDefault)
     event.preventDefault();
@@ -161,24 +159,17 @@ function snapinDrag(event) {
   // Drag the snapin
   snapinDragging.style.position = 'absolute';
   var newTop = event.clientY  - snapinOffset[0] - snapinScrollTop;
-  /*if (weAreIEF__k) 
+  /*if (weAreIEF__k)
       newTop += document.getElementById('side_content').scrollTop;*/
   snapinDragging.style.top      = newTop + 'px';
   snapinDragging.style.left     = (event.clientX - snapinOffset[1]) + 'px';
-  snapinDragging.style.width    = '175px';
   snapinDragging.style.zIndex   = 200;
 
   // Refresh the drop marker
   removeSnapinDragIndicator();
-  
+
   var line = document.createElement('div');
   line.setAttribute('id', 'snapinDragIndicator');
-  line.style.height          = '3px';
-  line.style.lineHeight      = '1px';
-  line.style.fontSize        = '1px';
-  line.style.width           = '250px';
-  line.style.backgroundColor = '#fff';
-  line.style.margin          = '1px 0px 0px 5px';
   var o = getSnapinTargetPos();
   if (o != null) {
     snapinAddBefore(o.parentNode, o, line);
@@ -227,7 +218,7 @@ function snapinDrop(event, targetpos) {
     event.returnValue = false;
     return false;
   }
-  
+
   var par = snapinDragging.parentNode;
   par.removeChild(snapinDragging);
   snapinAddBefore(par, targetpos, snapinDragging);
@@ -257,7 +248,7 @@ function snapinTerminateDrag() {
 function snapinStopDrag(event) {
   if (!event)
     event = window.event;
-  
+
   removeSnapinDragIndicator();
   snapinDrop(event, getSnapinTargetPos());
   snapinDragging = false;
@@ -276,10 +267,10 @@ function getDivChildNodes(node) {
 function getSnapinList() {
   if (snapinDragging === false)
     return true;
-  
+
   var l = [];
   var childs = getDivChildNodes(snapinDragging.parentNode);
-  for(var i = 0; i < childs.length; i++) { 
+  for(var i = 0; i < childs.length; i++) {
     var child = childs[i];
     // Skip
     // - non snapin objects
@@ -293,15 +284,15 @@ function getSnapinList() {
 
 function getSnapinCoords(obj) {
   var snapinTop = snapinDragging.offsetTop + document.getElementById('side_content').scrollTop;
-  
+
   var bottomOffset = obj.offsetTop + obj.clientHeight - snapinTop;
   if (bottomOffset < 0)
     bottomOffset = -bottomOffset;
-      
+
   var topOffset = obj.offsetTop - snapinTop;
   if (topOffset < 0)
     topOffset = -topOffset;
-      
+
   var offset = topOffset;
   var corner = 0;
   if (bottomOffset < topOffset) {
@@ -320,7 +311,7 @@ function getSnapinTargetPos() {
 
   // Find the nearest snapin to current left/top corner of
   // the currently dragged snapin
-  for(var i = 0; i < childs.length; i++) { 
+  for(var i = 0; i < childs.length; i++) {
     var child = childs[i];
 
     if (!child.id || child.id.substr(0, 7) != 'snapin_' || child.id == snapinDragging.id)
@@ -330,7 +321,7 @@ function getSnapinTargetPos() {
     if (objId === -1) {
       objId = i;
       var coords = getSnapinCoords(child)
-      objCorner = coords[3]; 
+      objCorner = coords[3];
       continue;
     }
 
@@ -377,17 +368,28 @@ function debug(s) {
 // Set the size of the sidebar_content div to fit the whole screen
 // but without scrolling. The height of the header and footer divs need
 // to be treated here.
+var g_just_resizing = 0;
 function setSidebarHeight() {
   var oHeader  = document.getElementById('side_header');
   var oContent = document.getElementById('side_content');
   var oFooter  = document.getElementById('side_footer');
   var height   = pageHeight();
 
+  // Resize sidebar frame on Chrome (and other webkit browsers)
+  if (isWebkit()) {
+      var oldcols = parent.document.body.cols.split(",");
+      var oldwidth = parseInt(oldcols[0]);
+      var width = oHeader.clientWidth;
+      var target_width = oldwidth * 280.0 / width;
+      var newcols = target_width.toString() + ",*";
+      parent.document.body.cols = newcols;
+  }
+
   // Don't handle zero heights
   if (height == 0)
     return;
 
-  oContent.style.height = (height - oHeader.clientHeight - oFooter.clientHeight - 5) + 'px';
+  oContent.style.height = (height - oHeader.clientHeight - oFooter.clientHeight + 4) + 'px';
 
   oFooter = null;
   oContent = null;
@@ -398,12 +400,12 @@ var scrolling = true;
 
 function scrollwindow(speed){
   var c = document.getElementById('side_content');
-  
+
   if (scrolling) {
     c.scrollTop += speed;
     setTimeout("scrollwindow("+speed+")", 10);
   }
-  
+
   c = null;
 }
 
@@ -421,18 +423,19 @@ function startDragScroll(event) {
 
   var target = getTarget(event);
   var button = getButton(event);
-  
+
   // Evtl. auch nur mit Shift Taste: (e.button == 0 && (e["shiftKey"])
   if (dragging === false && button == 'LEFT'
       && target.tagName != 'A'
       && target.tagName != 'INPUT'
+      && target.tagName != 'SELECT'
       && !(target.tagName == 'DIV' && target.className == 'heading')) {
     if (event.preventDefault)
       event.preventDefault();
     if (event.stopPropagation)
       event.stopPropagation();
     event.returnValue = false;
-    
+
     dragging = event;
     startY = event.clientY;
     startScroll = document.getElementById('side_content').scrollTop;
@@ -442,37 +445,37 @@ function startDragScroll(event) {
   return true;
 }
 
-function stopDragScroll(event){ 
+function stopDragScroll(event){
   dragging = false;
 }
 
 function dragScroll(event) {
   if (!event)
     event = window.event;
-  
+
   if (dragging === false)
     return true;
-  
+
   if (event.preventDefault)
     event.preventDefault();
   event.returnValue = false;
-  
+
   if (event.stopPropagation)
     event.stopPropagation();
   event.cancelBubble = true;
-  
+
   var inhalt = document.getElementById('side_content');
   var diff = startY - event.clientY;
-  
+
   inhalt.scrollTop += diff;
 
 	// Opera does not fire onunload event which is used to store the scroll
 	// position. So call the store function manually here.
   if(weAreOpera)
     storeScrollPos();
-  
+
   startY = event.clientY;
-  
+
   dragging = event;
   inhalt = null;
 
@@ -527,15 +530,14 @@ function scrollWheel(event){
 // Sidebar ajax stuff
 //
 
-// TODO: The sidebar cannot longer be embedded. We can use relative
-// links again and do not need to know the base url any longer :-)
-
 // The refresh snapins do reload after a defined amount of time
 refresh_snapins = null;
 // The restart snapins are notified about the restart of the nagios instance(s)
 restart_snapins = null;
 // Contains a timestamp which holds the time of the last nagios restart handling
 sidebar_restart_time = null;
+// Configures the number of seconds to reload all snapins which request it
+sidebar_update_interval = null;
 
 // Removes a snapin from the sidebar without reloading anything
 function removeSnapin(id, code) {
@@ -543,6 +545,15 @@ function removeSnapin(id, code) {
   var myparent = container.parentNode;
   myparent.removeChild(container);
 
+  // remove this snapin from the refresh list, if it is contained
+  for (var i in refresh_snapins) {
+      var name    = refresh_snapins[i][0];
+      if (id == "snapin_" + name) {
+          refresh_snapins.splice(i, 1);
+          break;
+      }
+  }
+
   // reload main frame if it is just displaying the "add snapin" page
   var href = escape(parent.frames[1].location);
   if (href.indexOf("sidebar_add_snapin.py") > -1)
@@ -555,7 +566,14 @@ function removeSnapin(id, code) {
 
 
 function toggle_sidebar_snapin(oH2, url) {
-    var childs = oH2.parentNode.parentNode.childNodes;
+    // oH2 can also be an <a>. In that case it is the minimize
+    // image itself
+
+    var childs;
+    if (oH2.tagName == "A")
+        childs = oH2.parentNode.parentNode.parentNode.childNodes;
+    else
+        childs = oH2.parentNode.parentNode.childNodes;
     for (var i in childs) {
         child = childs[i];
         if (child.tagName == 'DIV' && child.className == 'content')
@@ -565,20 +583,24 @@ function toggle_sidebar_snapin(oH2, url) {
         else if (child.tagName == 'DIV' && child.className == 'foot')
             var oFoot = child;
     }
+    var oImgMini = oHead.childNodes[0].childNodes[0].childNodes[0];
+
     // FIXME: Does oContent really exist?
     var closed = oContent.style.display == "none";
     if (closed) {
-        oContent.style.display = "";
-        oFoot.style.display = "";
+        oContent.style.display = "block";
+        oFoot.style.display = "block";
         oHead.className = "head open";
+        oImgMini.src = "images/button_minisnapin_lo.png";
     }
     else {
         oContent.style.display = "none";
         oFoot.style.display = "none";
         oHead.className = "head closed";
+        oImgMini.src = "images/button_maxisnapin_lo.png";
     }
     /* make this persistent -> save */
-    get_url(url + (closed ? "open" : "closed")); 
+    get_url(url + (closed ? "open" : "closed"));
     oContent = null;
     oHead = null;
     oFoot = null;
@@ -596,45 +618,44 @@ function switch_site(switchvar) {
        everything is affected by the switch */
 }
 
-/*
- * Is called as response handler of the generic nagios process restarted
- * check. If an instance start time is newer than the current stored time
- * the function triggers a reload of all snapins which have been registered
- * as restart snapins.
- */
-function handle_nagios_restarted(_unused, response) {
-    var t = parseFloat(response);
-    if(sidebar_restart_time == t)
-        return;
-    // Some instance has been restarted. Trigger the snapins
-    for (var i in restart_snapins) {
-        var name = restart_snapins[i];
-        get_url("sidebar_snapin.py?name=" + name, updateContents, "snapin_" + name);
-    }
-    sidebar_restart_time = t;
-}
-
 function sidebar_scheduler() {
     var timestamp = Date.parse(new Date()) / 1000;
     var newcontent = "";
+    var to_be_updated = [];
 
-    // One generic request to detect if nagios instance(s) have been restarted
-    // For testing hardcoded interval of 30 seconds
-    if(timestamp % 30 == 0 && sidebar_restart_time !== null)
-        get_url('nagios_restarted.py?since=' + escape(sidebar_restart_time),
-                                                       handle_nagios_restarted);
+    for (var i in refresh_snapins) {
+        var name = refresh_snapins[i][0];
+        if (refresh_snapins[i][1] != '') {
+            // Special handling for snapins like the nagvis maps snapin which request
+            // to be updated from a special URL, use direct update of those snapins
+            // from this url
+            var url = refresh_snapins[i][1];
 
-    for (var i in refresh_snapins) { 
-        var name    = refresh_snapins[i][0];
-        var refresh = refresh_snapins[i][1];
-        var url = "sidebar_snapin.py?name=" + name;
-        if(refresh_snapins[i][2] != '')
-            url = refresh_snapins[i][2];
-
-        if (timestamp % refresh == 0) {
-            get_url(url, updateContents, "snapin_" + name);
+            if (timestamp % sidebar_update_interval == 0) {
+                get_url(url, updateContents, "snapin_" + name);
+            }
+        } else {
+            // Internal update handling, use bulk update
+            to_be_updated.push(name);
         }
     }
+
+    // Are there any snapins to be bulk updates?
+    if(to_be_updated.length > 0) {
+        if (timestamp % sidebar_update_interval == 0) {
+            var url = 'sidebar_snapin.py?names=' + to_be_updated.join(',');
+            if (sidebar_restart_time !== null)
+                url += '&since=' + sidebar_restart_time;
+
+            var ids = [];
+            for (var i = 0, len = to_be_updated.length; i < len; i++) {
+                ids.push('snapin_' + to_be_updated[i]);
+            }
+
+            get_url(url, bulkUpdateContents, ids);
+        }
+    }
+
     // Detect page changes and re-register the mousemove event handler
     // in the content frame. another bad hack ... narf
     if (contentFrameAccessible() && contentLocation != parent.frames[1].document.location) {
@@ -650,39 +671,6 @@ function addBookmark() {
     get_url("add_bookmark.py?title=" + escape(title) + "&href=" + escape(href), updateContents, "snapin_bookmarks");
 }
 
-function toggle_folder(o, folderId) {
-    var par = o.parentNode;
-    var next = null;
-    var one_more = false;
-    var img = null;
-
-    for (var i in par.childNodes) {
-        var child = par.childNodes[i];
-        if (one_more && child.nodeName == "DIV") {
-            next = child;
-            break;
-        }
-        if (child == o) 
-            one_more = true;
-    }
-
-    for (var i in o.childNodes) {
-        var child = o.childNodes[i];
-        if (child.nodeName == "IMG") {
-            img = child;
-            break;
-        }
-    }
-
-    if (next)
-	toggle_tree_state('customlinks', folderId, next);
-
-    child = null;
-    par = null;
-    next = null;
-    img = null;
-}
-
 /************************************************
  * Save/Restore scroll position
  *************************************************/
@@ -701,7 +689,7 @@ function getCookie(cookieName) {
     var cookieStart = document.cookie.indexOf(cookieName + "=");
     if(cookieStart == -1)
         return null;
-    
+
     cookieStart = cookieStart + cookieName.length + 1;
     var cookieEnd = document.cookie.indexOf(";", cookieStart);
     if(cookieEnd == -1)
@@ -721,23 +709,51 @@ function storeScrollPos() {
 }
 
 /* Foldable Tree in snapin */
-function wato_tree_click(filename) {
-    // Get the URL in the main frame. If that is a view or a dashboard,
-    // then reload that view with an updated "filename" variable. If that is
-    // a wato.py page, navigate to the according wato page. In all other
-    // cases, select the view "allhosts" with the according path.
-    var href = parent.frames[1].location + ""; // adding "" converts url from something magic to string
-    if (href.indexOf("/view.py") >= 0) 
-        href = add_html_var(href, "filename", filename);
-    else if (href.indexOf("/dashboard.py") >= 0) 
-        href = add_html_var(href, "filename", filename);
-    else if (href.indexOf("/wato.py") >= 0)
-        href = "wato.py?filename=" + escape(filename);
-    else
-        href = "view.py?view_name=allhosts&filename=" + escape(filename);
+function wato_tree_click(folderpath) {
+    var topic  = document.getElementById('topic').value;
+    var target = document.getElementById('target_' + topic).value;
+
+    if(target.substr(0, 9) == 'dashboard') {
+        dashboard_name = target.substr(10, target.length);
+        href = 'dashboard.py?name=' + escape(dashboard_name);
+    } else {
+        href = 'view.py?view_name=' + escape(target);
+    }
+
+    href += '&wato_folder=' + escape(folderpath);
+
     parent.frames[1].location = href;
 }
 
+function wato_tree_topic_changed(topic_field) {
+    // First toggle the topic dropdown field
+    var topic = topic_field.value;
+
+    // Hide all select fields but the wanted one
+    var select_fields = document.getElementsByTagName('select');
+    for(var i = 0; i < select_fields.length; i++) {
+        if(select_fields[i].id && select_fields[i].id.substr(0, 7) == 'target_') {
+            select_fields[i].selected = '';
+            if(select_fields[i].id == 'target_' + topic) {
+                select_fields[i].style.display = 'inline';
+            } else {
+                select_fields[i].style.display = 'none';
+            }
+        }
+    }
+
+    // Then send the info to python code via ajax call for persistance
+    get_url('ajax_set_foldertree.py?topic=' + escape(topic) + '&target=');
+}
+
+function wato_tree_target_changed(target_field) {
+    var topic = target_field.id.substr(7, target_field.id.length);
+    var target = target_field.value;
+
+    // Send the info to python code via ajax call for persistance
+    get_url('ajax_set_foldertree.py?topic=' + escape(topic) + '&target=' + escape(target));
+}
+
 // adds a variable to a GET url, but tries to remove that
 // variable, if it is already existing in the URL. In order
 // to simplify the thing, we just look at the *end* of the
