@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -167,6 +167,10 @@ class BaseConnection:
         self.socketurl = socketurl
         self.socket = None
         self.timeout = None
+        self.successful_persistence = False
+
+    def successfully_persisted(self):
+        return self.successful_persistence
 
     def add_header(self, header):
         self.add_headers += header + "\n"
@@ -179,15 +183,18 @@ class BaseConnection:
     def connect(self):
         if self.persist and self.socketurl in persistent_connections:
             self.socket = persistent_connections[self.socketurl]
+            self.successful_persistence = True
             return
 
+        self.successful_persistence = False
+
         # Create new socket
         self.socket = None
         url = self.socketurl
         parts = url.split(":")
         if parts[0] == "unix":
             if len(parts) != 2:
-                raise MKLivestatusConfigError("Invalid livestatus unix url: %s. "
+                raise MKLivestatusConfigError("Invalid livestatus unix URL: %s. "
                         "Correct example is 'unix:/var/run/nagios/rw/live'" % url)
             self.socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
             target = parts[1]
@@ -197,21 +204,37 @@ class BaseConnection:
                 host = parts[1]
                 port = int(parts[2])
             except:
-                raise MKLivestatusConfigError("Invalid livestatus tcp url '%s'. "
+                raise MKLivestatusConfigError("Invalid livestatus tcp URL '%s'. "
                         "Correct example is 'tcp:somehost:6557'" % url)
             self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
             target = (host, port)
         else:
-            raise MKLivestatusConfigError("Invalid livestatus url '%s'. "
+            raise MKLivestatusConfigError("Invalid livestatus URL '%s'. "
                     "Must begin with 'tcp:' or 'unix:'" % url)
 
-        try:
-            if self.timeout:
-                self.socket.settimeout(float(self.timeout))
-            self.socket.connect(target)
-        except Exception, e:
-            self.socket = None
-            raise MKLivestatusSocketError("Cannot connect to '%s': %s" % (self.socketurl, e))
+        # If a timeout is set, then we retry after a failure with mild
+        # a binary backoff.
+        if self.timeout:
+            before = time.time()
+            sleep_interval = 0.1
+
+        while True:
+            try:
+                if self.timeout:
+                    self.socket.settimeout(float(sleep_interval))
+                self.socket.connect(target)
+                break
+            except Exception, e:
+                if self.timeout:
+                    time_left = self.timeout - (time.time() - before)
+                    # only try again, if there is substantial time left
+                    if time_left > sleep_interval:
+                        time.sleep(sleep_interval)
+                        sleep_interval *= 1.5
+                        continue
+
+                self.socket = None
+                raise MKLivestatusSocketError("Cannot connect to '%s': %s" % (self.socketurl, e))
 
         if self.persist:
             persistent_connections[self.socketurl] = self.socket
@@ -223,6 +246,8 @@ class BaseConnection:
 
     def receive_data(self, size):
         result = ""
+        # Timeout is only honored when connecting
+        self.socket.settimeout(None)
         while size > 0:
             packet = self.socket.recv(size)
             if len(packet) == 0:
@@ -235,7 +260,8 @@ class BaseConnection:
         self.send_query(query, add_headers)
         return self.recv_response(query, add_headers)
 
-    def send_query(self, query, add_headers = ""):
+    def send_query(self, query, add_headers = "", do_reconnect=True):
+        orig_query = query
         if self.socket == None:
             self.connect()
         if not query.endswith("\n"):
@@ -248,17 +274,30 @@ class BaseConnection:
         query += "\n"
 
         try:
+            # socket.send() will implicitely cast to str(), we need ot
+            # convert to UTF-8 in order to avoid exceptions
+            if type(query) == unicode:
+                query = query.encode("utf-8")
             self.socket.send(query)
         except IOError, e:
             if self.persist:
                 del persistent_connections[self.socketurl]
+                self.successful_persistence = False
             self.socket = None
-            raise MKLivestatusSocketError(str(e))
+
+            if do_reconnect:
+                # Automatically try to reconnect in case of an error, but
+                # only once.
+                self.connect()
+                self.send_query(orig_query, add_headers, False)
+                return
+
+            raise MKLivestatusSocketError("RC1:" + str(e))
 
     # Reads a response from the livestatus socket. If the socket is closed
     # by the livestatus server, we automatically make a reconnect and send
     # the query again (once). This is due to timeouts during keepalive.
-    def recv_response(self, query = None, add_headers = ""):
+    def recv_response(self, query = None, add_headers = "", timeout_at = None):
         try:
             resp = self.receive_data(16)
             code = resp[0:3]
@@ -274,22 +313,30 @@ class BaseConnection:
                     raise MKLivestatusSocketError("Malformed output")
             else:
                 raise MKLivestatusQueryError(code, data.strip())
-        except MKLivestatusSocketClosed:
+
+        # In case of an IO error or the other side having
+        # closed the socket do a reconnect and try again, but
+        # only once
+        except (MKLivestatusSocketClosed, IOError), e:
             self.disconnect()
-            if query:
+            now = time.time()
+            if query and (not timeout_at or timeout_at > now):
+                if timeout_at == None:
+                    timeout_at = now + self.timeout
+                time.sleep(0.1)
                 self.connect()
                 self.send_query(query, add_headers)
-                return self.recv_response() # do not send query again -> danger of infinite loop
+                return self.recv_response(query, add_headers, timeout_at) # do not send query again -> danger of infinite loop
             else:
-                raise
+                raise MKLivestatusSocketError(str(e))
+
+        except Exception, e:
+            raise MKLivestatusSocketError("Unhandled exception: %s" % e)
 
-        except IOError, e:
-            self.socket = None
-            if self.persist:
-                del persistent_connections[self.socketurl]
-            raise MKLivestatusSocketError(str(e))
 
     def do_command(self, command):
+        if self.socket == None:
+            self.connect()
         if not command.endswith("\n"):
             command += "\n"
         try:
@@ -384,9 +431,9 @@ class MultiSiteConnection(Helpers):
             i = 0
             for name, site, connection in self.connections:
                 if name == sitename:
-                    break
+                    del self.connections[i]
+		    return
                 i += 1
-            del self.connections[i]
 
 
         # Status host: A status host helps to prevent trying to connect
@@ -507,6 +554,12 @@ class MultiSiteConnection(Helpers):
     def alive_sites(self):
         return self.connections.keys()
 
+    def successfully_persisted(self):
+        for sitename, site, connection in self.connections:
+            if connection.successfully_persisted():
+                return True
+        return False
+
     def set_auth_user(self, domain, user):
         for sitename, site, connection in self.connections:
             connection.set_auth_user(domain, user)
