@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -45,12 +45,12 @@ except:
 
 # in case of local directories (OMD) use those instead
 package_parts = [ (part, title, ldir and ldir or dir) for part, title, dir, ldir in [
-  ( "checks",        "Checks",                    checks_dir,          local_checks_dir),
+  ( "checks",        "Checks",                    checks_dir,          local_checks_dir ),
+  ( "notifications", "Notification scripts",      notifications_dir,   local_notifications_dir ),
   ( "checkman",      "Checks' man pages",         check_manpages_dir,  local_check_manpages_dir ),
   ( "agents",        "Agents",                    agents_dir,          local_agents_dir ),
   ( "web",           "Multisite extensions",      web_dir,             local_web_dir ),
   ( "pnp-templates", "PNP4Nagios templates",      pnp_templates_dir,   local_pnp_templates_dir ),
-  ( "pnp-rraconf",   "RRA configuration for PNP", pnp_rraconf_dir,     local_pnp_rraconf_dir ),
   ( "doc",           "Documentation files",       doc_dir,             local_doc_dir ),
 ]]
 
