@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -25,9 +25,13 @@
 # Boston, MA 02110-1301 USA.
 
 import wato
-import sidebar
 
 pagehandlers.update({
-    "wato"            : wato.page_handler,
-    "ajax_wato_files" : sidebar.ajax_wato_files,
+    "wato"                  : wato.page_handler,
+    "wato_ajax_replication" : wato.ajax_replication,
+    "wato_ajax_activation"  : wato.ajax_activation,
+    "automation_login"      : wato.page_automation_login,
+    "automation"            : wato.page_automation,
+    "user_profile"          : wato.page_user_profile,
+    "ajax_set_foldertree"   : wato.ajax_set_foldertree,
 })
