@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -33,10 +33,7 @@
 import views, defaults
 
 def render_nagvis_maps():
-    nagvis_base_url = '/nagvis'
-    if hasattr(config, 'nagvis_base_url'):
-        nagvis_base_url = config.nagvis_base_url
-    refresh_url = "%s/server/core/ajax_handler.php?mod=Multisite&act=getMaps" % nagvis_base_url
+    refresh_url = "%snagvis/server/core/ajax_handler.php?mod=Multisite&act=getMaps" % (defaults.url_prefix)
     return refresh_url
 
 sidebar_snapins["nagvis_maps"] = {
@@ -44,9 +41,8 @@ sidebar_snapins["nagvis_maps"] = {
     "description": _("List of available NagVis maps. This only works with NagVis 1.5 and above. " \
                    "At the moment it is neccessarry to authenticate with NagVis first by opening " \
                    "a NagVis map in the browser. After this the maplist should be filled."),
-    "author":      "Lars Michelsen",
     "render":      render_nagvis_maps,
     "allowed":     [ "user", "admin", "guest" ],
-    "refresh":     30,
+    "refresh":     True,
     "styles":      ""
 }
