@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -47,7 +47,8 @@ def render_searchform():
 
     html.write('<div id="mk_side_search" class="content_center" onclick="mkSearchClose();">')
     html.write('<input id="mk_side_search_field" type="text" name="search" autocomplete="off" />')
-    html.write('</div>')
+    html.icon_button("#", _("Search"), "quicksearch", onclick="mkSearchButton();")
+    html.write('</div><div id=mk_side_clear></div>')
     html.write("<script type='text/javascript' src='js/search.js'></script>\n")
     html.write("<script type='text/javascript'>\n")
 
@@ -91,38 +92,50 @@ def render_searchform():
 sidebar_snapins["search"] = {
     "title":       _("Quicksearch"),
     "description": _("Interactive search field for direct access to hosts"),
-    "author":      "Lars Michelsen",
     "render":      render_searchform,
     "restart":     True,
     "allowed":     [ "user", "admin", "guest" ],
     "styles":      """
+
 #mk_side_search {
-    width: %dpx;
+    width: 232px;
     padding: 0;
 }
 
+#mk_side_clear {
+    clear: both;
+}
+
+#mk_side_search img.iconbutton {
+    width: 33px;
+    height: 26px;
+    margin-top: -25px;
+    left: 196px;
+    float:right;
+    z-index:100;
+}
+
 #mk_side_search input {
     margin:  0;
-    padding: 0;
+    padding: 0px 5px;
     font-size: 8pt;
-    /* The padding and border defined below stretch the object by 2px in each
-     * direction. This needs to be substracted from the width to position the
-     * field correctly
-     */
-    width: %dpx;
-    padding: 1px;
-    border: 1px #73767A solid;
-    outline: 1px solid #fff;
+    width: 194px;
+    height: 25px;
+    background-image: url("images/quicksearch_field_bg.png");
+    background-repeat: no-repeat;
+    -moz-border-radius: 0px;
+    border-style: none;
+    float: left;
 }
 
 #mk_search_results {
     position: relative;
+    float:left;
     border: 1px solid white;
-    top: 1px;
     background-color: #DFDFDF;
     color: #000;
-    font-size: 80%%;
-    width:140px;
+    font-size: 80%;
+    width: 223px;
 }
 
 #mk_search_results a {
@@ -131,12 +144,12 @@ sidebar_snapins["search"] = {
     text-decoration: none;
     text-align: left;
     padding-left: 5px;
-    width: 135px;
+    width: 217px;
 }
 
 #mk_search_results a:hover, #mk_search_results a.active {
     background-color: #BFBFBF;
 }
 
-""" % (snapin_width - 4, snapin_width - 8)
+"""
 }
