@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -24,7 +24,7 @@
 # to the Free Software Foundation, Inc., 51 Franklin St,  Fifth Floor,
 # Boston, MA 02110-1301 USA.
 
-import views, time, defaults
+import views, time, defaults, dashboard
 import weblib
 from lib import *
 
@@ -55,37 +55,10 @@ def render_about():
 sidebar_snapins["about"] = {
     "title" : _("About Check_MK"),
     "description" : _("Version information and Links to Documentation, Homepage and Download of Check_MK"),
-    "author" : "Mathias Kettner",
     "render" : render_about,
     "allowed" : [ "admin", "user", "guest" ],
 }
 
-# -----------------------------------------------------------------------
-#      _       _           _       _     _             _   _
-#     / \   __| |_ __ ___ (_)_ __ (_)___| |_ _ __ __ _| |_(_) ___  _ __
-#    / _ \ / _` | '_ ` _ \| | '_ \| / __| __| '__/ _` | __| |/ _ \| '_ \
-#   / ___ \ (_| | | | | | | | | | | \__ \ |_| | | (_| | |_| | (_) | | | |
-#  /_/   \_\__,_|_| |_| |_|_|_| |_|_|___/\__|_|  \__,_|\__|_|\___/|_| |_|
-#
-# -----------------------------------------------------------------------
-def render_admin():
-    html.write('<ul>')
-    if config.may("use_wato"):
-        bulletlink(_("WATO - Configure Hosts and Services"), "wato.py")
-    bulletlink(_("View permissions"), "view_permissions.py")
-    if config.may("edit_permissions"):
-        bulletlink(_("Edit permissions"), "edit_permissions.py")
-    html.write('</ul>')
-
-sidebar_snapins["admin"] = {
-    "title" : _("Administration"),
-    "description" : _("Links to administrations functions, e.g. configuration of permissions"),
-    "author" : "Mathias Kettner",
-    "render" : render_admin,
-    "allowed" : [ "admin" ],
-}
-
-
 # --------------------------------------------------------------
 #   __     ___
 #   \ \   / (_) _____      _____
@@ -96,6 +69,36 @@ sidebar_snapins["admin"] = {
 # --------------------------------------------------------------
 visible_views = [ "allhosts", "searchsvc" ]
 
+def views_by_topic():
+    s = [ (view.get("topic", _("Other")), view["title"], name)
+          for name, view
+          in html.available_views.items()
+          if not view["hidden"] and not view.get("mobile")]
+
+    # Add all the dashboards to the views list
+    s += [ (_('Dashboards'), d['title'] and d['title'] or d_name, d_name)
+           for d_name, d
+           in dashboard.dashboards.items()
+    ]
+
+    s.sort()
+
+    # Enforce a certain order on the topics
+    known_topics = [ _('Dashboards'), _("Hosts"), _("Hostgroups"), _("Services"), _("Servicegroups"),
+                     _("Business Intelligence"), _("Problems"), _("Addons") ]
+
+    result = []
+    for topic in known_topics:
+        result.append((topic, s))
+
+    rest = list(set([ t for (t, _t, _v) in s if t not in known_topics ]))
+    rest.sort()
+    for topic in rest:
+        if topic:
+            result.append((topic, s))
+
+    return result
+
 def render_views():
     def render_topic(topic, s):
         first = True
@@ -106,33 +109,20 @@ def render_views():
                 continue
             if t == topic:
                 if first:
-                    html.begin_foldable_container("views", topic, False, topic)
-                    # html.write("<h3>%s</h3>\n" % topic)
+                    html.begin_foldable_container("views", topic, False, topic, indent=True)
                     first = False
-                    # html.write("<ul>")
-                bulletlink(title, "view.py?view_name=%s" % name)
+                if topic == _('Dashboards'):
+                    bulletlink(title, 'dashboard.py?name=%s' % name)
+                else:
+                    bulletlink(title, "view.py?view_name=%s" % name)
         if not first: # at least one item rendered
             html.end_foldable_container()
-            # html.write("</ul>")
 
-    s = [ (view.get("topic", _("Other")), view["title"], name) for name, view in html.available_views.items() if not view["hidden"] ]
-    s.sort()
-
-    # Enforce a certain order on the topics
-    known_topics = [ "Hosts", "Hostgroups", "Services", "Servicegroups", 
-                     _("Business Intelligence"), _("Problems"), _("Addons") ]
-    for topic in known_topics:
+    for topic, s in views_by_topic():
         render_topic(topic, s)
 
-    rest = list(set([ t for (t, _t, _v) in s if t not in known_topics ]))
-    rest.sort()
-    for topic in rest:
-        if topic:
-            render_topic(topic, s)
-
-
     links = []
-    if config.may("edit_views"):
+    if config.may("general.edit_views"):
         if config.debug:
             links.append((_("EXPORT"), "export_views.py"))
         links.append((_("EDIT"), "edit_views.py"))
@@ -141,7 +131,6 @@ def render_views():
 sidebar_snapins["views"] = {
     "title" : _("Views"),
     "description" : _("Links to all views"),
-    "author" : "Mathias Kettner",
     "render" : render_views,
     "allowed" : [ "user", "admin", "guest" ],
 }
@@ -165,17 +154,15 @@ def render_groups(what):
     name_to_alias = dict(data)
     groups = [(name_to_alias[name].lower(), name_to_alias[name], name) for name in name_to_alias.keys()]
     groups.sort() # sort by Alias in lowercase
-    target = views.get_context_link(html.req.user, "%sgroup" % what)
-    if target:
-        html.write('<ul>')
-        for alias_lower, alias, name in groups:
-            bulletlink(alias, target + "&%sgroup=%s" % (what, htmllib.urlencode(name)))
-        html.write('</ul>')
+    html.write('<ul>')
+    for alias_lower, alias, name in groups:
+        url = "view.py?view_name=%sgroup&%sgroup=%s" % (what, what, htmllib.urlencode(name))
+        bulletlink(alias or name, url)
+    html.write('</ul>')
 
 sidebar_snapins["hostgroups"] = {
     "title" : _("Hostgroups"),
     "description" : _("Directs links to all host groups"),
-    "author" : "Mathias Kettner",
     "render" : lambda: render_groups("host"),
     "restart":     True,
     "allowed" : [ "user", "admin", "guest" ]
@@ -183,7 +170,6 @@ sidebar_snapins["hostgroups"] = {
 sidebar_snapins["servicegroups"] = {
     "title" : _("Servicegroups"),
     "description" : _("Direct links to all service groups"),
-    "author" : "Mathias Kettner",
     "render" : lambda: render_groups("service"),
     "restart":     True,
     "allowed" : [ "user", "admin", "guest" ]
@@ -199,7 +185,7 @@ sidebar_snapins["servicegroups"] = {
 # --------------------------------------------------------------
 def render_hosts(mode):
     html.live.set_prepend_site(True)
-    query = "GET hosts\nColumns: name state worst_service_state\n"
+    query = "GET hosts\nColumns: name state worst_service_state\nLimit: 100"
     view = "host"
 
     if mode == "summary":
@@ -224,7 +210,7 @@ def render_hosts(mode):
         num_columns = 2
 
     views.load_views()
-    target = views.get_context_link(html.req.user, view)
+    target = views.get_context_link(config.user_id, view)
     html.write("<table class=allhosts>\n")
     col = 1
     for site, host, state, worstsvc in hosts:
@@ -261,33 +247,27 @@ snapin_allhosts_styles = """
 sidebar_snapins["hosts"] = {
     "title" : _("All hosts"),
     "description" : _("A summary state of each host with a link to the view showing its services"),
-    "author" : "Mathias Kettner",
     "render" : lambda: render_hosts("hosts"),
     "allowed" : [ "user", "admin", "guest" ],
-    "refresh" : 60,
-    "restart":     True,
+    "refresh" : True,
     "styles" : snapin_allhosts_styles,
 }
 
 sidebar_snapins["summary_hosts"] = {
     "title" : _("Summary hosts"),
     "description" : _("A summary state of all summary hosts (summary hosts hold aggregated service states and are a feature of Check_MK)"),
-    "author" : "Mathias Kettner",
     "render" : lambda: render_hosts("summary"),
     "allowed" : [ "user", "admin", "guest" ],
-    "refresh" : 60,
-    "restart":     True,
+    "refresh" : True,
     "styles" : snapin_allhosts_styles,
 }
 
 sidebar_snapins["problem_hosts"] = {
     "title" : _("Problem hosts"),
     "description" : _("A summary state of all hosts that have problem, with links to problems of those hosts"),
-    "author" : "Mathias Kettner",
     "render" : lambda: render_hosts("problems"),
     "allowed" : [ "user", "admin", "guest" ],
-    "refresh" : 60,
-    "restart":     True,
+    "refresh" : True,
     "styles" : snapin_allhosts_styles,
 }
 
@@ -301,10 +281,16 @@ sidebar_snapins["problem_hosts"] = {
 # --------------------------------------------------------------
 def render_hostmatrix():
     html.live.set_prepend_site(True)
-    query = "GET hosts\nColumns: name state has_been_checked worst_service_state scheduled_downtime_depth\nFilter: custom_variable_names < _REALNAME\n"
+    query = "GET hosts\n" \
+            "Columns: name state has_been_checked worst_service_state scheduled_downtime_depth\n" \
+            "Filter: custom_variable_names < _REALNAME\n" \
+            "Limit: 901\n"
     hosts = html.live.query(query)
     html.live.set_prepend_site(False)
     hosts.sort()
+    if len(hosts) > 900:
+        html.write(_("Sorry, I will not display more than 900 hosts."))
+        return
 
     # Choose smallest square number large enough
     # to show all hosts
@@ -362,17 +348,16 @@ def render_hostmatrix():
 
 sidebar_snapins["hostmatrix"] = {
     "title"       : _("Host Matrix"),
-    "description" : _("A matrix showing s colored square for each host"),
-    "author"      : "Mathias Kettner",
+    "description" : _("A matrix showing a colored square for each host"),
     "render"      : render_hostmatrix,
     "allowed"     : [ "user", "admin", "guest" ],
-    "refresh"     : 10,
+    "refresh"     : True,
     "styles"      : """
 table.hostmatrix { border-spacing: 0;  }
 table.hostmatrix tr { padding: 0; border-spacing: 0; }
-table.hostmatrix a { display: block; width: 100%%; height: 100%%; line-height: 100%%; }
-table.hostmatrix td { border: 1px solid white; padding: 0; border-spacing: 0; }
-"""
+table.hostmatrix a { display: block; width: 100%; height: 100%; line-height: 100%; }
+table.hostmatrix td { border: 1px solid #123a4a; padding: 0; border-spacing: 0; }
+    """
 
 }
 
@@ -393,34 +378,40 @@ def render_sitestatus():
         sitenames = []
         for sitename, site in config.allsites().iteritems():
             sitenames.append((sitename, site['alias']))
-        sitenames = sorted(sitenames, key=lambda k: k[1])
+        sitenames = sorted(sitenames, key=lambda k: k[1], cmp = lambda a,b: cmp(a.lower(), b.lower()))
 
         for sitename, sitealias in sitenames:
             site = config.site(sitename)
-            state = html.site_status[sitename]["state"]
-            if state == "disabled":
-                switch = "on"
-                text = site["alias"]
-                title = _("Site %s is switched off") % site["alias"]
+            if sitename not in html.site_status or "state" not in html.site_status[sitename]:
+                state = "missing"
+                text = _("Missing site")
+                title = _("Site %s does not exist") % sitename
             else:
-                switch = "off"
-		try:
-		    linkview = config.sitestatus_link_view
-		except:
-		    linkview = "sitehosts"
-                text = link(site["alias"], "view.py?view_name=%s&site=%s" % (linkview, sitename))
-                ex = html.site_status[sitename].get("exception")
-                shs = html.site_status[sitename].get("status_host_state")
-
-                if ex:
-                    title = ex
+                state = html.site_status[sitename]["state"]
+                if state == "disabled":
+                    switch = "on"
+                    text = site["alias"]
+                    title = _("Site %s is switched off") % site["alias"]
                 else:
-                    title = "Site %s is online" % site["alias"]
+                    switch = "off"
+                    try:
+                        linkview = config.sitestatus_link_view
+                    except:
+                        linkview = "sitehosts"
+                    text = link(site["alias"], "view.py?view_name=%s&site=%s" % (linkview, sitename))
+                    ex = html.site_status[sitename].get("exception")
+                    shs = html.site_status[sitename].get("status_host_state")
+
+                    if ex:
+                        title = ex
+                    else:
+                        title = "Site %s is online" % site["alias"]
 
             html.write("<tr><td class=left>%s</td>" % text)
             onclick = "switch_site('_site_switch=%s:%s')" % (sitename, switch)
             html.write("<td class=state>")
-            html.write('<a title="%s" href="#" onclick="%s" class=%s>%s</a></td>' % (title, onclick, state, state))
+            html.icon_button("#", _("%s this site") % (state == "disabled" and "enable" or "disable"),
+                             "sitestatus_%s" % state, onclick=onclick)
             html.write("</tr>\n")
         html.write("</table>\n")
 
@@ -428,10 +419,9 @@ def render_sitestatus():
 sidebar_snapins["sitestatus"] = {
   "title" : _("Site status"),
   "description" : _("Connection state of each site and button for enabling and disabling the site connection"),
-  "author" : "Mathias Kettner",
   "render" : render_sitestatus,
   "allowed" : [ "user", "admin" ],
-  "refresh" : 90,
+  "refresh" : True,
   "styles" : """
 table.sitestate {
     width: %dpx;
@@ -446,14 +436,11 @@ table.sitestate td.left {
     text-align: left;
 }
 
-table.sitestate a {
-    font-weight: bold;
-    -moz-border-radius: 4px;
-    margin: 0px;
-    padding: 0px 0px;
-    text-align: center;
-    display: block;
+div.snapin table.sitestate td img.iconbutton {
+    width: 60px;
+    height: 16px;
 }
+
 table.sitestate td.left a {
     text-align: left;
     font-weight: normal;
@@ -464,17 +451,6 @@ table.sitestate td.state {
     font-size: 7pt;
 }
 
-table.sitestate td.state a {
-    border-width: 1px;
-    border-style: solid;
-}
-table.sitestate a.online   { background-color: #3c0; color: #fff; border-color: #0f0; }
-table.sitestate a.disabled { background-color: #666; color: #ccc; border-color: #888; }
-table.sitestate a.dead     { background-color: #c00; color: #f88; border-color: #f44; }
-table.sitestate a.waiting  { background-color: #666; color: #fff; border-color: #ccc; }
-table.sitestate a.down     { background-color: #f00; color: #fff; border-color: #800; }
-table.sitestate a.unreach  { background-color: #f80; color: #fff; border-color: #840; }
-table.sitestate a.unknown  { background-color: #26c; color: #fff; border-color: #44f; }
 """ % snapin_width
 }
 
@@ -529,8 +505,7 @@ def render_tactical_overview():
             (_("Hosts"),    hstdata, 'hostproblems', 'host'),
             (_("Services"), svcdata, 'svcproblems',  'service'),
             ]:
-        html.write("<tr><th>%s</th><th>%s</th><th>%s</th></tr>\n" % \
-                                     (title, _('Problems'), _('Unhandled')))
+        html.write("<tr><th>%s</th><th>%s</th><th>%s</th></tr>\n" % (title, _('Problems'), _('Unhandled')))
         html.write("<tr>")
 
         html.write('<td class=total><a target="main" href="view.py?view_name=all%ss">%d</a></td>' % (what, data[0]))
@@ -548,8 +523,7 @@ def render_tactical_overview():
 sidebar_snapins["tactical_overview"] = {
     "title" : _("Tactical Overview"),
     "description" : _("The total number of hosts and service with and without problems"),
-    "author" : "Mathias Kettner",
-    "refresh" : 10,
+    "refresh" : True,
     "render" : render_tactical_overview,
     "allowed" : [ "user", "admin", "guest" ],
     "styles" : """
@@ -561,10 +535,30 @@ table.tacticaloverview {
     * border-spacing: 5px 2px;
     */
    width: %dpx;
-   margin-top: 0;
+   margin-top: -7px;
+}
+table.tacticaloverview th {
+    font-size: 8pt;
+    line-height: 7pt;
+    text-align: left;
+    color: #123a4a;
+    font-weight: normal;
+    padding: 0;
+    padding-top: 2px;
+    vertical-align: bottom;
+}
+table.tacticaloverview td {
+    width: 33.3%%;
+    text-align: right;
+    /* border: 1px solid #123a4a; */
+    background-color: #6da1b8;
+    padding: 0px;
+    height: 14px;
+    /* box-shadow: 1px 0px 1px #386068; */
+}
+table.tacticaloverview td.prob {
+    box-shadow: 0px 0px 4px #ffd000;
 }
-table.tacticaloverview th { font-size: 7pt; text-align: left; font-weight: normal; padding: 0; padding-top: 2px; }
-table.tacticaloverview td { text-align: right; border: 1px solid #444; padding: 0px; }
 table.tacticaloverview a { display: block; margin-right: 2px; }
 """ % snapin_width
 }
@@ -579,49 +573,207 @@ table.tacticaloverview a { display: block; margin-right: 2px; }
 #
 # --------------------------------------------------------------
 def render_performance():
-    data = html.live.query("GET status\nColumns: service_checks_rate host_checks_rate external_commands_rate connections_rate forks_rate log_messages_rate cached_log_messages\n")
+    def write_line(left, right):
+        html.write("<tr><td class=left>%s</td>"
+                   "<td class=right><strong>%s</strong></td></tr>" % (left, right))
+
     html.write("<table class=\"content_center performance\">\n")
+
+    data = html.live.query("GET status\nColumns: service_checks_rate host_checks_rate "
+                           "external_commands_rate connections_rate forks_rate "
+                           "livechecks_rate livecheck_overflows_rate "
+                           "log_messages_rate cached_log_messages\n")
     for what, col, format in \
-        [("Service checks", 0, "%.2f/s"),
-        ("Host checks", 1, "%.2f/s"),
-        ("External commands", 2, "%.2f/s"),
-        ("Livestatus-connections", 3, "%.2f/s"),
-        ("Process creations", 4, "%.2f/s"),
-        ("New log messages", 5, "%.2f/s"),
-        ("Cached log messages", 6, "%d")]:
-       html.write(("<tr><td class=left>%s:</td><td class=right><strong>" + format + "</strong></td></tr>\n") % (what, sum([row[col] for row in data])))
-    data = html.live.query("GET status\nColumns: external_command_buffer_slots external_command_buffer_max\n")
-    size = sum([row[0] for row in data])
-    maxx = sum([row[1] for row in data])
-    html.write("<tr><td class=left>%s</td>"
-               "<td class=right><strong>%d / %d</strong></td></tr>" % \
-                                        (_('Com. buf. max/total'), maxx, size))
+        [("Service checks",        0, "%.2f/s"),
+        ("Host checks",            1, "%.2f/s"),
+        ("External commands",      2, "%.2f/s"),
+        ("Livestatus-conn.",       3, "%.2f/s"),
+        ("Process creations",      4, "%.2f/s"),
+        ("Livechecks",             5, "%.2f/s"),
+        ("Livecheck overflows",    6, "%.2f/s"),
+        ("New log messages",       7, "%.2f/s"),
+        ("Cached log messages",    8, "%d")]:
+        write_line(what + ":", format % sum([row[col] for row in data]))
+
+    if len(config.allsites()) == 1:
+        data = html.live.query("GET status\nColumns: external_command_buffer_slots "
+                               "external_command_buffer_max\n")
+        size = sum([row[0] for row in data])
+        maxx = sum([row[1] for row in data])
+        write_line(_('Com. buf. max/total'), "%d / %d" % (maxx, size))
+
+
     html.write("</table>\n")
 
 sidebar_snapins["performance"] = {
     "title" : _("Server performance"),
     "description" : _("Live monitor of the overall performance of all monitoring servers"),
-    "author" : "Mathias Kettner",
-    "refresh" : 10,
+    "refresh" : True,
     "render" : render_performance,
     "allowed" : [ "admin", ],
     "styles" : """
 table.performance {
     width: %dpx;
     -moz-border-radius: 5px;
-    font-size: 8pt;
     background-color: #589;
+    /* background-color: #6da1b8;*/
     border-style: solid;
     border-color: #444 #bbb #eee #666;
     /* The border needs to be substracted from the width */
     border-width: 1px;
 }
-table.performance td { padding: 0px; }
-table.Performance td.right { text-align: right; padding: 0px; }
+table.performance td {
+    padding: 0px 2px;
+    font-size: 8pt;
+}
+table.performance td.right {
+    text-align: right;
+    padding: 0px;
+    padding-right: 1px;
+    white-space: nowrap;
+}
 
 """ % (snapin_width - 2)
 }
 
+#.
+#   .----------------------------------------------------------------------.
+#   |    ____                      _                      _                |
+#   |   / ___| _ __   ___  ___  __| | ___  _ __ ___   ___| |_ ___ _ __     |
+#   |   \___ \| '_ \ / _ \/ _ \/ _` |/ _ \| '_ ` _ \ / _ \ __/ _ \ '__|    |
+#   |    ___) | |_) |  __/  __/ (_| | (_) | | | | | |  __/ ||  __/ |       |
+#   |   |____/| .__/ \___|\___|\__,_|\___/|_| |_| |_|\___|\__\___|_|       |
+#   |         |_|                                                          |
+#   '----------------------------------------------------------------------'
+def render_speedometer():
+    html.write("<div class=speedometer>");
+    html.write('<img id=speedometerbg src="images/speedometer.png">')
+    html.write('<canvas width=228 height=136 id=speedometer></canvas>')
+    html.write("</div>")
+
+    html.javascript("""
+function show_speed(percentage) {
+    var canvas = document.getElementById('speedometer');
+    if (!canvas)
+        return;
+
+    var context = canvas.getContext('2d');
+    if (!context)
+        return;
+
+    if (percentage > 100.0)
+        percentage = 100.0;
+
+    var orig_x = 116;
+    var orig_y = 181;
+    var angle_0   = 232.0;
+    var angle_100 = 307.0;
+    var angle = angle_0 + (angle_100 - angle_0) * percentage / 100.0;
+    var angle_rad = angle / 360.0 * Math.PI * 2;
+    var length = 120;
+    var end_x = orig_x + (Math.cos(angle_rad) * length);
+    var end_y = orig_y + (Math.sin(angle_rad) * length);
+
+    context.clearRect(0, 0, 228, 136);
+    context.beginPath();
+    context.moveTo(orig_x, orig_y);
+    context.lineTo(end_x, end_y);
+    context.closePath();
+    context.shadowOffsetX = 2;
+    context.shadowOffsetY = 2;
+    context.shadowBlur = 2;
+    context.stroStyle = "#000000";
+    context.stroke();
+    context = null;
+}
+
+function speedometer_show_speed(last_perc, program_start, scheduled_rate)
+{
+    try {
+        text = get_url_sync("sidebar_ajax_speedometer.py" +
+                            "?last_perc=" + last_perc +
+                            "&scheduled_rate=" + scheduled_rate +
+                            "&program_start=" + program_start);
+        code = eval(text);
+        scheduled_rate = code[0];
+        program_start    = code[1];
+        percentage       = code[2];
+        last_perc        = code[3];
+        title            = code[4];
+
+        oDiv = document.getElementById('speedometer');
+
+        // Terminate reschedule when the speedometer div does not exist anymore
+        // (e.g. the snapin has been removed)
+        if (!oDiv)
+            return;
+
+        oDiv.title = title
+        oDiv = document.getElementById('speedometerbg');
+        oDiv.title = title
+        oDiv = null;
+
+        move_needle(last_perc, percentage); // 50 * 100ms = 5s = refresh time
+    } catch(ie) {
+        // Ignore errors during re-rendering. Proceed with reschedule...
+    }
+
+    // large timeout for fetching new data via Livestatus
+    setTimeout("speedometer_show_speed("
+        + percentage       + ","
+        + program_start    + ","
+        + scheduled_rate + ");", 5000);
+}
+
+var needle_timeout = null;
+
+function move_needle(from_perc, to_perc)
+{
+    new_perc = from_perc * 0.9 + to_perc * 0.1;
+    show_speed(new_perc);
+    if (needle_timeout != null)
+        clearTimeout(needle_timeout);
+    needle_timeout = setTimeout("move_needle(" + new_perc + "," +  to_perc + ");", 50);
+}
+
+
+speedometer_show_speed(0, 0, 0);
+
+""")
+
+
+
+sidebar_snapins["speedometer"] = {
+    "title" : _("Speed-O-Meter"),
+    "description" : _("A gadget that shows your current check rate in relation to "
+                      "the scheduled check rate. If the Speed-O-Meter shows a speed "
+                      "of 100 percent, then all checks are being executed in exactly "
+                      "the rate that is configured (via check_interval)"),
+    "render" : render_speedometer,
+    "allowed" : [ "admin", ],
+    "styles" : """
+div.speedometer {
+    position: relative;
+    top: 0px;
+    left: 0px;
+    height: 223px;
+}
+img#speedometerbg {
+    position: absolute;
+    top: 0px;
+    left: 0px;
+}
+canvas#speedometer {
+    position: absolute;
+    top: 0px;
+    left: 0px;
+}
+"""}
+
+
+
+
+#.
 # --------------------------------------------------------------
 #    ____                           _   _
 #   / ___|  ___ _ ____   _____ _ __| |_(_)_ __ ___   ___
@@ -637,8 +789,7 @@ def render_current_time():
 sidebar_snapins["time"] = {
     "title" : _("Server time"),
     "description" : _("A large clock showing the current time of the web server"),
-    "author" : "Mathias Kettner",
-    "refresh" : 30,
+    "refresh" : True,
     "render" : render_current_time,
     "allowed" : [ "user", "admin", "guest", ],
     "styles" : """
@@ -718,9 +869,8 @@ def render_nagios():
                 nagioscgilink(text, ref)
 
 sidebar_snapins["nagios_legacy"] = {
-    "title" : _("Nagios"),
+    "title" : _("Old Nagios GUI"),
     "description" : _("The classical sidebar of Nagios 3.2.0 with links to your local Nagios instance (no multi site support)"),
-    "author" : "Mathias Kettner",
     "render" : render_nagios,
     "allowed" : [ "user", "admin", "guest", ],
 }
@@ -745,101 +895,59 @@ def render_master_control():
     html.live.set_prepend_site(True)
     data = html.live.query("GET status\nColumns: %s" % " ".join([ i[0] for i in items ]))
     html.live.set_prepend_site(False)
-    html.write("<table class=master_control>\n")
     for siteline in data:
         siteid = siteline[0]
         if siteid:
             sitealias = html.site_status[siteid]["site"]["alias"]
-            html.write("<tr><td class=left colspan=2>")
-            heading(sitealias)
-            html.write("</tr>\n")
+            html.begin_foldable_container("master_control", siteid, True, sitealias)
+        html.write("<table class=master_control>\n")
         for i, (colname, title) in enumerate(items):
             colvalue = siteline[i + 1]
             url = defaults.url_prefix + ("check_mk/switch_master_state.py?site=%s&switch=%s&state=%d" % (siteid, colname, 1 - colvalue))
             onclick = "get_url('%s', updateContents, 'snapin_master_control')" % url
-            enabled = colvalue and "enabled" or "disabled"
-            html.write("<tr><td class=left>%s</td><td class=%s><a onclick=\"%s\" href=\"#\">%s</a></td></tr>\n" % (title, enabled, onclick, enabled))
-    html.write("</table>")
+            html.write("<tr><td class=left>%s</td><td>" % title)
+            html.icon_button("#", _("Switch %s %s") % (title, colvalue and "off" or "on"),
+                             "snapin_switch_" + (colvalue and "on" or "off"), onclick=onclick)
+            html.write("</td></tr>")
+            # html.write("<a onclick=\"%s\" href=\"#\">%s</a></td></tr>\n" % (title, enabled, onclick, enabled))
+        html.write("</table>")
+        if siteid:
+            html.end_foldable_container()
 
 sidebar_snapins["master_control"] = {
     "title" : _("Master control"),
     "description" : _("Buttons for switching globally states such as enabling checks and notifications"),
-    "author" : "Mathias Kettner",
     "render" : render_master_control,
     "allowed" : [ "admin", ],
     "styles" : """
-div#check_mk_sidebar table.master_control {
-    width: %dpx;
-    margin: 0px;
+div.snapin table.master_control {
+    width: 100%;
+    margin: 0px 0px 0px 0px;
     border-spacing: 0px;
 }
 
-div#check_mk_sidebar table.master_control td {
+div.snapin table.master_control td {
     padding: 0px 0px;
     text-align: right;
 }
 
-div#check_mk_sidebar table.master_control td a {
-    font-weight: bold;
-    -moz-border-radius: 4px;
-    margin: 0px;
-    padding: 0px 3px;
-    text-align: center;
-    font-size: 7pt;
-    margin-right: 0px;
-    display: block;
-    border: 1px solid black;
-}
-div#check_mk_sidebar table.master_control td.left a {
+div.snapin table.master_control td.left a {
     text-align: left;
     font-size: 8pt;
     font-weight: normal;
 }
 
-div#check_mk_sidebar table.master_control td.left {
+div.snapin table.master_control td.left {
     text-align: left;
 }
 
-div#check_mk_sidebar table.master_control td.enabled a {
-    background-color: #4f6;
-    color: #000;
-    border-color: #080;
-}
-div#check_mk_sidebar table.master_control td.disabled a {
-    background-color: #f33;
-    border-color: #c00;
-    color: #fff;
-}
-""" % snapin_width
+div.snapin table.master_control td img.iconbutton {
+    width: 60px;
+    height: 16px;
 }
 
-def ajax_switch_masterstate():
-    site = html.var("site")
-    column = html.var("switch")
-    state = int(html.var("state"))
-    commands = {
-        ( "enable_notifications",     1) : "ENABLE_NOTIFICATIONS",
-        ( "enable_notifications",     0) : "DISABLE_NOTIFICATIONS",
-        ( "execute_service_checks",   1) : "START_EXECUTING_SVC_CHECKS",
-        ( "execute_service_checks",   0) : "STOP_EXECUTING_SVC_CHECKS",
-        ( "execute_host_checks",      1) : "START_EXECUTING_HOST_CHECKS",
-        ( "execute_host_checks",      0) : "STOP_EXECUTING_HOST_CHECKS",
-        ( "process_performance_data", 1) : "ENABLE_PERFORMANCE_DATA",
-        ( "process_performance_data", 0) : "DISABLE_PERFORMANCE_DATA",
-        ( "enable_event_handlers",    1) : "ENABLE_EVENT_HANDLERS",
-        ( "enable_event_handlers",    0) : "DISABLE_EVENT_HANDLERS",
-    }
-
-    command = commands.get((column, state))
-    if command:
-        html.live.command("[%d] %s" % (int(time.time()), command), site)
-        html.live.set_only_sites([site])
-        html.live.query("GET status\nWaitTrigger: program\nWaitTimeout: 10000\nWaitCondition: %s = %d\nColumns: %s\n" % \
-               (column, state, column))
-        html.live.set_only_sites()
-        render_master_control()
-    else:
-        html.write(_("Command %s/%d not found") % (column, state))
+"""
+}
 
 # ---------------------------------------------------------
 #   ____              _                         _
@@ -863,86 +971,32 @@ def render_bookmarks():
     bookmarks = load_bookmarks()
     n = 0
     for title, href in bookmarks:
-        html.write("<div id=\"bookmark_%d\">" % n)
-        iconbutton(_("delete"), "del_bookmark.py?num=%d" % n, "side", "updateContents", 'snapin_bookmarks')
-        iconbutton(_("edit"), "edit_bookmark.py?num=%d" % n, "main")
+        html.write("<div class=bookmark id=\"bookmark_%d\">" % n)
+        iconbutton(_("delete"), "del_bookmark.py?num=%d" % n, "side", "updateContents", 'snapin_bookmarks', css_class = 'bookmark')
+        iconbutton(_("edit"), "edit_bookmark.py?num=%d" % n, "main", css_class = 'bookmark')
         html.write(link(title, href))
         html.write("</div>")
         n += 1
 
     html.write("<div class=footnotelink><a href=\"#\" onclick=\"addBookmark()\">%s</a></div>\n" % _('Add Bookmark'))
 
-def page_edit_bookmark():
-    html.header(_("Edit Bookmark"))
-    n = int(html.var("num"))
-    bookmarks = load_bookmarks()
-    if n >= len(bookmarks):
-        raise MKGeneralException(_("Unknown bookmark id: %d. This is probably a problem with reload or browser history. Please try again.") % n)
-
-    if html.var("save") and html.check_transaction():
-        title = html.var("title")
-        url = html.var("url")
-        bookmarks[n] = (title, url)
-        save_bookmarks(bookmarks)
-        html.reload_sidebar()
-
-    html.begin_form("edit_bookmark")
-    if html.var("save"):
-        title = html.var("title")
-        url = html.var("url")
-        bookmarks[n] = (title, url)
-        save_bookmarks(bookmarks)
-        html.reload_sidebar()
-    else:
-        title, url = bookmarks[n]
-        html.set_var("title", title)
-        html.set_var("url", url)
-
-    html.write("<table class=edit_bookmarks>")
-    html.write("<tr><td>%s</td><td>" % _('Title:'))
-    html.text_input("title", size = 50)
-    html.write("</td></tr><tr><td>%s:</td><td>" % _('URL:'))
-    html.text_input("url", size = 50)
-    html.write("</td></tr><tr><td></td><td>")
-    html.button("save", _("Save"))
-    html.write("</td></tr></table>\n")
-    html.hidden_field("num", str(n))
-    html.end_form()
-
-    html.footer()
-
-def ajax_del_bookmark():
-    num = int(html.var("num"))
-    bookmarks = load_bookmarks()
-    del bookmarks[num]
-    save_bookmarks(bookmarks)
-    render_bookmarks()
-
-def ajax_add_bookmark():
-    title = html.var("title")
-    href = html.var("href")
-    if title and href:
-        bookmarks = load_bookmarks()
-        # We try to remove http://hostname/some/path/check_mk from the
-        # URI. That keeps the configuration files (bookmarks) portable.
-        # Problem here: We have not access to our own URL, only to the
-        # path part. The trick: we use the Referrer-field from our
-        # request. That points to the sidebar.
-        referer = html.req.headers_in.get("Referer")
-        if referer:
-            while '/' in referer and referer.split('/')[0] == href.split('/')[0]:
-                referer = referer.split('/', 1)[1]
-                href = href.split('/', 1)[1]
-        bookmarks.append((title, href))
-        save_bookmarks(bookmarks)
-    render_bookmarks()
 
 sidebar_snapins["bookmarks"] = {
     "title" : _("Bookmarks"),
     "description" : _("A simple and yet practical snapin allowing to create bookmarks to views and other content in the main frame"),
-    "author" : "Mathias Kettner",
     "render" : render_bookmarks,
     "allowed": [ "user", "admin", "guest" ],
+    "styles" : """
+div.bookmark {
+    width: 230px;
+    max-width: 230px;
+    overflow: hidden;
+    text-overflow: ellipsis;
+    -o-text-overflow: ellipsis;
+    white-space: nowrap;
+    color: white;
+}
+"""
 }
 
 
@@ -957,7 +1011,7 @@ sidebar_snapins["bookmarks"] = {
 # ------------------------------------------------------------
 
 def render_custom_links():
-    links = config.custom_links.get(config.role)
+    links = config.custom_links.get(config.user_baserole_id)
     if not links:
         html.write((_("Please edit <tt>%s</tt> in order to configure which links are shown in this snapin.") %
                   (defaults.default_config_dir + "/multisite.mk")) + "\n")
@@ -971,26 +1025,18 @@ def render_custom_links():
             try:
                 if type(entry[1]) == type(True):
                     idss = ids + [str(n)]
-                    if states.get(''.join(idss), entry[1] and 'on' or 'off') == 'on': # open
-                        display = ""
-                        img = "link_folder_open.gif"
-                    else:
-                        display = "display: none; "
-                        img = "link_folder.gif"
-                    html.write('<h3 onclick="toggle_folder(this, \'%s\');" ' % ''.join(idss))
-                    html.write('onmouseover="this.style.cursor=\'pointer\';" ')
-                    html.write('onmouseout="this.style.cursor=\'auto\';">')
-                    html.write('<img src="images/%s" align="center" />' % img)
-                    html.write("%s</h3>\n" % entry[0])
-                    html.write('<div style="%s" class=sublist>' % display)
+                    is_open = entry[1]
+                    id = '/'.join(idss)
+                    html.begin_foldable_container("customlinks", id, isopen=entry[1], title=entry[0])
                     render_list(idss, entry[2])
-                    html.write('</div>\n')
+                    html.end_foldable_container()
                 elif type(entry[1]) == str:
-                    if len(entry) > 2:
+                    frame = len(entry) > 3 and entry[3] or "main"
+                    if len(entry) > 2 and entry[2]:
                         html.write('<img src="images/%s">' % entry[2])
                     else:
                         html.write('<img src="images/link_link.gif">')
-                    simplelink(entry[0], entry[1])
+                    simplelink(entry[0], entry[1], frame)
                 else:
                     html.write(_("Second part of tuple must be list or string, not %s\n") % str(entry[1]))
             except Exception, e:
@@ -1001,7 +1047,6 @@ def render_custom_links():
 sidebar_snapins["custom_links"] = {
     "title" : _("Custom Links"),
     "description" : _("This snapin contains custom links which can be configured via the configuration variable <tt>custom_links</tt> in <tt>multisite.mk</tt>"),
-    "author" : "Mathias Kettner",
     "render" : render_custom_links,
     "allowed" : [ "user", "admin", "guest" ],
     "styles" : """
