@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -53,7 +53,8 @@ public:
     virtual ~Column() {}
     const char *name() const { return _name.c_str(); }
     const char *description() const { return _description.c_str(); }
-    virtual string valueAsString(void *data __attribute__ ((__unused__)), Query *) { return "invalid"; }
+    virtual string valueAsString(void *data __attribute__ ((__unused__)), Query *)
+        { return "invalid"; }
     virtual int type() = 0;
     virtual void output(void *data, Query *) = 0;
     virtual Filter *createFilter(int opid __attribute__ ((__unused__)), char *value __attribute__ ((__unused__))) { return 0; }
