@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -35,3 +35,10 @@ Filter *DoubleColumn::createFilter(int operator_id, char *value)
 {
     return new DoubleColumnFilter(this, operator_id, value);
 }
+
+string DoubleColumn::valueAsString(void *data, Query *query)
+{
+    char buf[64];
+    snprintf(buf, sizeof(buf), "%.10e", getValue(data));
+    return buf;
+}
