@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -34,18 +34,30 @@ void DownCommColumn::output(void *data, Query *query)
     TableDownComm *table = _is_downtime ? g_table_downtimes : g_table_comments;
     query->outputBeginList();
     data = shiftPointer(data); // points to host or service
-    if (data) 
+    if (data)
     {
         bool first = true;
+        bool found_match = false;
 
-        for (map<unsigned long, DowntimeOrComment *>::iterator it = table->entriesIteratorBegin();
+        for (map<pair<unsigned long, bool>, DowntimeOrComment *>::iterator it = table->entriesIteratorBegin();
                 it != table->entriesIteratorEnd();
                 ++it)
         {
-            unsigned long id = it->first;
-            DowntimeOrComment *dt = it->second;
-            if ((void *)dt->_service == data ||
-                    (dt->_service == 0 && dt->_host == data))
+            unsigned long id     = it->first.first;
+            bool is_service       =  it->first.second;
+            DowntimeOrComment *dt =  it->second;
+
+            found_match = false;
+
+            if (!is_service){
+                if (dt->_host->name == ((host_struct*)data)->name)
+                    found_match = true;
+            }
+            else
+               if ( dt->_service->description == ((service_struct*)data)->description && dt->_service->host_name == ((service_struct*)data)->host_name )
+                    found_match = true;
+
+            if (found_match)
             {
                 if (first)
                     first = false;
@@ -59,6 +71,12 @@ void DownCommColumn::output(void *data, Query *query)
                     query->outputString(dt->_author_name);
                     query->outputSublistSeparator();
                     query->outputString(dt->_comment);
+                    query->outputSublistSeparator();
+                    if(_with_extra_info && !_is_downtime) {
+                        query->outputInteger(((Comment*)dt)->_entry_type);
+                        query->outputSublistSeparator();
+                        query->outputTime(dt->_entry_time);
+                    }
                     query->outputEndSublist();
                 }
                 else
@@ -81,10 +99,9 @@ bool DownCommColumn::isNagiosMember(void *data, void *member)
     // data points to a host or service
     // member is not a pointer, but an unsigned int (hack)
     int64_t id = (int64_t)member; // Hack. Convert it back.
-    DowntimeOrComment *dt = table->findEntry(id);
-    return dt != 0 && 
-        ( dt->_service == (service *)data
-          || (dt->_service == 0 && dt->_host == (host *)data));
+    DowntimeOrComment *dt = table->findEntry(id, _is_service);
+    return dt != 0 &&
+        ( dt->_service == (service *)data || (dt->_service == 0 && dt->_host == (host *)data));
 }
 
 bool DownCommColumn::isEmpty(void *data)
@@ -92,7 +109,7 @@ bool DownCommColumn::isEmpty(void *data)
     if (!data) return true;
 
     TableDownComm *table = _is_downtime ? g_table_downtimes : g_table_comments;
-    for (map<unsigned long, DowntimeOrComment *>::iterator it = table->entriesIteratorBegin();
+    for (map<pair<unsigned long, bool>, DowntimeOrComment *>::iterator it = table->entriesIteratorBegin();
             it != table->entriesIteratorEnd();
             ++it)
     {
