@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -36,9 +36,11 @@ class DownCommColumn : public ListColumn
 {
     bool _is_downtime;
     bool _with_info;
+    bool _is_service; // and not host
+    bool _with_extra_info; // provides date and type
 public:
-    DownCommColumn(string name, string description, int indirect_offset, bool is_downtime, bool with_info)
-        : ListColumn(name, description, indirect_offset), _is_downtime(is_downtime), _with_info(with_info) {}
+    DownCommColumn(string name, string description, int indirect_offset, bool is_downtime, bool is_service, bool with_info, bool with_extra_info)
+        : ListColumn(name, description, indirect_offset), _is_downtime(is_downtime), _is_service(is_service), _with_info(with_info), _with_extra_info(with_extra_info) {}
     int type() { return COLTYPE_LIST; }
     void output(void *, Query *);
     void *getNagiosObject(char *name);
