@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -37,6 +37,11 @@ void IntAggregator::consume(void *data, Query *query)
         case STATS_OP_AVG:
             _aggr += value; break;
 
+        case STATS_OP_SUMINV:
+        case STATS_OP_AVGINV:
+            _sumq += 1.0 / (double)value;
+            break;
+
         case STATS_OP_MIN:
             if (_count == 1)
                 _aggr = value;
@@ -65,13 +70,21 @@ void IntAggregator::output(Query *q)
         case STATS_OP_SUM:
         case STATS_OP_MIN:
         case STATS_OP_MAX:
-            q->outputInteger64(_aggr); 
+            q->outputInteger64(_aggr);
+            break;
+
+        case STATS_OP_SUMINV:
+            q->outputInteger64(_sumq);
             break;
 
         case STATS_OP_AVG:
             q->outputDouble(double(_aggr) / _count);
             break;
 
+        case STATS_OP_AVGINV:
+            q->outputInteger64(_sumq / _count);
+            break;
+
         case STATS_OP_STD:
             if (_count <= 1)
                 q->outputDouble(0.0);
@@ -81,7 +94,7 @@ void IntAggregator::output(Query *q)
     }
 }
 
-/* Algorithmus fuer die Standardabweichung, 
+/* Algorithmus fuer die Standardabweichung,
    bei der man zuvor den Mittelwert nicht
    wissen muss:
 
