@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -29,18 +29,16 @@
 #include "logger.h"
 #include "LogEntry.h"
 #include "Query.h"
-#include "TableLog.h"
+#include "LogCache.h"
 
-extern int num_cached_log_messages;
-extern int g_debug_level;
 
-    Logfile::Logfile(const char *path, bool watch)
-    : _path(strdup(path))
-    , _since(0)
-    , _watch(watch)
-    , _inode(0)
-    , _lineno(0)
-      , _logclasses_read(0)
+Logfile::Logfile(const char *path, bool watch)
+  : _path(strdup(path))
+  , _since(0)
+  , _watch(watch)
+  , _inode(0)
+  , _lineno(0)
+  , _logclasses_read(0)
 {
     int fd = open(path, O_RDONLY);
     if (fd < 0) {
@@ -75,24 +73,20 @@ Logfile::~Logfile()
 
 void Logfile::flush()
 {
-    for (_entries_t::iterator it = _entries.begin();
-            it != _entries.end();
-            ++it)
-    {
+	for (logfile_entries_t::iterator it = _entries.begin(); it != _entries.end(); ++it)
         delete it->second;
-    }
+
     _entries.clear();
     _logclasses_read = 0;
 }
 
 
-void Logfile::load(TableLog *tablelog, time_t since, time_t until, unsigned logclasses)
+void Logfile::load(LogCache *logcache, time_t since, time_t until, unsigned logclasses)
 {
     unsigned missing_types = logclasses & ~_logclasses_read;
-
     FILE *file = 0;
     // The current logfile has the _watch flag set to true.
-    // In that case, if the logfile has grown, we need to 
+    // In that case, if the logfile has grown, we need to
     // load the rest of the file, even if no logclasses
     // are missing.
     if (_watch) {
@@ -110,19 +104,19 @@ void Logfile::load(TableLog *tablelog, time_t since, time_t until, unsigned logc
         // have read to the end of the file
         if (_logclasses_read) {
             fsetpos(file, &_read_pos); // continue at previous end
-            load(file, _logclasses_read, tablelog, since, until, logclasses);
+            loadRange(file, _logclasses_read, logcache, since, until, logclasses);
             fgetpos(file, &_read_pos);
         }
         if (missing_types) {
-            fseek(file, 0, SEEK_SET);
+        	fseek(file, 0, SEEK_SET);
             _lineno = 0;
-            load(file, missing_types, tablelog, since, until, logclasses);
+            loadRange(file, missing_types, logcache, since, until, logclasses);
             _logclasses_read |= missing_types;
             fgetpos(file, &_read_pos); // remember current end of file
         }
         fclose(file);
     }
-    else 
+    else
     {
         if (missing_types == 0)
             return;
@@ -133,32 +127,28 @@ void Logfile::load(TableLog *tablelog, time_t since, time_t until, unsigned logc
             return;
         }
 
-        load(file, missing_types, tablelog, since, until, logclasses);
+        loadRange(file, missing_types, logcache, since, until, logclasses);
         fclose(file);
         _logclasses_read |= missing_types;
     }
 }
 
-void Logfile::load(FILE *file, unsigned missing_types, 
-        TableLog *tablelog, time_t since, time_t until, unsigned logclasses)
+void Logfile::loadRange(FILE *file, unsigned missing_types,
+        LogCache *logcache, time_t since, time_t until, unsigned logclasses)
 {
     while (fgets(_linebuffer, MAX_LOGLINE, file))
     {
         _lineno++;
         if (processLogLine(_lineno, missing_types)) {
-            num_cached_log_messages ++;
-            tablelog->handleNewMessage(this, since, until, logclasses); // memory management
+            logcache->handleNewMessage(this, since, until, logclasses); // memory management
         }
-    }	
+    }
 }
 
-
 long Logfile::freeMessages(unsigned logclasses)
 {
     long freed = 0;
-    for (_entries_t::iterator it = _entries.begin();
-            it != _entries.end();
-            ++it)
+    for (logfile_entries_t::iterator it = _entries.begin(); it != _entries.end(); ++it)
     {
         LogEntry *entry = it->second;
         if ((1 << entry->_logclass) & logclasses)
@@ -172,7 +162,6 @@ long Logfile::freeMessages(unsigned logclasses)
     return freed;
 }
 
-
 bool Logfile::processLogLine(uint32_t lineno, unsigned logclasses)
 {
     LogEntry *entry = new LogEntry(lineno, _linebuffer);
@@ -198,12 +187,17 @@ bool Logfile::processLogLine(uint32_t lineno, unsigned logclasses)
     }
 }
 
-
-bool Logfile::answerQuery(Query *query, TableLog *tablelog, time_t since, time_t until, unsigned logclasses)
+logfile_entries_t* Logfile::getEntriesFromQuery(Query *query, LogCache *logcache, time_t since, time_t until, unsigned logclasses)
 {
-    load(tablelog, since, until, logclasses); // make sure all messages are present
+	 load(logcache, since, until, logclasses); // make sure all messages are present
+	 return &_entries;
+}
+
+bool Logfile::answerQuery(Query *query, LogCache *logcache, time_t since, time_t until, unsigned logclasses)
+{
+    load(logcache, since, until, logclasses); // make sure all messages are present
     uint64_t sincekey = makeKey(since, 0);
-    _entries_t::iterator it = _entries.lower_bound(sincekey);
+    logfile_entries_t::iterator it = _entries.lower_bound(sincekey);
     while (it != _entries.end())
     {
         LogEntry *entry = it->second;
@@ -216,11 +210,11 @@ bool Logfile::answerQuery(Query *query, TableLog *tablelog, time_t since, time_t
     return true;
 }
 
-bool Logfile::answerQueryReverse(Query *query, TableLog *tablelog, time_t since, time_t until, unsigned logclasses)
+bool Logfile::answerQueryReverse(Query *query, LogCache *logcache, time_t since, time_t until, unsigned logclasses)
 {
-    load(tablelog, since, until, logclasses); // make sure all messages are present
+    load(logcache, since, until, logclasses); // make sure all messages are present
     uint64_t untilkey = makeKey(until, 999999999);
-    _entries_t::iterator it = _entries.upper_bound(untilkey);
+    logfile_entries_t::iterator it = _entries.upper_bound(untilkey);
     while (it != _entries.begin())
     {
         --it;
