@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -34,7 +34,8 @@ class OffsetDoubleColumn : public DoubleColumn
 {
     int _offset;
 public:
-    OffsetDoubleColumn(string name, string description, int offset, int indirect_offset = -1) 
+    OffsetDoubleColumn(string name, string description,
+                       int offset, int indirect_offset = -1)
         : DoubleColumn(name, description, indirect_offset), _offset(offset) {}
     double getValue(void *data);
 protected:
