@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -25,12 +25,14 @@
 #ifndef StatsColumn_h
 #define StatsColumn_h
 
-#define STATS_OP_COUNT 0
-#define STATS_OP_SUM   1
-#define STATS_OP_MIN   2
-#define STATS_OP_MAX   3
-#define STATS_OP_AVG   4
-#define STATS_OP_STD   5
+#define STATS_OP_COUNT  0
+#define STATS_OP_SUM    1
+#define STATS_OP_MIN    2
+#define STATS_OP_MAX    3
+#define STATS_OP_AVG    4
+#define STATS_OP_STD    5
+#define STATS_OP_SUMINV 6
+#define STATS_OP_AVGINV 7
 
 class Aggregator;
 class Column;
@@ -43,7 +45,7 @@ class StatsColumn
     int _operation;
 
 public:
-    StatsColumn(Column *c, Filter *f, int o) : 
+    StatsColumn(Column *c, Filter *f, int o) :
         _column(c), _filter(f), _operation(o) {}
     ~StatsColumn();
     int operation() { return _operation; }
