@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -38,15 +38,15 @@
 extern int g_debug_level;
 extern unsigned long g_max_cached_messages;
 
-    Store::Store()
-    :_table_hosts(false)
-    , _table_hostsbygroup(true)
-    , _table_services(false, false)
-    , _table_servicesbygroup(true, false)
-    , _table_servicesbyhostgroup(false, true)
-    , _table_downtimes(true)
-    , _table_comments(false)
-     , _table_log(g_max_cached_messages)
+Store::Store()
+  : _log_cache(g_max_cached_messages)
+  , _table_hosts(false)
+  , _table_hostsbygroup(true)
+  , _table_services(false, false)
+  , _table_servicesbygroup(true, false)
+  , _table_servicesbyhostgroup(false, true)
+  , _table_downtimes(true)
+  , _table_comments(false)
 {
     _tables.insert(make_pair("hosts", &_table_hosts));
     _tables.insert(make_pair("hostsbygroup", &_table_hostsbygroup));
@@ -61,6 +61,7 @@ extern unsigned long g_max_cached_messages;
     _tables.insert(make_pair("comments", &_table_comments));
     _tables.insert(make_pair("status", &_table_status));
     _tables.insert(make_pair("log", &_table_log));
+    _tables.insert(make_pair("statehist", &_table_statehistory));
     _tables.insert(make_pair("timeperiods", &_table_timeperiods));
     _tables.insert(make_pair("contactgroups", &_table_contactgroups));
     _tables.insert(make_pair("columns", &_table_columns));
@@ -79,6 +80,7 @@ extern unsigned long g_max_cached_messages;
     g_table_timeperiods = &_table_timeperiods;
     g_table_contactgroups = &_table_contactgroups;
     g_table_log = &_table_log;
+    g_table_statehistory = &_table_statehistory;
     g_table_columns = &_table_columns;
 
     for (_tables_t::iterator it = _tables.begin();
@@ -115,7 +117,8 @@ bool Store::answerRequest(InputBuffer *input, OutputBuffer *output)
     int r = input->readRequest();
     if (r != IB_REQUEST_READ) {
         if (r != IB_END_OF_FILE)
-            output->setError(RESPONSE_CODE_INCOMPLETE_REQUEST, "Client connection terminated while request still incomplete");
+            output->setError(RESPONSE_CODE_INCOMPLETE_REQUEST,
+                "Client connection terminated while request still incomplete");
         return false;
     }
     string l = input->nextLine();
@@ -131,8 +134,9 @@ bool Store::answerRequest(InputBuffer *input, OutputBuffer *output)
         output->setDoKeepalive(true);
     }
     else if (!strncmp(line, "LOGROTATE", 9)) {
-        logger(LG_INFO, "Forcing logfile rotation");
-        rotate_log_file(time(0));
+    	logger(LG_INFO, "Forcing logfile rotation");
+		rotate_log_file(time(0));
+        schedule_new_event(EVENT_LOG_ROTATION,TRUE,get_next_log_rotation_time(),FALSE,0,(void *)get_next_log_rotation_time,TRUE,NULL,NULL,0);
     }
     else {
         logger(LG_INFO, "Invalid request '%s'", line);
@@ -152,7 +156,6 @@ void Store::answerCommandRequest(const char *command)
 void Store::answerGetRequest(InputBuffer *input, OutputBuffer *output, const char *tablename)
 {
     output->reset();
-
     if (!tablename[0]) {
         output->setError(RESPONSE_CODE_INVALID_REQUEST, "Invalid GET request, missing tablename");
     }
