@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -45,19 +45,19 @@ TableDownComm::TableDownComm(bool is_downtime)
         _name = "comments";
 
     DowntimeOrComment *ref = 0;
-    addColumn(new OffsetStringColumn("author", 
-                is_downtime ? "The contact that scheduled the downtime" : "The contact that entered the comment", 
+    addColumn(new OffsetStringColumn("author",
+                is_downtime ? "The contact that scheduled the downtime" : "The contact that entered the comment",
                 (char *)&(ref->_author_name) - (char *)ref));
-    addColumn(new OffsetStringColumn("comment", 
+    addColumn(new OffsetStringColumn("comment",
                 "A comment text", (char *)&(ref->_comment) - (char *)ref));
-    addColumn(new OffsetIntColumn("id", 
+    addColumn(new OffsetIntColumn("id",
                 is_downtime ? "The id of the downtime" : "The id of the comment", (char *)&(ref->_id) - (char *)ref));
     addColumn(new OffsetTimeColumn("entry_time",
                 "The time the entry was made as UNIX timestamp", (char *)&(ref->_entry_time) - (char *)ref));
     addColumn(new OffsetIntColumn("type",
                 is_downtime ?  "The type of the downtime: 0 if it is active, 1 if it is pending" :
                 "The type of the comment: 1 is host, 2 is service", (char *)&(ref->_type) - (char *)ref));
-    addColumn(new OffsetIntColumn("is_service", 
+    addColumn(new OffsetIntColumn("is_service",
                 "0, if this entry is for a host, 1 if it is for a service", (char *)&(ref->_is_service) - (char *)ref));
 
     if (is_downtime)
@@ -72,7 +72,7 @@ TableDownComm::TableDownComm(bool is_downtime)
         addColumn(new OffsetIntColumn("duration",
                     "The duration of the downtime in seconds", (char *)&(ref->_duration) - (char *)ref));
         addColumn(new OffsetIntColumn("triggered_by",
-                    "The id of the downtime this downtime was triggered by or 0 if it was not triggered by another downtime", 
+                    "The id of the downtime this downtime was triggered by or 0 if it was not triggered by another downtime",
                     (char *)&(ref->_triggered_by) - (char *)ref));
     }
     else {
@@ -109,11 +109,11 @@ void TableDownComm::addComment(nebstruct_comment_data *data) {
         add(new Comment(data));
     }
     else if (data->type == NEBTYPE_COMMENT_DELETE) {
-        remove(id);
+        Comment comment(data);
+        remove(&comment);
     }
 }
 
-
 void TableDownComm::addDowntime(nebstruct_downtime_data *data)
 {
     unsigned long id = data->downtime_id;
@@ -121,27 +121,30 @@ void TableDownComm::addDowntime(nebstruct_downtime_data *data)
         add(new Downtime(data));
     }
     else if (data->type == NEBTYPE_DOWNTIME_DELETE) {
-        remove(id);
+        Downtime downtime(data);
+        remove(&downtime);
     }
 }
 
-
 void TableDownComm::add(DowntimeOrComment *data)
 {
-    _entries_t::iterator it = _entries.find(data->_id);
+    dc_key tmp_key = make_pair(data->_id, data->_service != 0);
+    _entries_t::iterator it = _entries.find(tmp_key);
+
     // might be update -> delete previous data set
     if (it != _entries.end()) {
         delete it->second;
         _entries.erase(it);
     }
-    _entries.insert(make_pair(data->_id, data));
+    _entries.insert(make_pair(tmp_key, data));
 }
 
-void TableDownComm::remove(unsigned id)
+void TableDownComm::remove(DowntimeOrComment *data)
 {
-    _entries_t::iterator it = _entries.find(id);
+    dc_key tmp_key = make_pair(data->_id, data->_service != 0);
+    _entries_t::iterator it = _entries.find(tmp_key);
     if (it == _entries.end())
-        logger(LG_INFO, "Cannot delete non-existing downtime/comment %u", id);
+        logger(LG_INFO, "Cannot delete non-existing downtime/comment %u", data->_id);
     else {
         delete it->second;
         _entries.erase(it);
@@ -161,15 +164,17 @@ void TableDownComm::answerQuery(Query *query)
 
 bool TableDownComm::isAuthorized(contact *ctc, void *data)
 {
-    DowntimeOrComment *dtc = (DowntimeOrComment *)data; 
+    DowntimeOrComment *dtc = (DowntimeOrComment *)data;
     return is_authorized_for(ctc, dtc->_host, dtc->_service);
 }
 
-DowntimeOrComment *TableDownComm::findEntry(unsigned long id)
+DowntimeOrComment *TableDownComm::findEntry(unsigned long id, bool is_service)
 {
-    _entries_t::iterator it = _entries.find(id);
-    if (it != _entries.end())
+    dc_key tmp_key = make_pair(id, is_service);
+    _entries_t::iterator it = _entries.find(tmp_key);
+    if (it != _entries.end()) {
         return it->second;
+    }
     else
         return 0;
 }
