@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -41,18 +41,20 @@ using namespace std;
 class TableDownComm : public Table
 {
     const char *_name;
-    typedef map<unsigned long, DowntimeOrComment *> _entries_t;
+
+    typedef pair<unsigned long, bool> dc_key;
+    typedef map<dc_key, DowntimeOrComment *> _entries_t;
     _entries_t _entries;
 
 public:
     TableDownComm(bool is_downtime);
     const char *name() { return _name; }
     ~TableDownComm();
-    DowntimeOrComment *findEntry(unsigned long id);
+    DowntimeOrComment *findEntry(unsigned long id, bool is_service);
     void addDowntime(nebstruct_downtime_data *);
     void addComment(nebstruct_comment_data *);
     void add(DowntimeOrComment *data);
-    void remove(unsigned id);
+    void remove(DowntimeOrComment *data);
     void answerQuery(Query *);
     bool isAuthorized(contact *ctc, void *data);
     _entries_t::iterator entriesIteratorBegin() { return _entries.begin(); }
