@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -145,7 +145,7 @@ void TableServices::answerQuery(Query *query)
                 smem = smem->next;
             }
             mem = mem->next;
-        }  
+        }
         return;
     }
 
@@ -164,9 +164,10 @@ bool TableServices::isAuthorized(contact *ctc, void *data)
     return is_authorized_for(ctc, svc->host_ptr, svc);
 }
 
-    TableServices::TableServices(bool by_group, bool by_hostgroup)
-    : _by_group(by_group)
-      , _by_hostgroup(by_hostgroup)
+
+TableServices::TableServices(bool by_group, bool by_hostgroup)
+  : _by_group(by_group)
+  , _by_hostgroup(by_hostgroup)
 {
     struct servicebygroup     sgref;
     struct servicebyhostgroup hgref;
@@ -187,105 +188,107 @@ void TableServices::addColumns(Table *table, string prefix, int indirect_offset,
 
     service svc;
     const char *ref = (const char *)&svc;
-    table->addColumn(new OffsetStringColumn(prefix + "description", 
+    table->addColumn(new OffsetStringColumn(prefix + "description",
                 "Description of the service (also used as key)", (char *)(&svc.description) - ref, indirect_offset));
-    table->addColumn(new OffsetStringColumn(prefix + "display_name", 
+    table->addColumn(new OffsetStringColumn(prefix + "display_name",
                 "An optional display name (not used by Nagios standard web pages)", (char *)(&svc.display_name) - ref, indirect_offset));
-    table->addColumn(new OffsetStringColumn(prefix + "check_command", 
+    table->addColumn(new OffsetStringColumn(prefix + "check_command",
                 "Nagios command used for active checks", (char *)(&svc.service_check_command) - ref, indirect_offset));
-    table->addColumn(new OffsetStringColumn(prefix + "event_handler", 
+    table->addColumn(new OffsetStringServiceMacroColumn(prefix + "check_command_expanded",
+                "Nagios command used for active checks with the macros expanded", (char *)(&svc.service_check_command) - ref, indirect_offset));
+    table->addColumn(new OffsetStringColumn(prefix + "event_handler",
                 "Nagios command used as event handler", (char *)(&svc.event_handler) - ref, indirect_offset));
-    table->addColumn(new OffsetStringColumn(prefix + "plugin_output", 
+    table->addColumn(new OffsetStringColumn(prefix + "plugin_output",
                 "Output of the last check plugin", (char *)(&svc.plugin_output) - ref, indirect_offset));
-    table->addColumn(new OffsetStringColumn(prefix + "long_plugin_output", 
+    table->addColumn(new OffsetStringColumn(prefix + "long_plugin_output",
                 "Unabbreviated output of the last check plugin", (char *)(&svc.long_plugin_output) - ref, indirect_offset));
-    table->addColumn(new OffsetStringColumn(prefix + "perf_data", 
+    table->addColumn(new OffsetStringColumn(prefix + "perf_data",
                 "Performance data of the last check plugin", (char *)(&svc.perf_data) - ref, indirect_offset));
-    table->addColumn(new OffsetStringColumn(prefix + "notification_period", 
+    table->addColumn(new OffsetStringColumn(prefix + "notification_period",
                 "The name of the notification period of the service. It this is empty, service problems are always notified.", (char *)(&svc.notification_period) - ref, indirect_offset));
-    table->addColumn(new OffsetStringColumn(prefix + "check_period", 
+    table->addColumn(new OffsetStringColumn(prefix + "check_period",
                 "The name of the check period of the service. It this is empty, the service is always checked.", (char *)(&svc.check_period) - ref, indirect_offset));
-    table->addColumn(new OffsetStringColumn(prefix + "notes", 
+    table->addColumn(new OffsetStringColumn(prefix + "notes",
                 "Optional notes about the service", (char *)(&svc.notes) - ref, indirect_offset));
     table->addColumn(new OffsetStringServiceMacroColumn(prefix + "notes_expanded",
                 "The notes with (the most important) macros expanded", (char *)(&svc.notes) - ref, indirect_offset));
-    table->addColumn(new OffsetStringColumn(prefix + "notes_url", 
+    table->addColumn(new OffsetStringColumn(prefix + "notes_url",
                 "An optional URL for additional notes about the service", (char *)(&svc.notes_url) - ref, indirect_offset));
-    table->addColumn(new OffsetStringServiceMacroColumn(prefix + "notes_url_expanded", 
+    table->addColumn(new OffsetStringServiceMacroColumn(prefix + "notes_url_expanded",
                 "The notes_url with (the most important) macros expanded", (char *)(&svc.notes_url) - ref, indirect_offset));
-    table->addColumn(new OffsetStringColumn(prefix + "action_url", 
+    table->addColumn(new OffsetStringColumn(prefix + "action_url",
                 "An optional URL for actions or custom information about the service", (char *)(&svc.action_url) - ref, indirect_offset));
-    table->addColumn(new OffsetStringServiceMacroColumn(prefix + "action_url_expanded", 
+    table->addColumn(new OffsetStringServiceMacroColumn(prefix + "action_url_expanded",
                 "The action_url with (the most important) macros expanded", (char *)(&svc.action_url) - ref, indirect_offset));
-    table->addColumn(new OffsetStringColumn(prefix + "icon_image", 
+    table->addColumn(new OffsetStringColumn(prefix + "icon_image",
                 "The name of an image to be used as icon in the web interface", (char *)(&svc.icon_image) - ref, indirect_offset));
-    table->addColumn(new OffsetStringServiceMacroColumn(prefix + "icon_image_expanded", 
+    table->addColumn(new OffsetStringServiceMacroColumn(prefix + "icon_image_expanded",
                 "The icon_image with (the most important) macros expanded", (char *)(&svc.icon_image) - ref, indirect_offset));
-    table->addColumn(new OffsetStringColumn(prefix + "icon_image_alt", 
+    table->addColumn(new OffsetStringColumn(prefix + "icon_image_alt",
                 "An alternative text for the icon_image for browsers not displaying icons", (char *)(&svc.icon_image_alt) - ref, indirect_offset));
 
-    table->addColumn(new OffsetIntColumn(prefix + "initial_state", 
+    table->addColumn(new OffsetIntColumn(prefix + "initial_state",
                 "The initial state of the service", (char *)&svc.initial_state - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "max_check_attempts", 
+    table->addColumn(new OffsetIntColumn(prefix + "max_check_attempts",
                 "The maximum number of check attempts", (char *)&svc.max_attempts - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "current_attempt", 
+    table->addColumn(new OffsetIntColumn(prefix + "current_attempt",
                 "The number of the current check attempt", (char *)&svc.current_attempt - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "state", 
+    table->addColumn(new OffsetIntColumn(prefix + "state",
                 "The current state of the service (0: OK, 1: WARN, 2: CRITICAL, 3: UNKNOWN)", (char *)&svc.current_state - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "has_been_checked", 
+    table->addColumn(new OffsetIntColumn(prefix + "has_been_checked",
                 "Whether the service already has been checked (0/1)", (char *)&svc.has_been_checked - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "last_state", 
+    table->addColumn(new OffsetIntColumn(prefix + "last_state",
                 "The last state of the service", (char *)&svc.last_state - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "last_hard_state", 
+    table->addColumn(new OffsetIntColumn(prefix + "last_hard_state",
                 "The last hard state of the service", (char *)&svc.last_hard_state - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "state_type", 
+    table->addColumn(new OffsetIntColumn(prefix + "state_type",
                 "The type of the current state (0: soft, 1: hard)", (char *)&svc.state_type - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "check_type", 
+    table->addColumn(new OffsetIntColumn(prefix + "check_type",
                 "The type of the last check (0: active, 1: passive)", (char *)&svc.check_type - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "acknowledged", 
+    table->addColumn(new OffsetIntColumn(prefix + "acknowledged",
                 "Whether the current service problem has been acknowledged (0/1)", (char *)&svc.problem_has_been_acknowledged - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "acknowledgement_type", 
+    table->addColumn(new OffsetIntColumn(prefix + "acknowledgement_type",
                 "The type of the acknownledgement (0: none, 1: normal, 2: sticky)", (char *)&svc.acknowledgement_type - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "no_more_notifications", 
+    table->addColumn(new OffsetIntColumn(prefix + "no_more_notifications",
                 "Whether to stop sending notifications (0/1)", (char *)(&svc.no_more_notifications) - ref, indirect_offset));
-    table->addColumn(new OffsetTimeColumn(prefix + "last_state_change", 
+    table->addColumn(new OffsetTimeColumn(prefix + "last_state_change",
                 "The time of the last state change (Unix timestamp)", (char *)&svc.last_state_change - ref, indirect_offset));
-    table->addColumn(new OffsetTimeColumn(prefix + "last_time_ok", 
+    table->addColumn(new OffsetTimeColumn(prefix + "last_time_ok",
                 "The last time the service was OK (Unix timestamp)", (char *)&svc.last_time_ok - ref, indirect_offset));
-    table->addColumn(new OffsetTimeColumn(prefix + "last_time_warning", 
+    table->addColumn(new OffsetTimeColumn(prefix + "last_time_warning",
                 "The last time the service was in WARNING state (Unix timestamp)", (char *)&svc.last_time_warning - ref, indirect_offset));
-    table->addColumn(new OffsetTimeColumn(prefix + "last_time_critical", 
+    table->addColumn(new OffsetTimeColumn(prefix + "last_time_critical",
                 "The last time the service was CRITICAL (Unix timestamp)", (char *)&svc.last_time_critical - ref, indirect_offset));
-    table->addColumn(new OffsetTimeColumn(prefix + "last_time_unknown", 
+    table->addColumn(new OffsetTimeColumn(prefix + "last_time_unknown",
                 "The last time the service was UNKNOWN (Unix timestamp)", (char *)&svc.last_time_unknown - ref, indirect_offset));
 
-    table->addColumn(new OffsetTimeColumn(prefix + "last_check", 
+    table->addColumn(new OffsetTimeColumn(prefix + "last_check",
                 "The time of the last check (Unix timestamp)", (char *)&svc.last_check - ref, indirect_offset));
-    table->addColumn(new OffsetTimeColumn(prefix + "next_check", 
+    table->addColumn(new OffsetTimeColumn(prefix + "next_check",
                 "The scheduled time of the next check (Unix timestamp)", (char *)&svc.next_check - ref, indirect_offset));
-    table->addColumn(new OffsetTimeColumn(prefix + "last_notification", 
+    table->addColumn(new OffsetTimeColumn(prefix + "last_notification",
                 "The time of the last notification (Unix timestamp)", (char *)&svc.last_notification - ref, indirect_offset));
-    table->addColumn(new OffsetTimeColumn(prefix + "next_notification", 
+    table->addColumn(new OffsetTimeColumn(prefix + "next_notification",
                 "The time of the next notification (Unix timestamp)", (char *)&svc.next_notification - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "current_notification_number", 
+    table->addColumn(new OffsetIntColumn(prefix + "current_notification_number",
                 "The number of the current notification", (char *)&svc.current_notification_number - ref, indirect_offset));
-    table->addColumn(new OffsetTimeColumn(prefix + "last_state_change", 
+    table->addColumn(new OffsetTimeColumn(prefix + "last_state_change",
                 "The time of the last state change - soft or hard (Unix timestamp)", (char *)&svc.last_state_change - ref, indirect_offset));
-    table->addColumn(new OffsetTimeColumn(prefix + "last_hard_state_change", 
+    table->addColumn(new OffsetTimeColumn(prefix + "last_hard_state_change",
                 "The time of the last hard state change (Unix timestamp)", (char *)&svc.last_hard_state_change - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "scheduled_downtime_depth", 
+    table->addColumn(new OffsetIntColumn(prefix + "scheduled_downtime_depth",
                 "The number of scheduled downtimes the service is currently in", (char *)(&svc.scheduled_downtime_depth) - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "is_flapping", 
+    table->addColumn(new OffsetIntColumn(prefix + "is_flapping",
                 "Whether the service is flapping (0/1)", (char *)(&svc.is_flapping) - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "checks_enabled", 
+    table->addColumn(new OffsetIntColumn(prefix + "checks_enabled",
                 "Whether active checks are enabled for the service (0/1)", (char *)(&svc.checks_enabled) - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "accept_passive_checks", 
+    table->addColumn(new OffsetIntColumn(prefix + "accept_passive_checks",
                 "Whether the service accepts passive checks (0/1)", (char *)(&svc.accept_passive_service_checks) - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "event_handler_enabled", 
+    table->addColumn(new OffsetIntColumn(prefix + "event_handler_enabled",
                 "Whether and event handler is activated for the service (0/1)", (char *)(&svc.event_handler_enabled) - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "notifications_enabled", 
+    table->addColumn(new OffsetIntColumn(prefix + "notifications_enabled",
                 "Whether notifications are enabled for the service (0/1)", (char *)(&svc.notifications_enabled) - ref, indirect_offset));
-    table->addColumn(new OffsetIntColumn(prefix + "process_performance_data", 
+    table->addColumn(new OffsetIntColumn(prefix + "process_performance_data",
                 "Whether processing of performance data is enabled for the service (0/1)", (char *)(&svc.process_performance_data) - ref, indirect_offset));
     table->addColumn(new OffsetIntColumn(prefix + "is_executing",
                 "is there a service check currently running... (0/1)", (char *)(&svc.is_executing) - ref, indirect_offset));
@@ -295,6 +298,8 @@ void TableServices::addColumns(Table *table, string prefix, int indirect_offset,
                 "The current check option, forced, normal, freshness... (0/1)", (char *)(&svc.check_options) - ref, indirect_offset));
     table->addColumn(new OffsetIntColumn(prefix + "flap_detection_enabled",
                 "Whether flap detection is enabled for the service (0/1)", (char *)(&svc.flap_detection_enabled) - ref, indirect_offset));
+    table->addColumn(new OffsetIntColumn(prefix + "check_freshness",
+                "Whether freshness checks are activated (0/1)", (char *)(&svc.check_freshness) - ref, indirect_offset));
     table->addColumn(new OffsetIntColumn(prefix + "obsess_over_service",
                 "Whether 'obsess_over_service' is enabled for the service (0/1)", (char *)(&svc.obsess_over_service) - ref, indirect_offset));
     table->addColumn(new AttributelistColumn(prefix + "modified_attributes",
@@ -325,34 +330,36 @@ void TableServices::addColumns(Table *table, string prefix, int indirect_offset,
                 "Percent state change", (char *)(&svc.percent_state_change) - ref, indirect_offset));
 
 
-    table->addColumn(new OffsetTimeperiodColumn(prefix + "in_check_period", 
+    table->addColumn(new OffsetTimeperiodColumn(prefix + "in_check_period",
                 "Whether the service is currently in its check period (0/1)", (char *)&svc.check_period_ptr - ref, indirect_offset));
-    table->addColumn(new OffsetTimeperiodColumn(prefix + "in_notification_period", 
+    table->addColumn(new OffsetTimeperiodColumn(prefix + "in_notification_period",
                 "Whether the service is currently in its notification period (0/1)", (char *)&svc.notification_period_ptr - ref, indirect_offset));
 
 
-    table->addColumn(new ServiceContactsColumn(prefix + "contacts", 
+    table->addColumn(new ServiceContactsColumn(prefix + "contacts",
                 "A list of all contacts of the service, either direct or via a contact group", indirect_offset));
-    table->addColumn(new DownCommColumn(prefix + "downtimes", 
-                "A list of all downtime ids of the service", indirect_offset, true, false));
-    table->addColumn(new DownCommColumn(prefix + "downtimes_with_info", 
-                "A list of all downtimes of the service with id, author and comment", indirect_offset, true, true));
-    table->addColumn(new DownCommColumn(prefix + "comments", 
-                "A list of all comment ids of the service", indirect_offset, false, false));
-    table->addColumn(new DownCommColumn(prefix + "comments_with_info", 
-                "A list of all comments of the service with id, author and comment", indirect_offset, false, true));
+    table->addColumn(new DownCommColumn(prefix + "downtimes",
+                "A list of all downtime ids of the service", indirect_offset, true, true, false, false));
+    table->addColumn(new DownCommColumn(prefix + "downtimes_with_info",
+                "A list of all downtimes of the service with id, author and comment", indirect_offset, true, true, true, false));
+    table->addColumn(new DownCommColumn(prefix + "comments",
+                "A list of all comment ids of the service", indirect_offset, false, true, false, false));
+    table->addColumn(new DownCommColumn(prefix + "comments_with_info",
+                "A list of all comments of the service with id, author and comment", indirect_offset, false, true, true, false));
+    table->addColumn(new DownCommColumn(prefix + "comments_with_extra_info",
+                "A list of all comments of the service with id, author, comment, entry type and entry time", indirect_offset, false, true, true, true));
 
     if (add_hosts)
         g_table_hosts->addColumns(this, "host_", (char *)(&svc.host_ptr) - ref);
 
-    table->addColumn(new CustomVarsColumn(prefix + "custom_variable_names", 
+    table->addColumn(new CustomVarsColumn(prefix + "custom_variable_names",
                 "A list of the names of all custom variables of the service", (char *)(&svc.custom_variables) - ref, indirect_offset, CVT_VARNAMES));
-    table->addColumn(new CustomVarsColumn(prefix + "custom_variable_values", 
+    table->addColumn(new CustomVarsColumn(prefix + "custom_variable_values",
                 "A list of the values of all custom variable of the service", (char *)(&svc.custom_variables) - ref, indirect_offset, CVT_VALUES));
-    table->addColumn(new CustomVarsColumn(prefix + "custom_variables", 
-                "A dictorionary of the custom variables", (char *)(&svc.custom_variables) - ref, indirect_offset, CVT_DICT));
+    table->addColumn(new CustomVarsColumn(prefix + "custom_variables",
+                "A dictionary of the custom variables", (char *)(&svc.custom_variables) - ref, indirect_offset, CVT_DICT));
 
-    table->addColumn(new ServicegroupsColumn(prefix + "groups", 
+    table->addColumn(new ServicegroupsColumn(prefix + "groups",
                 "A list of all service groups the service is in", (char *)(&svc.servicegroups_ptr) - ref, indirect_offset));
     table->addColumn(new ContactgroupsColumn(prefix + "contact_groups",
                 "A list of all contact groups this service is in", (char *)(&svc.contact_groups) - ref, indirect_offset));
