@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -42,13 +42,35 @@ TimeperiodsCache::~TimeperiodsCache()
     pthread_mutex_destroy(&_cache_lock);
 }
 
+void TimeperiodsCache::logCurrentTimeperiods(){
+    pthread_mutex_lock(&_cache_lock);
+    time_t now = time(0);
+    // Loop over all timeperiods and compute if we are
+    // currently in. Detect the case where no time periods
+    // are known (yet!). This might be the case when a timed
+    // event broker message arrives *before* the start of the
+    // event loop.
+    timeperiod *tp = timeperiod_list;
+    while (tp) {
+        bool is_in = 0 == check_time_against_period(now, tp);
+        // check previous state and log transition if state has changed
+        _cache_t::iterator it = _cache.find(tp);
+        if (it == _cache.end()) { // first entry
+            logTransition(tp->name, -1, is_in ? 1 : 0);
+            _cache.insert(std::make_pair(tp, is_in));
+        }
+        logTransition(tp->name, it->second ? 1 : 0, is_in ? 1 : 0);
+        tp = tp->next;
+    }
+    pthread_mutex_unlock(&_cache_lock);
+}
 
 void TimeperiodsCache::update(time_t now)
 {
     pthread_mutex_lock(&_cache_lock);
 
     // update cache only once a minute. The timeperiod
-    // definitions have 1 minute as granularity, so a 
+    // definitions have 1 minute as granularity, so a
     // 1sec resultion is not needed.
     int minutes = now / 60;
     if (minutes == _cache_time) {
@@ -100,7 +122,7 @@ bool TimeperiodsCache::inTimeperiod(timeperiod *tp)
         logger(LG_INFO, "No timeperiod information available for %s. Assuming out of period.", tp->name);
         is_in = false;
         // Problem: The method check_time_against_period is to a high
-        // degree not thread safe. In the current situation Icinga is 
+        // degree not thread safe. In the current situation Icinga is
         // very probable to hang up forever.
         // time_t now = time(0);
         // is_in = 0 == check_time_against_period(now, tp);
