@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -39,8 +39,9 @@
 #include <signal.h>
 #include <fcntl.h>
 
-#include "livestatus.h"
 #include "nagios.h"
+#include "livestatus.h"
+#include "downtime.h"
 #include "store.h"
 #include "logger.h"
 #include "config.h"
@@ -49,6 +50,8 @@
 #include "auth.h"
 #include "data_encoding.h"
 #include "waittriggers.h"
+#include "livechecking.h"
+
 
 #ifndef AF_LOCAL
 #define   AF_LOCAL AF_UNIX
@@ -60,16 +63,19 @@
 // usually - but not always - in sys/un.h
 #ifndef SUN_LEN
 # define SUN_LEN(ptr) ((size_t) (((struct sockaddr_un *) 0)->sun_path) + strlen ((ptr)->sun_path))
-#endif 
+#endif
 
 NEB_API_VERSION(CURRENT_NEB_API_VERSION)
 extern int event_broker_options;
+extern int enable_environment_macros;
 extern char *log_file;
 
 int g_idle_timeout_msec = 300 * 1000; /* maximum idle time for connection in keep alive state */
 int g_query_timeout_msec = 10 * 1000;      /* maximum time for reading a query */
 
-unsigned g_num_clientthreads = 10;     /* allow 10 concurrent connections per default */
+int g_num_clientthreads = 10;     /* allow 10 concurrent connections per default */
+int g_num_queued_connections = 0;     /* current number of queued connections (for statistics) */
+int g_num_active_connections = 0;     /* current number of active connections (for statistics) */
 size_t g_thread_stack_size = 65536; /* stack size of threads */
 
 #define false 0
@@ -81,6 +87,7 @@ int g_unix_socket = -1;
 int g_max_fd_ever = 0;
 char g_socket_path[4096];
 char g_pnp_path[4096];
+char g_livecheck_path[4096];
 char g_logfile_path[4096];
 int g_debug_level = 0;
 int g_should_terminate = false;
@@ -94,6 +101,39 @@ int g_service_authorization = AUTH_LOOSE;
 int g_group_authorization = AUTH_STRICT;
 int g_data_encoding = ENCODING_UTF8;
 
+/* simple statistics data for TableStatus */
+extern struct host *host_list;
+extern struct service *service_list;
+extern scheduled_downtime *scheduled_downtime_list;
+extern int log_initial_states;
+
+
+int g_num_hosts;
+int g_num_services;
+
+
+void count_hosts()
+{
+    g_num_hosts = 0;
+    host *h = (host *)host_list;
+    while (h) {
+        g_num_hosts ++;
+        h = h->next;
+    }
+}
+
+void count_services()
+{
+    g_num_services = 0;
+    service *s = (service *)service_list;
+    while (s) {
+        g_num_services ++;
+        s = s->next;
+    }
+}
+
+
+
 void* voidp;
 
 void livestatus_count_fork()
@@ -131,10 +171,11 @@ void livestatus_cleanup_after_fork()
     }
 }
 
+
 void *main_thread(void *data __attribute__ ((__unused__)))
 {
     g_thread_pid = getpid();
-    while (!g_should_terminate) 
+    while (!g_should_terminate)
     {
         do_statistics();
         if (g_thread_pid != getpid()) {
@@ -156,6 +197,7 @@ void *main_thread(void *data __attribute__ ((__unused__)))
             if (0 < fcntl(cc, F_SETFD, FD_CLOEXEC))
                 logger(LG_INFO, "Cannot set FD_CLOEXEC on client socket: %s", strerror(errno));
             queue_add_connection(cc); // closes fd
+            g_num_queued_connections++;
             g_counters[COUNTER_CONNECTIONS]++;
         }
     }
@@ -163,7 +205,6 @@ void *main_thread(void *data __attribute__ ((__unused__)))
     return voidp;
 }
 
-
 void *client_thread(void *data __attribute__ ((__unused__)))
 {
     void *input_buffer = create_inputbuffer(&g_should_terminate);
@@ -171,6 +212,8 @@ void *client_thread(void *data __attribute__ ((__unused__)))
 
     while (!g_should_terminate) {
         int cc = queue_pop_connection();
+        g_num_queued_connections--;
+        g_num_active_connections++;
         if (cc >= 0) {
             if (g_debug_level >= 2)
                 logger(LG_INFO, "Accepted client connection on fd %d", cc);
@@ -187,6 +230,7 @@ void *client_thread(void *data __attribute__ ((__unused__)))
             }
             close(cc);
         }
+        g_num_active_connections--;
     }
     delete_outputbuffer(output_buffer);
     delete_inputbuffer(input_buffer);
@@ -195,6 +239,9 @@ void *client_thread(void *data __attribute__ ((__unused__)))
 
 void start_threads()
 {
+    count_hosts();
+    count_services();
+
     if (!g_thread_running) {
         /* start thread that listens on socket */
         pthread_atfork(livestatus_count_fork, NULL, livestatus_cleanup_after_fork);
@@ -209,13 +256,13 @@ void start_threads()
         size_t defsize;
         if (g_debug_level >= 2 && 0 == pthread_attr_getstacksize(&attr, &defsize))
             logger(LG_INFO, "Default stack size is %lu", defsize);
-        if (0 != pthread_attr_setstacksize(&attr, g_thread_stack_size)) 
+        if (0 != pthread_attr_setstacksize(&attr, g_thread_stack_size))
             logger(LG_INFO, "Error: Cannot set thread stack size to %lu", g_thread_stack_size);
         else {
             if (g_debug_level >= 2)
                 logger(LG_INFO, "Setting thread stack size to %lu", g_thread_stack_size);
         }
-        for (t=0; t < g_num_clientthreads; t++) 
+        for (t=0; t < g_num_clientthreads; t++)
             pthread_create(&g_clientthread_id[t], &attr, client_thread, (void *)0);
 
         g_thread_running = 1;
@@ -239,6 +286,7 @@ void terminate_threads()
         if (g_debug_level > 0)
             logger(LG_INFO, "Main thread + %u client threads have finished", g_num_clientthreads);
         g_thread_running = 0;
+        g_should_terminate = false;
     }
     free(g_clientthread_id);
 }
@@ -318,13 +366,20 @@ int broker_host(int event_type __attribute__ ((__unused__)), void *data __attrib
 
 int broker_check(int event_type, void *data)
 {
+    int result = NEB_OK;
     if (event_type == NEBCALLBACK_SERVICE_CHECK_DATA) {
+        if (g_livecheck_enabled)
+            result = broker_service_livecheck(event_type, data);
+
         nebstruct_service_check_data *c = (nebstruct_service_check_data *)data;
         if (c->type == NEBTYPE_SERVICECHECK_PROCESSED) {
             g_counters[COUNTER_SERVICE_CHECKS]++;
         }
     }
     else if (event_type == NEBCALLBACK_HOST_CHECK_DATA) {
+        if (g_livecheck_enabled)
+            result = broker_host_livecheck(event_type, data);
+
         nebstruct_host_check_data *c = (nebstruct_host_check_data *)data;
         if (c->type == NEBTYPE_HOSTCHECK_PROCESSED) {
             g_counters[COUNTER_HOST_CHECKS]++;
@@ -332,7 +387,7 @@ int broker_check(int event_type, void *data)
     }
     pthread_cond_broadcast(&g_wait_cond[WT_ALL]);
     pthread_cond_broadcast(&g_wait_cond[WT_CHECK]);
-    return 0;
+    return result;
 }
 
 
@@ -358,6 +413,7 @@ int broker_downtime(int event_type __attribute__ ((__unused__)), void *data)
 
 int broker_log(int event_type __attribute__ ((__unused__)), void *data __attribute__ ((__unused__)))
 {
+
     g_counters[COUNTER_NEB_CALLBACKS]++;
     g_counters[COUNTER_LOG_MESSAGES]++;
     pthread_cond_broadcast(&g_wait_cond[WT_ALL]);
@@ -365,11 +421,10 @@ int broker_log(int event_type __attribute__ ((__unused__)), void *data __attribu
     return 0;
 }
 
-
 int broker_command(int event_type __attribute__ ((__unused__)), void *data)
 {
     nebstruct_external_command_data *sc = (nebstruct_external_command_data *)data;
-    if (sc->type == NEBTYPE_EXTERNALCOMMAND_START) 
+    if (sc->type == NEBTYPE_EXTERNALCOMMAND_START)
         g_counters[COUNTER_COMMANDS]++;
     g_counters[COUNTER_NEB_CALLBACKS]++;
     pthread_cond_broadcast(&g_wait_cond[WT_ALL]);
@@ -393,24 +448,87 @@ int broker_program(int event_type __attribute__ ((__unused__)), void *data __att
     return 0;
 }
 
+char* get_downtime_comment(char* host_name, char* svc_desc)
+{
+    char* comment;
+    int matches = 0;
+    scheduled_downtime* dt_list = scheduled_downtime_list;
+    while (dt_list != NULL) {
+        if (dt_list->type == HOST_DOWNTIME) {
+            if (strcmp(dt_list->host_name, host_name) == 0) {
+                matches++;
+                comment = dt_list->comment;
+            }
+        }
+        if (svc_desc != NULL && dt_list->type == SERVICE_DOWNTIME) {
+            if( strcmp(dt_list->host_name, host_name) == 0
+                    && strcmp(dt_list->service_description, svc_desc) == 0) {
+                matches++;
+                comment = dt_list->comment;
+            }
+        }
+        dt_list = dt_list->next;
+    }
+    return matches == 0 ? "No comment" : matches > 1 ? "Multiple Downtime Comments" : comment;
+}
+
+void livestatus_log_initial_states()
+{
+    // Log DOWNTIME hosts
+    host *h = (host *)host_list;
+    char buffer[8192];
+
+    while (h) {
+        if (h->scheduled_downtime_depth > 0) {
+            snprintf(buffer, sizeof(buffer), "HOST DOWNTIME ALERT: %s;STARTED;%s", h->name, get_downtime_comment(h->name, NULL));
+            write_to_all_logs(buffer, LG_INFO);
+        }
+        h = h->next;
+    }
+    // Log DOWNTIME services
+    service *s = (service *)service_list;
+    while (s) {
+        if (s->scheduled_downtime_depth > 0) {
+            snprintf(buffer, sizeof(buffer), "SERVICE DOWNTIME ALERT: %s;%s;STARTED;%s", s->host_name, s->description,
+                    get_downtime_comment(s->host_name, s->description));
+            write_to_all_logs(buffer, LG_INFO);
+        }
+        s = s->next;
+    }
+    // Log TIMERPERIODS
+    log_timeperiods_cache();
+}
+
+
 int broker_event(int event_type __attribute__ ((__unused__)), void *data)
 {
     g_counters[COUNTER_NEB_CALLBACKS]++;
     struct nebstruct_timed_event_struct *ts = (struct nebstruct_timed_event_struct *)data;
+    if (ts->event_type == EVENT_LOG_ROTATION)
+    {
+        if (g_thread_running == 1 )
+            livestatus_log_initial_states();
+        else if (log_initial_states == 1) // initi
+            write_to_all_logs("logging intitial states", LG_INFO); // initial info during startup
+    }
     update_timeperiods_cache(ts->timestamp.tv_sec);
     return 0;
 }
 
+
+
 int broker_process(int event_type __attribute__ ((__unused__)), void *data)
 {
     struct nebstruct_process_struct *ps = (struct nebstruct_process_struct *)data;
     if (ps->type == NEBTYPE_PROCESS_EVENTLOOPSTART) {
         update_timeperiods_cache(time(0));
+        init_livecheck();
         start_threads();
     }
     return 0;
 }
 
+
 int verify_event_broker_options()
 {
     int errors = 0;
@@ -515,17 +633,20 @@ void livestatus_parse_arguments(const char *args_orig)
     strncpy(g_socket_path, DEFAULT_SOCKET_PATH, sizeof(g_socket_path));
 
     /* set default path to our logfile to be in the same path as nagios.log */
-    strncpy(g_logfile_path, log_file, sizeof(g_logfile_path) - 16 /* len of "livestatus.log" */); 
+    strncpy(g_logfile_path, log_file, sizeof(g_logfile_path) - 16 /* len of "livestatus.log" */);
     char *slash = strrchr(g_logfile_path, '/');
     if (!slash)
         strcpy(g_logfile_path, "/tmp/livestatus.log");
-    else 
+    else
         strcpy(slash + 1, "livestatus.log");
 
     /* there is no default PNP path */
     g_pnp_path[0] = 0;
 
-    if (!args_orig) 
+    /* also livecheck is disabled per default */
+    g_livecheck_path[0] = 0;
+
+    if (!args_orig)
         return; // no arguments, use default options
 
     char *args = strdup(args_orig);
@@ -557,12 +678,12 @@ void livestatus_parse_arguments(const char *args_orig)
             }
             else if (!strcmp(left, "max_response_size")) {
                 g_max_response_size = strtoul(right, 0, 10);
-                logger(LG_INFO, "Setting maximum response size to %lu bytes (%.1f MB)", 
+                logger(LG_INFO, "Setting maximum response size to %lu bytes (%.1f MB)",
                         g_max_response_size, g_max_response_size / (1024.0*1024.0));
             }
             else if (!strcmp(left, "num_client_threads")) {
                 int c = atoi(right);
-                if (c <= 0 || c > 1000) 
+                if (c <= 0 || c > 1000)
                     logger(LG_INFO, "Error: Cannot set num_client_threads to %d. Must be > 0 and <= 1000", c);
                 else {
                     logger(LG_INFO, "Setting number of client threads to %d", c);
@@ -589,7 +710,7 @@ void livestatus_parse_arguments(const char *args_orig)
                     g_idle_timeout_msec = c;
                     if (c == 0)
                         logger(LG_INFO, "Disabled idle timeout!");
-                    else 
+                    else
                         logger(LG_INFO, "Setting idle timeout to %d ms", c);
                 }
             }
@@ -628,11 +749,22 @@ void livestatus_parse_arguments(const char *args_orig)
                     logger(LG_INFO, "Invalid data_encoding %s. Allowed are utf8, latin1 and mixed.", right);
                 }
             }
+            else if (!strcmp(left, "livecheck")) {
+                strncpy(g_livecheck_path, right, sizeof(g_livecheck_path) - 1);
+                if (0 != access(g_livecheck_path, X_OK)) {
+                    logger(LG_INFO, "ERROR: %s is not executable. Disabling livecheck.", g_livecheck_path);
+                    g_livecheck_path[0] = 0;
+                }
+                g_livecheck_enabled = true;
+            }
+            else if (!strcmp(left, "num_livecheck_helpers")) {
+                g_num_livehelpers = atoi(right);
+            }
             else {
                 logger(LG_INFO, "Ignoring invalid option %s=%s", left, right);
             }
         }
-    } 
+    }
     // free(args); won't free, since we use pointers?
 }
 
@@ -651,7 +783,7 @@ void omd_advertize()
 
 
 /* this function gets called when the module is loaded by the event broker */
-int nebmodule_init(int flags __attribute__ ((__unused__)), char *args, void *handle) 
+int nebmodule_init(int flags __attribute__ ((__unused__)), char *args, void *handle)
 {
     g_nagios_handle = handle;
     livestatus_parse_arguments(args);
@@ -671,7 +803,10 @@ int nebmodule_init(int flags __attribute__ ((__unused__)), char *args, void *han
         return 1;
     }
     else if (g_debug_level > 0)
-        logger(LG_INFO, "Your event_broker_options are sufficient for livestatus.");
+        logger(LG_INFO, "Your event_broker_options are sufficient for livestatus..");
+
+    if (enable_environment_macros == 1)
+        logger(LG_INFO, "Warning: environment_macros are enabled. This might decrease the overall nagios performance");
 
     store_init();
     register_callbacks();
@@ -694,6 +829,7 @@ int nebmodule_deinit(int flags __attribute__ ((__unused__)), int reason __attrib
     close_unix_socket();
     store_deinit();
     deregister_callbacks();
+    deinit_livecheck();
     close_logfile();
     return 0;
 }
