@@ -5,7 +5,7 @@
 // |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 // |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 // |                                                                  |
-// | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+// | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 // +------------------------------------------------------------------+
 //
 // This file is part of Check_MK.
@@ -27,7 +27,7 @@
 
 #include "config.h"
 
-#ifdef __cplusplus 
+#ifdef __cplusplus
 extern "C"
 {
 #endif
@@ -47,6 +47,8 @@ extern "C"
     int  queue_pop_connection();
     void queue_wakeup_all();
     void update_timeperiods_cache(time_t);
+    void log_timeperiods_cache();
+
 
 #ifdef __cplusplus
 }
