@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -26,36 +26,47 @@
 
 import bi
 
-#     ____        _                                          
-#    |  _ \  __ _| |_ __ _ ___  ___  _   _ _ __ ___ ___  ___ 
+#     ____        _
+#    |  _ \  __ _| |_ __ _ ___  ___  _   _ _ __ ___ ___  ___
 #    | | | |/ _` | __/ _` / __|/ _ \| | | | '__/ __/ _ \/ __|
 #    | |_| | (_| | || (_| \__ \ (_) | |_| | | | (_|  __/\__ \
 #    |____/ \__,_|\__\__,_|___/\___/ \__,_|_|  \___\___||___/
-#                                                            
+#
 
 
 multisite_datasources["bi_aggregations"] = {
     "title"       : _("BI Aggregations"),
-    "table"       : bi.table, 
+    "table"       : bi.table,
     "infos"       : [ "aggr" ],
     "keys"        : [],
     "idkeys"      : [ 'aggr_name' ],
 }
 
 multisite_datasources["bi_host_aggregations"] = {
-    "title"       : _("BI Host Aggregations"),
-    "table"       : bi.host_table, 
+    "title"       : _("BI Aggregations affected by one host"),
+    "table"       : bi.host_table,
     "infos"       : [ "host", "aggr" ],
     "keys"        : [],
     "idkeys"      : [ 'aggr_name' ],
 }
 
-#     ____       _       _                
-#    |  _ \ __ _(_)_ __ | |_ ___ _ __ ___ 
+# Similar to host aggregations, but the name of the aggregation
+# is used to join the host table rather then the affected host
+multisite_datasources["bi_hostname_aggregations"] = {
+    "title"       : _("BI Hostname Aggregations"),
+    "table"       : bi.hostname_table,
+    "infos"       : [ "host", "aggr" ],
+    "keys"        : [],
+    "idkeys"      : [ 'aggr_name' ],
+}
+
+
+#     ____       _       _
+#    |  _ \ __ _(_)_ __ | |_ ___ _ __ ___
 #    | |_) / _` | | '_ \| __/ _ \ '__/ __|
 #    |  __/ (_| | | | | | ||  __/ |  \__ \
 #    |_|   \__,_|_|_| |_|\__\___|_|  |___/
-#                                         
+#
 
 def paint_aggr_state_short(state, assumed = False):
     if state == None:
@@ -74,6 +85,13 @@ multisite_painters["aggr_state"] = {
     "paint"   : lambda row: paint_aggr_state_short(row["aggr_effective_state"], row["aggr_effective_state"] != row["aggr_state"])
 }
 
+multisite_painters["aggr_state_num"] = {
+    "title"   : _("Aggregated state (number)"),
+    "short"   : _("State"),
+    "columns" : [ "aggr_effective_state" ],
+    "paint"   : lambda row: ("", str(row["aggr_effective_state"]['state']))
+}
+
 multisite_painters["aggr_real_state"] = {
     "title"   : _("Aggregated real state (never assumed)"),
     "short"   : _("R.State"),
@@ -110,10 +128,10 @@ multisite_painters["aggr_output"] = {
     "paint"   : lambda row: ("", row["aggr_output"])
 }
 
-def paint_aggr_hosts(row):
+def paint_aggr_hosts(row, link_to_view):
     h = []
     for site, host in row["aggr_hosts"]:
-        url = html.makeuri([("view_name", "aggr_host"), ("site", site), ("host", host)])
+        url = html.makeuri([("view_name", link_to_view), ("site", site), ("host", host)])
         h.append('<a href="%s">%s</a>' % (url, host))
     return "", " ".join(h)
 
@@ -121,9 +139,15 @@ multisite_painters["aggr_hosts"] = {
     "title"   : _("Aggregation: affected hosts"),
     "short"   : _("Hosts"),
     "columns" : [ "aggr_hosts" ],
-    "paint"   : paint_aggr_hosts,
+    "paint"   : lambda row: paint_aggr_hosts(row, "aggr_host"),
 }
 
+multisite_painters["aggr_hosts_services"] = {
+    "title"   : _("Aggregation: affected hosts (link to host page)"),
+    "short"   : _("Hosts"),
+    "columns" : [ "aggr_hosts" ],
+    "paint"   : lambda row: paint_aggr_hosts(row, "host"),
+}
 
 multisite_painter_options["aggr_expand"] = {
  "title"   : _("Initial expansion of aggregations"),
@@ -140,7 +164,12 @@ multisite_painter_options["aggr_onlyproblems"] = {
 multisite_painter_options["aggr_treetype"] = {
  "title"   : _("Type of tree layout"),
  "default" : "foldable",
- "values"  : [ ("foldable", "foldable"), ("bottom-up", "bottom up"), ("top-down", "top down")]
+ "values"  : [
+    ("foldable",     _("foldable")),
+    ("boxes",        _("boxes")),
+    ("boxes-omit-root", _("boxes (omit root)")),
+    ("bottom-up",    _("bottom up")),
+    ("top-down",     _("top down"))]
 }
 
 multisite_painter_options["aggr_wrap"] = {
@@ -150,166 +179,7 @@ multisite_painter_options["aggr_wrap"] = {
 }
 
 
-def render_bi_state(state):
-    return { bi.PENDING: _("PD"),
-             bi.OK:      _("OK"),
-             bi.WARN:    _("WA"),
-             bi.CRIT:    _("CR"),
-             bi.UNKNOWN: _("UN"),
-             bi.MISSING: _("MI"),
-             bi.UNAVAIL: _("NA"),
-    }.get(state, _("??"))
 
-def render_assume_icon(site, host, service):
-    if service:
-        key = (site, host, service)
-    else:
-        key = (site, host)
-    ass = bi.g_assumptions.get(key)
-    # TODO: Non-Ascii-Characters do not work yet!
-    mousecode = \
-       u'onmouseover="this.style.cursor=\'pointer\';" ' \
-       'onmouseout="this.style.cursor=\'auto\';" ' \
-       'title="%s" ' \
-       'onclick="toggle_assumption(this, \'%s\', \'%s\', \'%s\');" ' % \
-         (_("Assume another state for this item (reload page to activate)"),
-         # MIST: DAS HIER MUSS verfünftig für Javascript encodiert werden.
-         # Das Ausgangsmaterial sind UTF-8 kodierte str-Objekte.
-          site, host, service != None and service or '')
-    current = str(ass).lower()
-    return u'<img state="%s" class=assumption %s src="images/assume_%s.png">\n' % (current, mousecode, current)
-
-def aggr_render_leaf(tree, show_host):
-    site, host = tree[2]["host"]
-    service = tree[2].get("service")
-    content = u"" + render_assume_icon(site, host, service) 
-
-    # Four cases:
-    # (1) zbghora17 . Host status   (show_host == True, service == None)
-    # (2) zbghora17 . CPU load      (show_host == True, service != None)
-    # (3) Host Status               (show_host == False, service == None)
-    # (4) CPU load                  (show_host == False, service != None)
-
-    if show_host or not service:
-        host_url = html.makeuri([("view_name", "hoststatus"), ("site", site), ("host", host)])
-
-    if service:
-        service_url = html.makeuri([("view_name", "service"), ("site", site), ("host", host), ("service", service)])
-
-    if show_host:
-        content += '<a href="%s">%s</a><b class=bullet>&diams;</b>' % (host_url, host)
-
-    if not service:
-        content += '<a href="%s">%s</a>' % (host_url, _("Host status"))
-    else:
-        content += '<a href="%s">%s</a>' % (service_url, service)
-
-    return aggr_render_node(tree, content, None, show_host)
-
-def aggr_render_node(tree, title, mousecode, show_host):
-    state = tree[0]
-    assumed_state = tree[1]
-    if assumed_state != None:
-        effective_state = assumed_state 
-    else:
-        effective_state = state
-
-    if (effective_state != state):
-        addclass = " " + _("assumed")
-    else:
-        addclass = ""
-
-    h = '<span class="content state state%d%s">%s</span>\n' \
-         % (effective_state["state"], addclass, render_bi_state(effective_state["state"]))
-    if mousecode:
-        h += '<img class=opentree %s>' % mousecode
-        h += '<span class="content name">%s</span>' % title
-    else:
-        h += title
-
-    output = effective_state["output"]
-    if output:
-        output = "<b class=bullet>&diams;</b>" + output
-    else:
-        output = ""
-    h += '<span class="content output">%s</span>\n' % output
-    return h
-
-
-# Convert tree to tree contain only node in non-OK state
-def filter_tree_only_problems(tree):
-    state, assumed_state, node, subtrees = tree
-    # remove subtrees in state OK
-    new_subtrees = []
-    for subtree in subtrees:
-        effective_state = subtree[1] != None and subtree[1] or subtree[0]
-        if effective_state["state"] != bi.OK:
-            if len(subtree) == 3:
-                new_subtrees.append(subtree)
-            else:
-                new_subtrees.append(filter_tree_only_problems(subtree))
-
-    return state, assumed_state, node, new_subtrees
-
-
-def paint_aggr_tree_foldable(row):
-    saved_expansion_level = bi.load_ex_level()
-    treestate = weblib.get_tree_states('bi')
-    expansion_level = int(get_painter_option("aggr_expand"))
-    if expansion_level != saved_expansion_level:
-        treestate = {}
-        weblib.set_tree_states('bi', treestate)
-        weblib.save_tree_states()
-
-    mousecode = \
-       'onmouseover="this.style.cursor=\'pointer\';" ' \
-       'onmouseout="this.style.cursor=\'auto\';" ' \
-       'onclick="toggle_subtree(this);" '
-
-
-    def render_subtree(tree, path, show_host):
-        if len(tree) == 3: # leaf
-            return aggr_render_leaf(tree, show_host)
-        else:
-            h = '<span class=title>'
-
-            path_id = "/".join(path)
-            is_open = treestate.get(path_id)
-            if is_open == None:
-                is_open = len(path) <= expansion_level
-
-            is_empty = len(tree[3]) == 0
-
-            if is_empty:
-                style = ''
-                mc = ''
-            elif is_open:
-                style = ''
-                mc = mousecode + 'src="images/tree_open.png" '
-            else:
-                style = 'style="display: none" '
-                mc = mousecode + 'src="images/tree_00.png" '
-
-            h += aggr_render_node(tree, tree[2]["title"], mc, show_host)
-            if not is_empty:
-                h += '<ul id="%d:%s" %sclass="subtree">' % (expansion_level, path_id, style)
-
-                for node in tree[3]:
-                    estate = node[1] != None and node[1] or node[0]
-
-                    if not node[2].get("hidden"):
-                        new_path = path + [node[2]["title"]]
-                        h += '<li>' + render_subtree(node, new_path, show_host) + '</li>\n'
-                h += '</ul>'
-            return h + '</span>\n'
-
-    tree = row["aggr_treestate"]
-    if get_painter_option("aggr_onlyproblems") == "1":
-        tree = filter_tree_only_problems(tree)
-
-    affected_hosts = row["aggr_hosts"]
-    htmlcode = render_subtree(tree, [tree[2]["title"]], len(affected_hosts) > 1)
-    return "aggrtree", htmlcode
 
 
 def paint_aggr_tree_ltr(row, mirror):
@@ -321,30 +191,30 @@ def paint_aggr_tree_ltr(row, mirror):
         td = '<td style="white-space: nowrap;"'
 
     def gen_table(tree, height, show_host):
-        if len(tree) == 3: 
+        if len(tree) == 3:
             return gen_leaf(tree, height, show_host)
         else:
             return gen_node(tree, height, show_host)
 
     def gen_leaf(tree, height, show_host):
-        return [(aggr_render_leaf(tree, show_host), height, [])]
+        return [(bi.aggr_render_leaf(tree, show_host), height, [])]
 
     def gen_node(tree, height, show_host):
         leaves = []
         for node in tree[3]:
             if not node[2].get("hidden"):
                 leaves += gen_table(node, height - 1, show_host)
-        h = '<div class="aggr tree">' + aggr_render_node(tree, tree[2]["title"], '', show_host) + "</div>"
+        h = '<div class="aggr tree">' + bi.aggr_render_node(tree, tree[2]["title"], '', show_host) + "</div>"
         if leaves:
             leaves[0][2].append((len(leaves), h))
         return leaves
 
     tree = row["aggr_treestate"]
     if get_painter_option("aggr_onlyproblems") == "1":
-        tree = filter_tree_only_problems(tree)
+        tree = bi.filter_tree_only_problems(tree)
     depth = bi.status_tree_depth(tree)
     leaves = gen_table(tree, depth, row["aggr_hosts"] > 1)
-    h = '<table class="aggrtree">'
+    h = '<table class="aggrtree ltr">'
     odd = "odd"
     for code, colspan, parents in leaves:
         h += '<tr>\n'
@@ -367,11 +237,17 @@ def paint_aggr_tree_ltr(row, mirror):
 
 def paint_aggregated_tree_state(row):
     treetype = get_painter_option("aggr_treetype")
+    expansion_level = int(get_painter_option("aggr_expand"))
+    only_problems = get_painter_option("aggr_onlyproblems") == "1"
     if treetype == "foldable":
-        return paint_aggr_tree_foldable(row)
+        return bi.render_tree_foldable(row,  False,  False, expansion_level, only_problems, lazy=True)
+    elif treetype == "boxes":
+        return bi.render_tree_foldable(row,  True, False, expansion_level, only_problems, lazy=True)
+    elif treetype == "boxes-omit-root":
+        return bi.render_tree_foldable(row,  True, True, expansion_level, only_problems, lazy=True)
     elif treetype == "bottom-up":
         return paint_aggr_tree_ltr(row, False)
-    else:
+    elif treetype == "top-down":
         return paint_aggr_tree_ltr(row, True)
 
 multisite_painters["aggr_treestate"] = {
@@ -382,13 +258,20 @@ multisite_painters["aggr_treestate"] = {
     "paint"   : paint_aggregated_tree_state,
 }
 
+multisite_painters["aggr_treestate_boxed"] = {
+    "title"   : _("Aggregation: simplistic boxed layout"),
+    "short"   : _("Tree"),
+    "columns" : [ "aggr_treestate", "aggr_hosts" ],
+    "paint"   : lambda row: bi.render_tree_foldable(row, boxes=True, omit_root=True,
+                expansion_level=bi.load_ex_level(), only_problems=False, lazy=True),
+}
 
-#     _____ _ _ _                
-#    |  ___(_) | |_ ___ _ __ ___ 
+#     _____ _ _ _
+#    |  ___(_) | |_ ___ _ __ ___
 #    | |_  | | | __/ _ \ '__/ __|
 #    |  _| | | | ||  __/ |  \__ \
 #    |_|   |_|_|\__\___|_|  |___/
-#                                
+#
 
 class BIGroupFilter(Filter):
     def __init__(self):
@@ -399,9 +282,8 @@ class BIGroupFilter(Filter):
         return [ (self.htmlvars[0], row[self.column]) ]
 
     def display(self):
-        bi.compile_forest(html.req.user)
         htmlvar = self.htmlvars[0]
-        html.select(htmlvar, [ ("", "") ] + [(g,g) for g in bi.g_user_cache["forest"].keys()])
+        html.select(htmlvar, [ ("", "") ] + [(g, g) for g in bi.aggregation_groups()])
 
     def selected_group(self):
         return html.var(self.htmlvars[0])
@@ -480,6 +362,9 @@ class BIServiceFilter(Filter):
     def __init__(self):
         Filter.__init__(self, "aggr_service", _("Affected by service"), "aggr", ["site", "host", "service"], [])
 
+    def double_height(self):
+        return True
+
     def display(self):
         html.write(_("Host") + ": ")
         html.text_input("host")
@@ -515,17 +400,22 @@ class BIStatusFilter(Filter):
     def filter(self, tablename):
         return ""
 
+    def double_height(self):
+        return self.column == "aggr_assumed_state"
+
     def display(self):
         if html.var("filled_in"):
             defval = ""
         else:
             defval = "on"
-        for varend, text in [('0', 'OK'), ('1', 'WARN'), ('2', 'CRIT'), ('3', 'UNKNOWN'), ('-1', 'PENDING'), ('n', 'unset')]:
+        for varend, text in [('0', 'OK'), ('1', 'WARN'), ('2', 'CRIT'),
+                             ('3', 'UNKN'), ('-1', 'PENDING'), ('n', _('no assumed state set'))]:
             if self.code != 'a' and varend == 'n':
                 continue # no unset for read and effective state
+            if varend == 'n':
+                html.write("<br>")
             var = self.prefix + varend
-            html.checkbox(var, defval)
-            html.write(" %s " % text)
+            html.checkbox(var, defval, label = text)
 
     def filter_table(self, rows):
         jeaders = []
