@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -42,7 +42,7 @@ class FilterText(Filter):
         htmlvar = self.htmlvars[0]
         current_value = html.var(htmlvar)
         if current_value:
-            return _("Filter: %s %s %s\n") % (self.column, self.op, current_value)
+            return "Filter: %s %s %s\n" % (self.column, self.op, current_value)
         else:
             return ""
 
@@ -56,23 +56,58 @@ class FilterText(Filter):
 declare_filter(100, FilterText("hostregex",    _("Hostname"),        "host",    "host_name",      "host",    "~~"),
                           _("Search field allowing regular expressions and partial matches"))
 
-declare_filter(101, FilterText("host",    _("Hostname"),             "host",    "host_name",          "host",    "="),
+declare_filter(101, FilterText("host",    _("Hostname (exact match)"),             "host",    "host_name",          "host",    "="),
                           _("Exact match, used for linking"))
 
 declare_filter(200, FilterText("serviceregex", _("Service"),         "service", "service_description",   "service", "~~"),
                           _("Search field allowing regular expressions and partial matches"))
 
-declare_filter(201, FilterText("service", _("Service"),              "service", "service_description",   "service", "="),
+declare_filter(201, FilterText("service", _("Service (exact match)"),              "service", "service_description",   "service", "="),
                           _("Exact match, used for linking"))
 
-declare_filter(100, FilterText("servicegroupname", _("Servicegroup"),   "servicegroup", "servicegroup_name",   "servicegroup_name", "="),
-                          _("Exact match, used for linking"))
 
 declare_filter(101, FilterText("servicegroupnameregex", _("Servicegroup"),   "servicegroup", "servicegroup_name",   "servicegroup_name", "~~"),
                           _("Search field allowing regular expression and partial matches"))
 
-declare_filter(202, FilterText("output",  _("Service check output"), "service", "service_plugin_output", "service_output", "~~"))
+declare_filter(101, FilterText("servicegroupname", _("Servicegroup (enforced)"),   "servicegroup", "servicegroup_name",   "servicegroup_name", "="),
+                          _("Exact match, used for linking"))
 
+declare_filter(202, FilterText("output",  _("Status detail"), "service", "service_plugin_output", "service_output", "~~"))
+
+
+
+class FilterIPAddress(Filter):
+    def __init__(self):
+        Filter.__init__(self, "host_address", _("Host IP Address"), "host", ["host_address", "host_address_prefix"], ["host_address"])
+
+    def display(self):
+        html.text_input("host_address")
+        html.write("<br><br>")
+        html.begin_radio_group()
+        html.radiobutton("host_address_prefix", "yes", True, _("Prefix match"))
+        html.radiobutton("host_address_prefix", "no", False, _("Exact match"))
+        html.end_radio_group()
+
+    def double_height(self):
+        return True
+
+    def filter(self, infoname):
+        address = html.var("host_address")
+        if address:
+            if html.var("host_address_prefix") == "yes":
+                return "Filter: host_address ~ ^%s\n" % address
+            else:
+                return "Filter: host_address = %s\n" % address
+        else:
+            return ""
+
+    def variable_settings(self, row):
+        return [ ("host_address", row["host_address"]) ]
+
+    def heading_info(self, infoname):
+        return html.var("host_address")
+
+declare_filter(102, FilterIPAddress())
 
 
 # Helper that retrieves the list of host/service/contactgroups via Livestatus
@@ -80,17 +115,18 @@ def all_groups(what):
     groups = dict(html.live.query("GET %sgroups\nColumns: name alias\n" % what))
     names = groups.keys()
     names.sort()
-    return [ (name, groups[name]) for name in names ]
+    # use alias by default but fallback to name if no alias defined
+    return [ (name, groups[name] or name) for name in names ]
 
 class FilterGroupCombo(Filter):
-    def __init__(self, what, enforce):
+    def __init__(self, what, title, enforce):
         self.enforce = enforce
         self.prefix = not self.enforce and "opt" or ""
         htmlvars = [ self.prefix + what + "group" ]
         if not enforce:
             htmlvars.append("neg_" + htmlvars[0])
         Filter.__init__(self, self.prefix + what + "group", # name,     e.g. "hostgroup"
-                what[0].upper() + what[1:].replace("_"," ") + " group",       # title,    e.g. "Hostgroup"
+                title,                                      # title,    e.g. "Hostgroup"
                 what.split("_")[0],                         # info,     e.g. "host"
                 htmlvars,                                   # htmlvars, e.g. "hostgroup"
                 [ what + "group_name" ])                    # rows needed to fetch for link information
@@ -103,8 +139,7 @@ class FilterGroupCombo(Filter):
         html.select(self.htmlvars[0], choices)
         if not self.enforce:
             html.write(" <nobr>")
-            html.checkbox(self.htmlvars[1])
-            html.write(_("negate"))
+            html.checkbox(self.htmlvars[1], label=_("negate"))
             html.write("</nobr>")
 
     def current_value(self, infoname):
@@ -150,12 +185,17 @@ class FilterGroupCombo(Filter):
             return alias
 
 
-declare_filter(104, FilterGroupCombo("host",            True),  _("Dropdown list, selection of host group is <b>enforced</b>"))
-declare_filter(204, FilterGroupCombo("service",         True),  _("Dropdown list, selection of service group is <b>enforced</b>"))
-declare_filter(105, FilterGroupCombo("host",            False), _("Optional selection of host group"))
-declare_filter(205, FilterGroupCombo("service",         False), _("Optional selection of service group"))
-declare_filter(106, FilterGroupCombo("host_contact",    False), _("Optional selection of host contact group group"))
-declare_filter(206, FilterGroupCombo("service_contact", False), _("Optional selection of service contact group group"))
+declare_filter(104, FilterGroupCombo("host",            _("Hostgroup"),            False), _("Optional selection of host group"))
+declare_filter(104, FilterGroupCombo("host",            _("Hostgroup (enforced)"),            True),  _("Dropdown list, selection of host group is <b>enforced</b>"))
+declare_filter(204, FilterGroupCombo("service",         _("Servicegroup"),         False), _("Optional selection of service group"))
+declare_filter(205, FilterGroupCombo("service",         _("Servicegroup (enforced)"),         True),  _("Dropdown list, selection of service group is <b>enforced</b>"))
+declare_filter(106, FilterGroupCombo("host_contact",    _("Host Contactgroup"),    False), _("Optional selection of host contact group group"))
+declare_filter(206, FilterGroupCombo("service_contact", _("Service Contactgroup"), False), _("Optional selection of service contact group group"))
+
+declare_filter(107, FilterText("host_ctc", _("Host Contact"), "host", "host_contacts", "host_ctc", ">="))
+declare_filter(207, FilterText("service_ctc", _("Service Contact"), "service", "service_contacts", "service_ctc", ">="))
+
+
 # Livestatus still misses "contact_groups" column.
 # declare_filter(FilterGroupCombo("contact"))
 
@@ -188,37 +228,33 @@ class FilterServiceState(Filter):
         self.prefix = prefix
 
     def display(self):
-        if html.var("filled_in"):
-            defval = ""
-        else:
-            defval = "on"
+        html.begin_checkbox_group()
         for var, text in [(self.prefix + "st0", "OK"), (self.prefix + "st1", "WARN"), \
-                          (self.prefix + "st2", "CRIT"), (self.prefix + "st3", "UNKNOWN"), (self.prefix + "stp", "PENDING")]:
-            html.checkbox(var, defval)
-            html.write(" %s " % text)
+                          (self.prefix + "st2", "CRIT"), (self.prefix + "st3", "UNKNOWN"),
+                          (self.prefix + "stp", "PEND.")]:
+	    #if html.mobile:
+	        #text = text[:1]
+            html.checkbox(var, True, label=text)
+            # html.write(" %s " % text)
+        html.end_checkbox_group()
 
     def filter(self, infoname):
         headers = []
-        if html.var("filled_in"):
-            defval = ""
-        else:
-            defval = "on"
-
         for i in [0,1,2,3]:
-            if html.var(self.prefix + "st%d" % i, defval) == "on":
+            if html.get_checkbox(self.prefix + "st%d" % i) == False:
                 if self.prefix == "hd":
                     column = "service_last_hard_state"
                 else:
                     column = "service_state"
-                headers.append("Filter: %s = %d\nFilter: service_has_been_checked = 1\nAnd: 2\n" % (column, i))
-        if html.var(self.prefix + "stp", defval) == "on":
-            headers.append("Filter: service_has_been_checked = 0\n")
-        if len(headers) == 0:
-            return "Limit: 0\n" # no allowed state
-        elif len(headers) == 5: # all states allowed
+                headers.append("Filter: %s = %d\n"
+                               "Filter: service_has_been_checked = 1\n"
+                               "And: 2\nNegate:\n" % (column, i))
+        if html.get_checkbox(self.prefix + "stp") == False:
+            headers.append("Filter: service_has_been_checked = 1\n")
+        if len(headers) == 5: # none allowed = all allowed (makes URL building easier)
             return ""
         else:
-            return "".join(headers) + ("Or: %d\n" % len(headers))
+            return "".join(headers)
 
 declare_filter(215, FilterServiceState("svcstate",     _("Service states"),      ""))
 declare_filter(216, FilterServiceState("svchardstate", _("Service hard states"), "hd"))
@@ -229,30 +265,25 @@ class FilterHostState(Filter):
                 "host", [ "hst0", "hst1", "hst2", "hstp" ], [])
 
     def display(self):
-        if html.var("filled_in"):
-            defval = ""
-        else:
-            defval = "on"
-        for var, text in [("hst0", _("UP")), ("hst1", _("DOWN")), ("hst2", _("UNREACH")), ("hstp", _("PENDING"))]:
-            html.checkbox(var, defval)
-            html.write(" %s " % text)
+        html.begin_checkbox_group()
+        for var, text in [("hst0", _("UP")), ("hst1", _("DOWN")),
+                          ("hst2", _("UNREACH")), ("hstp", _("PENDING"))]:
+            html.checkbox(var, True, label=text)
+        html.end_checkbox_group()
 
     def filter(self, infoname):
         headers = []
-        if html.var("filled_in"):
-            defval = ""
-        else:
-            defval = "on"
-
         for i in [0,1,2]:
-            if html.var("hst%d" % i, defval) == "on":
-                headers.append("Filter: host_state = %d\nFilter: host_has_been_checked = 1\nAnd: 2\n" % i)
-        if html.var("stp", defval) == "on":
-            headers.append("Filter: host_has_been_checked = 0\n")
-        if len(headers) == 0:
-            return "Limit: 0\n" # not allowed state
+            if html.get_checkbox("hst%d" % i) == False:
+                headers.append("Filter: host_state = %d\n"
+                               "Filter: host_has_been_checked = 1\n"
+                               "And: 2\nNegate:\n" % i)
+        if html.get_checkbox("hstp") == False:
+            headers.append("Filter: host_has_been_checked = 1\n")
+        if len(headers) == 4: # none allowed = all allowed (makes URL building easier)
+            return ""
         else:
-            return "".join(headers) + ("Or: %d\n" % len(headers))
+            return "".join(headers)
 
 declare_filter(115, FilterHostState())
 
@@ -265,9 +296,11 @@ class FilterTristate(Filter):
 
     def display(self):
         current = html.var(self.varname)
+        html.begin_radio_group(horizontal = True)
         for value, text in [("1", _("yes")), ("0", _("no")), ("-1", _("(ignore)"))]:
             checked = current == value or (current in [ None, ""] and int(value) == self.deflt)
             html.radiobutton(self.varname, value, checked, text + " &nbsp; ")
+        html.end_radio_group()
 
     def tristate_value(self):
         current = html.var(self.varname)
@@ -330,9 +363,12 @@ declare_filter(232, FilterNagiosExpression("service", "in_downtime",
 
 class FilterSite(Filter):
     def __init__(self, name, enforce):
-        Filter.__init__(self, name, _("Site"), None, ["site"], [])
+        Filter.__init__(self, name, _("Site") + (enforce and _( " (enforced)") or ""), None, ["site"], [])
         self.enforce = enforce
 
+    def visible(self):
+        return config.is_multisite()
+
     def display(self):
         site_selector(html, "site", self.enforce)
 
@@ -340,11 +376,11 @@ class FilterSite(Filter):
         if config.is_multisite():
             site = html.var("site")
             if site:
-                return "%s: %s\n" % (_('Sites'), html.var("site", ""))
+                return "Sites: %s\n" % (html.var("site", ""))
             elif not self.enforce:
                 return ""
             else:
-                return "%s:\n" % _('Sites') # no site at all
+                return "Sites:\n" # no site at all
         else:
             return ""
 
@@ -357,8 +393,8 @@ class FilterSite(Filter):
     def variable_settings(self, row):
         return [("site", row["site"])]
 
-declare_filter(500, FilterSite("site",    True),  _("Selection of site is enforced, use this filter for joining"))
-declare_filter(501, FilterSite("siteopt", False), _("Optional selection of a site"))
+declare_filter(500, FilterSite("siteopt", False), _("Optional selection of a site"))
+declare_filter(501, FilterSite("site",    True),  _("Selection of site is enforced, use this filter for joining"))
 
 # Filter for setting time ranges, e.g. on last_state_change and last_check
 # Variante eins:
@@ -371,73 +407,96 @@ class FilterTime(Filter):
     def __init__(self, info, name, title, column):
         self.column = column
         self.name = name
-        self.ranges = [ (1, _("sec")), (60, _("min")), (3600, _("hours")), (86400, _("days")) ]
-        Filter.__init__(self, name, title, info, [ name ] + [ name + "_" + n for (s, n) in self.ranges], [column])
+        self.ranges = [
+           (86400, _("days")),
+           (3600,  _("hours")),
+           (60,    _("min")),
+           (1,     _("sec")),
+        ]
+        varnames = [ name + "_from", name + "_from_range",
+                     name + "_until", name + "_until_range" ]
+
+        Filter.__init__(self, name, title, info, varnames, [column])
+
+    def double_height(self):
+        return True
 
     def display(self):
-        for s, n in self.ranges:
-            htmlvar = self.name + "_" + n
-            html.write("<nobr>")
-            html.number_input(htmlvar, 0, 2)
-            html.write(" %s</nobr> " % n)
-        if config.filter_columns > 1:
-            html.write("<br>\n")
-        current = html.var(self.name, "since")
-        for t in [ "before", "since" ]:
-            html.radiobutton(self.name, t, current == t, t + " &nbsp; ")
-            html.write(" ")
+        choices = [ (str(sec), title + " " + _("ago")) for sec, title in self.ranges ] + \
+                  [ ("abs", _("Date (YYYY-MM-DD)")) ]
+
+        html.write("<table class=filtertime>")
+        for what, whatname in [
+            ( "from", _("From") ),
+            ( "until", _("Until") ) ]:
+            varprefix = self.name + "_" + what
+            html.write("<tr><td>%s:</td>" % whatname)
+            html.write("<td>")
+            html.text_input(varprefix, style="width: 116px;")
+            html.write("</td><td>")
+            html.select(varprefix + "_range", choices, "3600")
+            html.write("</td></tr>")
+        html.write("</table>")
+
 
     def filter(self, infoname):
-        range = self.get_time_range()
-        if range == None:
-            return ""
-
-        timestamp, negate = range
-        if negate:
-            neg = "!"
-        else:
-            neg = ""
-
-        return "Filter: %s %s>= %d\n" % (self.column, neg, timestamp)
+        fromsecs, untilsecs = self.get_time_range()
+        filtertext = ""
+        if fromsecs != None:
+            filtertext += "Filter: %s >= %d\n" % (self.column, fromsecs)
+        if untilsecs != None:
+            filtertext += "Filter: %s <= %d\n" % (self.column, untilsecs)
+        return filtertext
 
 
     # Extract timerange user has selected from HTML variables
     def get_time_range(self):
-        secs = 0
-        for s, n in self.ranges:
-            htmlvar = self.name + "_" + n
-            v = html.var(htmlvar)
-            if v:
-                try:
-                    secs += int(v) * s
-                except:
-                    pass
+        range = []
+        for what in [ "from", "until" ]:
+            varprefix = self.name + "_" + what
+            count = html.var(varprefix)
+            if count == "":
+                range.append(None)
+            else:
+                rangename = html.var(varprefix + "_range")
+                if rangename == "abs":
+                    try:
+                        range.append(time.mktime(time.strptime(count, "%Y-%m-%d")))
+                    except:
+                        html.add_user_error(varprefix, _("Please enter the date in the format YYYY-MM-DD."))
+                        range.append(None)
+                else:
+                    try:
+                        count = int(count)
+                        secs = count * int(rangename)
+                        range.append(int(time.time()) - secs)
+                    except:
+                        range.append(None)
+                        html.set_var(varprefix, "")
 
-        if secs > 0:
-            timestamp = int(time.time()) - secs
-            neg = html.var(self.name, "since") != "since"
-            return timestamp, neg
-        else:
-            return None
+        return range
 
     # I'm not sure if this function is useful or ever been called.
     # Problem is, that it is not clear wether to use "since" or "before"
     # here.
-    def variable_settings(self, row):
-        vars = []
-        secs = int(time.time()) - row[self.column]
-        for s, n in self.ranges[::-1]:
-            v = secs / s
-            secs -= v * s
-            vars.append((self.name + "_" + n, secs))
-        return vars
+    # def variable_settings(self, row):
+    #     vars = []
+    #     secs = int(time.time()) - row[self.column]
+    #     for s, n in self.ranges[::-1]:
+    #         v = secs / s
+    #         secs -= v * s
+    #         vars.append((self.name + "_" + n, secs))
+    #     return vars
 
-    def heading_info(self, infoname):
-        return _("since the last couple of seconds")
+    # def heading_info(self, infoname):
+    #     return _("since the last couple of seconds")
 
 declare_filter(250, FilterTime("service", "svc_last_state_change", _("Last service state change"), "service_last_state_change"))
 declare_filter(251, FilterTime("service", "svc_last_check", _("Last service check"), "service_last_check"))
 
+declare_filter(250, FilterTime("host", "host_last_state_change", _("Last host state change"), "host_last_state_change"))
+declare_filter(251, FilterTime("host", "host_last_check", _("Last host check"), "host_last_check"))
+
 #    _
 #   | |    ___   __ _
 #   | |   / _ \ / _` |
@@ -464,6 +523,9 @@ class FilterLogClass(Filter):
         Filter.__init__(self, "log_class", _("Logentry class"),
                 "log", [ "logclass%d" % l for l, c in self.log_classes ], [])
 
+    def double_height(self):
+        return True
+
     def display(self):
         if html.var("filled_in"):
             defval = ""
@@ -510,3 +572,67 @@ declare_filter(255, FilterLogClass())
 declare_filter(202, FilterText("log_plugin_output",  _("Log: plugin output"), "log", "log_plugin_output", "log_plugin_output", "~~"))
 declare_filter(260, FilterText("log_contact_name",   _("Log: contact name"),  "log", "log_contact_name",  "log_contact_name",  "="),
                                                                                                   _("Exact match, used for linking"))
+
+class FilterLogState(Filter):
+    def __init__(self):
+        self._items = [ ("h0", "host", 0, _("Up")),("h1", "host", 1, _("Down")),("h2", "host", 2, _("Unreachable")),
+                        ("s0", "service", 0, _("OK")), ("s1", "service", 1, _("Warning")),
+                        ("s2", "service", 2, _("Critical")),("s3", "service", 3, _("Unknown")) ]
+
+        Filter.__init__(self, "log_state", _("Type of alerts of hosts and services"),
+                "log", [ "logst_" + e[0] for e in self._items ], [])
+
+    def double_height(self):
+        return True
+
+    def display(self):
+        html.write("<table class=alertstatefilter><tr><td>")
+        html.begin_checkbox_group()
+        for varsuffix, what, state, text in self._items:
+            if state == 0:
+                html.write("<u>%s:</u></td><td>" % (_(what.title())))
+            html.write("&nbsp; ")
+            html.checkbox("logst_" + varsuffix, True, label=text)
+            if not html.mobile:
+                html.write("<br>")
+            if varsuffix == "h2":
+                html.write("</td><td>")
+        html.end_checkbox_group()
+        html.write("</td></tr></table>")
+
+    def filter(self, infoname):
+        headers = []
+        for varsuffix, what, state, text in self._items:
+            if html.get_checkbox("logst_" + varsuffix) != False: # None = form not filled in = allow
+                headers.append("Filter: log_type ~ %s .*\nFilter: log_state = %d\nAnd: 2\n" %
+                            (what.upper(), state))
+        if len(headers) == 0:
+            return "Limit: 0\n" # no allowed state
+        elif len(headers) == len(self._items):
+            return "" # all allowed or form not filled in
+        else:
+            return "".join(headers) + ("Or: %d\n" % len(headers))
+
+declare_filter(270, FilterLogState())
+
+class BIServiceIsUsedFilter(FilterTristate):
+    def __init__(self):
+	FilterTristate.__init__(self, "aggr_service_used", _("Used in BI aggregate"), "service", None)
+
+    def filter(self, infoname):
+	return ""
+
+    def filter_table(self, rows):
+        current = self.tristate_value()
+        if current == -1:
+	    return rows
+        new_rows = []
+        for row in rows:
+	    is_part = bi.is_part_of_aggregation(
+                   "service", row["site"], row["host_name"], row["service_description"])
+	    if (is_part and current == 1) or \
+               (not is_part and current == 0):
+	        new_rows.append(row)
+	return new_rows
+
+declare_filter(300, BIServiceIsUsedFilter())
