@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -24,21 +24,20 @@
 # to the Free Software Foundation, Inc., 51 Franklin St,  Fifth Floor,
 # Boston, MA 02110-1301 USA.
 
-def init_rowselect():
+def init_rowselect(view):
     # Don't make rows selectable when no commands can be fired
     # Ignore "C" display option here. Otherwise the rows will not be selectable
     # after view reload.
-    if not config.may("act"):
+    if not config.may("general.act"):
         return
 
-    selected = []
-    if html.has_var('selected_rows'):
-        selected_rows = html.var('selected_rows', '')
-        if selected_rows or ',' in selected_rows:
-            selected = selected_rows.split(',')
-
-    html.javascript('g_selected_rows = %s;\n'
-                    'init_rowselect();' % repr(selected))
+    selected = weblib.get_rowselection('view-' + view['name'])
+    html.javascript(
+        'g_page_id = "view-%s";\n'
+        'g_selection = "%s";\n'
+        'g_selected_rows = %s;\n'
+        'init_rowselect();' % (view['name'], weblib.selection_id(), repr(selected))
+    )
 
 def render_checkbox(view, row, num_tds):
     # value contains the number of columns of this datarow. This is
@@ -67,8 +66,9 @@ def render_single_dataset(rows, view, group_painters, painters, num_columns, _ig
     for row in rows:
         register_events(row) # needed for playing sounds
 
-    html.write('<div class="tableshadow left"><table class=dataset>\n')
+    html.write('<table class="data single">\n')
     rownum = 0
+    odd = "odd"
     while rownum < len(rows):
         if rownum > 0:
             html.write("<tr class=gap><td class=gap colspan=%d></td></tr>\n" % (1 + num_columns))
@@ -77,10 +77,13 @@ def render_single_dataset(rows, view, group_painters, painters, num_columns, _ig
             painter, link = p[0:2]
             if len(p) >= 5 and p[4]:
                 title = p[4] # Use custom title
+            elif len(p) == 4 and p[3]:
+                title = p[3] # Use the join index (service name) as title
             else:
                 title = painter["title"]
 
-            html.write("<tr class=data>")
+            odd = odd == "odd" and "even" or "odd"
+            html.write('<tr class="data %s0">' % odd)
             if view.get("column_headers") != "off":
                 html.write("<td class=left>%s</td>" % title)
             for row in thispart:
@@ -89,7 +92,8 @@ def render_single_dataset(rows, view, group_painters, painters, num_columns, _ig
                 html.write("<td class=gap style=\"border-style: none;\" colspan=%d></td>" % (1 + num_columns - len(thispart)))
             html.write("</tr>\n")
         rownum += num_columns
-    html.write("</table></div>\n")
+    html.write("</table>\n")
+    html.write("</div>\n")
 
 multisite_layouts["dataset"] = {
     "title"  : _("Single dataset"),
@@ -107,6 +111,9 @@ multisite_layouts["dataset"] = {
 #
 # -------------------------------------------------------------------------
 def render_grouped_boxes(rows, view, group_painters, painters, num_columns, show_checkboxes):
+
+    repeat_heading_every = 20 # in case column_headers is "repeat"
+
     # N columns. Each should contain approx the same number of entries
     groups = []
     last_group = None
@@ -155,8 +162,8 @@ def render_grouped_boxes(rows, view, group_painters, painters, num_columns, show
 
 
     # render one group
-    def render_group(header, rows, paintheader):
-        html.write("<table class=groupheader><tr class=groupheader>")
+    def render_group(header, rows):
+        html.write("<table class=groupheader cellspacing=0 cellpadding=0 border=0><tr class=groupheader>")
         painted = False
         for p in group_painters:
             if painted:
@@ -164,11 +171,10 @@ def render_grouped_boxes(rows, view, group_painters, painters, num_columns, show
             painted = paint(p, rows[0][1])
         html.write("</tr></table>\n")
 
-        html.write("<div class=tableshadow><table class=data>")
+        html.write("<table class=data>")
         trclass = None
 
-        # paint table headers, if configured
-        if paintheader:
+        def show_header_line():
             html.write("<tr>")
             if show_checkboxes:
                 render_group_checkbox_th()
@@ -177,7 +183,17 @@ def render_grouped_boxes(rows, view, group_painters, painters, num_columns, show
                 html.write("\n")
             html.write("</tr>\n")
 
+        column_headers = view.get("column_headers")
+        if column_headers:
+            show_header_line()
+
+        visible_row_number = 0
         for index, row in rows:
+            if view.get("column_headers") == "repeat":
+                if visible_row_number > 0 and visible_row_number % repeat_heading_every == 0:
+                    show_header_line()
+            visible_row_number += 1
+
             register_events(row) # needed for playing sounds
             if trclass == "odd":
                 trclass = "even"
@@ -195,25 +211,15 @@ def render_grouped_boxes(rows, view, group_painters, painters, num_columns, show
                 paint(p, row)
             html.write("</tr>\n")
 
-        html.write("</table></div>\n")
-        init_rowselect()
+        html.write("</table>\n")
+        init_rowselect(view)
 
     # render table
-    if view.get("column_headers") != "off":
-        headerswitch = -1
-    else:
-        headerswitch = 0
-
     html.write("<table class=boxlayout><tr>")
     for column in columns:
         html.write("<td class=boxcolumn>")
         for header, rows in column:
-            if headerswitch:
-                paintheader = True
-                headerswitch -= 1
-            else:
-                paintheader = False
-            render_group(header, rows, paintheader)
+            render_group(header, rows)
         html.write("</td>")
     html.write("</tr></table>\n")
 
@@ -286,11 +292,11 @@ def render_tiled(rows, view, group_painters, painters, _ignore_num_columns, show
         if len(painters) < 5:
             painters = painters + ([ (empty_painter, None) ] * (5 - len(painters)))
 
-        rendered = [ ("", prepare_paint(p, row)[1]) for p in painters ]
+        rendered = [ prepare_paint(p, row) for p in painters ]
 
         html.write("<tr><td class=\"tl %s\">" % (rendered[1][0],))
         if show_checkboxes:
-            render_checkbox(view, row, len(painters) - 1)
+            render_checkbox(view, row, len(painters)-1)
         html.write("%s</td><td class=\"tr %s\">%s</td></tr>\n" % \
                     (rendered[1][1], rendered[2][0], rendered[2][1]))
         html.write("<tr><td colspan=2 class=\"center %s\">%s</td></tr>\n" % \
@@ -304,7 +310,7 @@ def render_tiled(rows, view, group_painters, painters, _ignore_num_columns, show
     if group_open:
         html.write("</td></tr>\n")
     html.write("</table>\n")
-    init_rowselect()
+    init_rowselect(view)
 
 
 multisite_layouts["tiled"] = {
@@ -323,9 +329,12 @@ multisite_layouts["tiled"] = {
 #
 # ------------------------------------------------------------------------
 def render_grouped_list(rows, view, group_painters, painters, num_columns, show_checkboxes):
-    html.write("<table class=data>\n")
+
+    repeat_heading_every = 20 # in case column_headers is "repeat"
+
+    html.write("<table class='data table'>\n")
     last_group = None
-    trclass = None
+    odd = "even"
     column = 1
     group_open = False
     num_painters = len(painters)
@@ -343,6 +352,9 @@ def render_grouped_list(rows, view, group_painters, painters, num_columns, show_
 
             for p in painters:
                 paint_header(view, p)
+            if n < num_columns:
+                html.write('<td class=gap></td>')
+
         html.write("</tr>\n")
 
     if len(group_painters) == 0 and view.get("column_headers") != "off":
@@ -363,6 +375,7 @@ def render_grouped_list(rows, view, group_painters, painters, num_columns, show_
 
 
     index = 0
+    visible_row_number = 0
     for row in rows:
         register_events(row) # needed for playing sounds
         # Show group header, if a new group begins. But only if grouping
@@ -372,40 +385,33 @@ def render_grouped_list(rows, view, group_painters, painters, num_columns, show_
             if this_group != last_group:
                 if column != 1: # not a the beginning of a new line
                     for i in range(column-1, num_columns):
+                        html.write('<td class=gap></td>')
                         html.write("<td class=fillup colspan=%d></td>" % num_painters)
                     html.write("</tr>\n")
-                    html.write("<tr>\n")
-                    for x in range(0, num_columns):
-                        html.write("<td colspan=%d class=shadowbottom></td>\n" % num_painters)
-                    html.write("</tr>\n")
                     column = 1
 
-                # paint group header
                 group_open = True
-                html.write("<tr class=groupheader>")
-                html.write("<td class=groupheader colspan=%d><table class=groupheader><tr>" % 
-                     (num_painters * (num_columns + 2) + (num_columns - 1)))
-                painted = False
+                visible_row_number = 0
+
+                # paint group header, but only if it is non-empty
+                header_is_empty = True
                 for p in group_painters:
-                    if painted:
-                        html.write("<td>,</td>")
-                    painted = paint(p, row)
+                    tdclass, content = prepare_paint(p, row)
+                    if content:
+                        header_is_empty = False
+                        break
 
-                html.write("</tr></table></td></tr>\n")
+                if not header_is_empty:
+                    html.write("<tr class=groupheader>")
+                    html.write("<td class=groupheader colspan=%d><table class=groupheader cellspacing=0 cellpadding=0 border=0><tr>" %
+                         (num_painters * (num_columns + 2) + (num_columns - 1)))
+                    painted = False
+                    for p in group_painters:
+                        if painted:
+                            html.write("<td>,</td>")
+                        painted = paint(p, row)
 
-                # paint top, left and right shadows
-                group_rows = count_group_members(row, rows[index:])
-                rowspan = 3 + group_rows / num_columns
-                if group_rows % num_columns:
-                    rowspan += 1
-                html.write("<tr>\n")
-                for x in range(0, num_columns):
-                    if x > 0:
-                        html.write("<td rowspan=%d class=tablegap></td>\n" % rowspan)
-                    html.write("<td rowspan=%d class=shadowleft></td>\n" % rowspan)
-                    html.write("<td colspan=%d class=shadowtop></td>\n" % num_painters)
-                    html.write("<td rowspan=%d class=shadowright></td>\n" % rowspan)
-                html.write("</tr>\n")
+                    html.write("</tr></table></td></tr>\n")
 
                 # Table headers
                 if view.get("column_headers") != "off":
@@ -420,6 +426,11 @@ def render_grouped_list(rows, view, group_painters, painters, num_columns, show_
 
         # At the beginning of the line? Beginn new line
         if column == 1:
+            if view.get("column_headers") == "repeat":
+                if visible_row_number > 0 and visible_row_number % repeat_heading_every == 0:
+                    show_header_line()
+            visible_row_number += 1
+
             # In one-column layout we use the state of the service
             # or host - if available - to color the complete line
             if num_columns == 1:
@@ -431,29 +442,31 @@ def render_grouped_list(rows, view, group_painters, painters, num_columns, show_
                     state = row.get("service_state", 0)
             else:
                 state = 0
-            if trclass == "odd":
-                trclass = "even"
-            else:
-                trclass = "odd"
-            html.write('<tr class="data %s%d">' % (trclass, state))
+
+            odd = odd == "odd" and "even" or "odd"
+            html.write('<tr class="data %s %s%d">' % (num_columns > 1 and "multicolumn" or "", odd, state))
+
+        # Not first columns: Create one empty column as separator
+        else:
+            html.write('<td class=gap></td>')
+
 
         if show_checkboxes:
-            render_checkbox_td(view, row, num_painters - 1)
+            render_checkbox_td(view, row, num_painters)
+
         for p in painters:
             paint(p, row)
+
         column += 1
         index += 1
 
     if group_open:
         for i in range(column-1, num_columns):
+            html.write('<td class=gap></td>')
             html.write("<td class=fillup colspan=%d></td>" % num_painters)
         html.write("</tr>\n")
-        html.write("<tr>\n")
-        for x in range(0, num_columns):
-            html.write("<td colspan=%d class=shadowbottom></td>\n" % num_painters)
-        html.write("</tr>\n")
     html.write("</table>\n")
-    init_rowselect()
+    init_rowselect(view)
 
 multisite_layouts["table"] = {
     "title"  : _("Table"),
@@ -461,3 +474,4 @@ multisite_layouts["table"] = {
     "group"  : True,
     "checkboxes" : True,
 }
+
