@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -62,13 +62,13 @@
 # change in future.
 # =================================================================== #
 
-#     ____       _       _                          _   _                 
-#    |  _ \ __ _(_)_ __ | |_ ___ _ __    ___  _ __ | |_(_) ___  _ __  ___ 
+#     ____       _       _                          _   _
+#    |  _ \ __ _(_)_ __ | |_ ___ _ __    ___  _ __ | |_(_) ___  _ __  ___
 #    | |_) / _` | | '_ \| __/ _ \ '__|  / _ \| '_ \| __| |/ _ \| '_ \/ __|
 #    |  __/ (_| | | | | | ||  __/ |    | (_) | |_) | |_| | (_) | | | \__ \
 #    |_|   \__,_|_|_| |_|\__\___|_|     \___/| .__/ \__|_|\___/|_| |_|___/
-#                                            |_|                          
-# 
+#                                            |_|
+#
 # Painter options influence how painters render their data. Painter options
 # are stored together with "refresh" and "columns" as "View options".
 
@@ -84,20 +84,35 @@ multisite_painter_options["pnpview"] = {
 
 multisite_painter_options["ts_format"] = {
  "title"   : _("Time stamp format"),
- "default" : "mixed",
- "values"  : [ ("mixed", _("Mixed")), ("abs", _("Absolute")), ("rel", _("Relative")) ]
+ "default" : config.default_ts_format,
+ "values"  : [
+     ("mixed", _("Mixed")),
+     ("abs", _("Absolute")),
+     ("rel", _("Relative")),
+     ("both", _("Both")),
+  ]
 }
 
 multisite_painter_options["ts_date"] = {
  "title" : _("Date format"),
  "default" : "%Y-%m-%d",
- "values" : [ ("%Y-%m-%d", "1970-12-18"), 
-              ("%d.%m.%Y", "18.12.1970"), 
-              ("%m/%d/%Y", "12/18/1970"), 
-              ("%d.%m.",   "18.12."), 
+ "values" : [ ("%Y-%m-%d", "1970-12-18"),
+              ("%d.%m.%Y", "18.12.1970"),
+              ("%m/%d/%Y", "12/18/1970"),
+              ("%d.%m.",   "18.12."),
               ("%m/%d",    "12/18") ]
 }
 
+# This helper function returns the value of the given custom var
+def paint_custom_host_var(what, row):
+    custom_vars = dict(zip(row["host_custom_variable_names"],
+                           row["host_custom_variable_values"]))
+
+    if what in custom_vars:
+        return what, custom_vars[what]
+    return what,  ""
+
+
 #    ___
 #   |_ _|___ ___  _ __  ___
 #    | |/ __/ _ \| '_ \/ __|
@@ -111,23 +126,23 @@ multisite_icons = []
 
 load_web_plugins('icons', globals())
 
+# Paint column with various icons. The icons use
+# a plugin based mechanism so it is possible to
+# register own icon "handlers".
+# what: either "host" or "service"
+# row: the data row of the host or service
 def paint_icons(what, row):
-    """
-    what: "host" or "service"
-    row:  The livestatus row of the current object
-    """
-    if what + "_custom_variable_names" in row:
-        custom_vars = dict(zip(row[what + "_custom_variable_names"],
-                               row[what + "_custom_variable_values"]))
+    if not row["host_name"]:
+        return "", ""# Host probably does not exist
 
-        # Extract host tags
-        if "TAGS" in custom_vars:
-            tags = custom_vars["TAGS"].split()
-        else:
-            tags = []
+    custom_vars = dict(zip(row["host_custom_variable_names"],
+                           row["host_custom_variable_values"]))
+
+    # Extract host tags
+    if "TAGS" in custom_vars:
+        tags = custom_vars["TAGS"].split()
     else:
-        tags        = None
-        custom_vars = None
+        tags = []
 
     output = ""
     for icon in multisite_icons:
@@ -140,25 +155,25 @@ def paint_icons(what, row):
 
     return "icons", output
 
-
 def iconpainter_columns(what):
-    if what == 'hostgroup':
-        cols = set([ "site", "hostgroup_name" ])
-    else:
-        cols = set(['site',
+    cols = set(['site',
                 'host_name',
-                what + '_custom_variable_names',
-                what + '_custom_variable_values' ])
+                'host_custom_variable_names',
+                'host_custom_variable_values' ])
 
-        if what == 'service':
-            cols.add('service_description')
+    if what == 'service':
+        cols.update([
+            'service_description',
+            'service_custom_variable_names',
+            'service_custom_variable_values',
+        ])
 
     for icon in multisite_icons:
-        # HACK: Don't use general columns requests for hostgroup iconpainters
-        if what != 'hostgroup' and 'columns' in icon:
-            cols.update([ what + '_' + i for i in icon['columns'] ])
-        if what + '_columns' in icon:
-            cols.update(icon[what + '_columns'])
+        if 'columns' in icon:
+            cols.update([ what + '_' + c for c in icon['columns'] ])
+        cols.update([ "host_" + c for c in icon.get("host_columns", [])])
+        if what == "service":
+            cols.update([ "service_" + c for c in icon.get("service_columns", [])])
 
     return cols
 
@@ -176,13 +191,6 @@ multisite_painters["host_icons"] = {
     "paint":   lambda row: paint_icons("host", row)
 }
 
-multisite_painters["hg_icons"] = {
-    "title":   _("Hostgroup icons"),
-    "short":   _("Icons"),
-    "columns": iconpainter_columns("hostgroup"),
-    "paint":   lambda row: paint_icons("hostgroup", row)
-}
-
 # -----------------------------------------------------------------------
 
 def paint_nagios_link(row):
@@ -199,11 +207,18 @@ def paint_nagios_link(row):
         what = "host"
     return "singleicon", "<a href=\"%s\"><img title=\"%s\" src=\"images/icon_nagios.gif\"></a>" % (url, _('Show this %s in Nagios') % what)
 
-def paint_age(timestamp, has_been_checked, bold_if_younger_than):
+def paint_age(timestamp, has_been_checked, bold_if_younger_than, mode=None):
     if not has_been_checked:
         return "age", "-"
 
-    mode = get_painter_option("ts_format")
+    if mode == None:
+        mode = get_painter_option("ts_format")
+
+    if mode == "both":
+        css, h1 = paint_age(timestamp, has_been_checked, bold_if_younger_than, "abs")
+        css, h2 = paint_age(timestamp, has_been_checked, bold_if_younger_than, "rel")
+        return css, "%s - %s" % (h1, h2)
+
     dateformat = get_painter_option("ts_date")
     age = time.time() - timestamp
     if mode == "abs" or \
@@ -222,6 +237,7 @@ def paint_age(timestamp, has_been_checked, bold_if_younger_than):
         age_class = "age"
     return age_class, prefix + html.age_text(age)
 
+
 def paint_future_time(timestamp):
     if timestamp <= 0:
         return "", "-"
@@ -258,7 +274,7 @@ multisite_painters["sitealias"] = {
 #   |____/ \___|_|    \_/ |_|\___\___||___/
 #
 
-def paint_service_state_short(row):
+def paint_service_state_short(row ):
     if row["service_has_been_checked"] == 1:
         state = row["service_state"]
         name = nagios_short_state_names[row["service_state"]]
@@ -292,6 +308,7 @@ multisite_painters["service_state"] = {
     "sorter"  : 'svcstate',
 }
 
+
 multisite_painters["site_icon"] = {
     "title"   : _("Icon showing the site"),
     "short"   : "",
@@ -300,20 +317,12 @@ multisite_painters["site_icon"] = {
     "sorter"  : 'site',
 }
 
-warn_marker    = '<b class="stmark state1">WARN</b>'
-crit_marker    = '<b class="stmark state2">CRIT</b>'
-unknown_marker = '<b class="stmark state3">UNKN</b>'
-
-def format_plugin_output(output):
-    return output.replace("(!)", warn_marker) \
-           .replace("(!!)", crit_marker) \
-           .replace("(?)", unknown_marker)
 
 multisite_painters["svc_plugin_output"] = {
     "title"   : _("Output of check plugin"),
     "short"   : _("Status detail"),
     "columns" : ["service_plugin_output"],
-    "paint"   : lambda row: ("", format_plugin_output(row["service_plugin_output"])),
+    "paint"   : lambda row: ("", format_plugin_output(row["service_plugin_output"], row)),
     "sorter"  : 'svcoutput',
 }
 multisite_painters["svc_long_plugin_output"] = {
@@ -352,70 +361,70 @@ def paint_perfdata_nth_value(row, n):
 
 multisite_painters["svc_perf_val01"] = {
     "title" : _("Service performance data - value number  1"),
-    "short" : _("Val. 1"), 
+    "short" : _("Val. 1"),
     "columns" : ["service_perf_data"],
     "paint" : lambda row: paint_perfdata_nth_value(row, 0)
 }
 
 multisite_painters["svc_perf_val02"] = {
     "title" : _("Service performance data - value number  2"),
-    "short" : _("Val. 2"), 
+    "short" : _("Val. 2"),
     "columns" : ["service_perf_data"],
     "paint" : lambda row: paint_perfdata_nth_value(row, 1)
 }
 
 multisite_painters["svc_perf_val03"] = {
     "title" : _("Service performance data - value number  3"),
-    "short" : _("Val. 3"), 
+    "short" : _("Val. 3"),
     "columns" : ["service_perf_data"],
     "paint" : lambda row: paint_perfdata_nth_value(row, 2)
 }
 
 multisite_painters["svc_perf_val04"] = {
     "title" : _("Service performance data - value number  4"),
-    "short" : _("Val. 4"), 
+    "short" : _("Val. 4"),
     "columns" : ["service_perf_data"],
     "paint" : lambda row: paint_perfdata_nth_value(row, 3)
 }
 
 multisite_painters["svc_perf_val05"] = {
     "title" : _("Service performance data - value number  5"),
-    "short" : _("Val. 5"), 
+    "short" : _("Val. 5"),
     "columns" : ["service_perf_data"],
     "paint" : lambda row: paint_perfdata_nth_value(row, 4)
 }
 
 multisite_painters["svc_perf_val06"] = {
     "title" : _("Service performance data - value number  6"),
-    "short" : _("Val. 6"), 
+    "short" : _("Val. 6"),
     "columns" : ["service_perf_data"],
     "paint" : lambda row: paint_perfdata_nth_value(row, 5)
 }
 
 multisite_painters["svc_perf_val07"] = {
     "title" : _("Service performance data - value number  7"),
-    "short" : _("Val. 7"), 
+    "short" : _("Val. 7"),
     "columns" : ["service_perf_data"],
     "paint" : lambda row: paint_perfdata_nth_value(row, 6)
 }
 
 multisite_painters["svc_perf_val08"] = {
     "title" : _("Service performance data - value number  8"),
-    "short" : _("Val. 8"), 
+    "short" : _("Val. 8"),
     "columns" : ["service_perf_data"],
     "paint" : lambda row: paint_perfdata_nth_value(row, 7)
 }
 
 multisite_painters["svc_perf_val09"] = {
     "title" : _("Service performance data - value number  9"),
-    "short" : _("Val. 9"), 
+    "short" : _("Val. 9"),
     "columns" : ["service_perf_data"],
     "paint" : lambda row: paint_perfdata_nth_value(row, 8)
 }
 
 multisite_painters["svc_perf_val10"] = {
     "title" : _("Service performance data - value number 10"),
-    "short" : _("Val. 10"), 
+    "short" : _("Val. 10"),
     "columns" : ["service_perf_data"],
     "paint" : lambda row: paint_perfdata_nth_value(row, 9)
 }
@@ -434,6 +443,13 @@ multisite_painters["svc_check_command"] = {
     "paint"   : lambda row: (None, row["service_check_command"]),
 }
 
+multisite_painters["svc_check_command_expanded"] = {
+    "title"   : _("Service check command expanded"),
+    "short"   : _("Check command expanded"),
+    "columns" : ["service_check_command_expanded"],
+    "paint"   : lambda row: (None, row["service_check_command_expanded"]),
+}
+
 multisite_painters["svc_contacts"] = {
     "title"   : _("Service contacts"),
     "short"   : _("Contacts"),
@@ -516,6 +532,26 @@ multisite_painters["svc_attempt"] = {
     "paint"   : lambda row: (None, "%d/%d" % (row["service_current_attempt"], row["service_max_check_attempts"])),
 }
 
+multisite_painters["svc_normal_interval"] = {
+    "title"   : _("Service normal check interval"),
+    "short"   : _("Check int."),
+    "columns" : [ "service_check_interval" ],
+    "paint"   : lambda row: (None, "%.0fs" % (row["service_check_interval"] * 60.0)),
+}
+multisite_painters["svc_retry_interval"] = {
+    "title"   : _("Service retry check interval"),
+    "short"   : _("Retry"),
+    "columns" : [ "service_retry_interval" ],
+    "paint"   : lambda row: (None, "%.0fs" % (row["service_retry_interval"] * 60.0)),
+}
+multisite_painters["svc_check_interval"] = {
+    "title"   : _("Service normal/retry check interval"),
+    "short"   : _("Interval"),
+    "columns" : [ "service_check_interval", "service_retry_interval" ],
+    "paint"   : lambda row: (None, "%.0fs/%.0fs" % (
+            row["service_check_interval"] * 60.0, row["service_retry_interval"] * 60.0)),
+}
+
 multisite_painters["svc_check_type"] = {
     "title"   : _("Service check type"),
     "short"   : _("Type"),
@@ -650,7 +686,7 @@ multisite_painters["check_manpage"] = {
 
 def paint_comments(prefix, row):
     comments = row[ prefix + "comments_with_info"]
-    text = ", ".join(["<i>%s</i>: %s" % (a,c) for (id,a,c) in comments ])
+    text = ", ".join(["<i>%s</i>: %s" % (a, htmllib.attrencode(c)) for (id, a, c) in comments ])
     return "", text
 
 multisite_painters["svc_comments"] = {
@@ -708,7 +744,10 @@ def paint_custom_notes(row):
             .replace('$HOSTNAME_UPPER$', host.upper())\
             .replace('$HOSTNAME_TITLE$', host[0].upper() + host[1:].lower())\
             .replace('$HOSTADDRESS$',    row["host_address"])\
+            .replace('$SERVICEOUTPUT$',  row.get("service_plugin_output", ""))\
+            .replace('$HOSTOUTPUT$',     row.get("host_plugin_output", ""))\
             .replace('$SERVICEDESC$',    row.get("service_description", ""))
+
     for f in files:
         contents.append(replace_tags(unicode(file(f).read(), "utf-8").strip()))
     return "", "<hr>".join(contents)
@@ -716,7 +755,7 @@ def paint_custom_notes(row):
 multisite_painters["svc_custom_notes"] = {
     "title"   : _("Custom services notes"),
     "short"   : _("Notes"),
-    "columns" : [ "host_name", "host_address", "service_description" ],
+    "columns" : [ "host_name", "host_address", "service_description", "service_plugin_output" ],
     "paint"   : paint_custom_notes,
 }
 
@@ -735,6 +774,14 @@ multisite_painters["host_state"] = {
     "sorter"  : 'hoststate',
 }
 
+multisite_painters["host_state_onechar"] = {
+    "title"   : _("Host state (first character)"),
+    "short"   : _("S."),
+    "columns" : ["host_has_been_checked","host_state"],
+    "paint"   : lambda row: paint_host_state_short(row, True),
+    "sorter"  : 'hoststate',
+}
+
 multisite_painters["host_plugin_output"] = {
     "title"   : _("Output of host check plugin"),
     "short"   : _("Status detail"),
@@ -756,6 +803,13 @@ multisite_painters["host_check_command"] = {
     "paint"   : lambda row: (None, row["host_check_command"]),
 }
 
+multisite_painters["host_check_command_expanded"] = {
+    "title"   : _("Host check command expanded"),
+    "short"   : _("Check command expanded"),
+    "columns" : ["host_check_command_expanded"],
+    "paint"   : lambda row: (None, row["host_check_command_expanded"]),
+}
+
 multisite_painters["host_state_age"] = {
     "title"   : _("The age of the current host state"),
     "short"   : _("Age"),
@@ -815,6 +869,26 @@ multisite_painters["host_attempt"] = {
     "paint"   : lambda row: (None, "%d/%d" % (row["host_current_attempt"], row["host_max_check_attempts"])),
 }
 
+multisite_painters["host_normal_interval"] = {
+    "title"   : _("Normal check interval"),
+    "short"   : _("Check int."),
+    "columns" : [ "host_check_interval" ],
+    "paint"   : lambda row: (None, "%.0fs" % (row["host_check_interval"] * 60.0)),
+}
+multisite_painters["host_retry_interval"] = {
+    "title"   : _("Retry check interval"),
+    "short"   : _("Retry"),
+    "columns" : [ "host_retry_interval" ],
+    "paint"   : lambda row: (None, "%.0fs" % (row["host_retry_interval"] * 60.0)),
+}
+multisite_painters["host_check_interval"] = {
+    "title"   : _("Normal/retry check interval"),
+    "short"   : _("Interval"),
+    "columns" : [ "host_check_interval", "host_retry_interval" ],
+    "paint"   : lambda row: (None, "%.0fs/%.0fs" % (
+            row["host_check_interval"] * 60.0, row["host_retry_interval"] * 60.0)),
+}
+
 multisite_painters["host_check_type"] = {
     "title"   : _("Host check type"),
     "short"   : _("Type"),
@@ -905,10 +979,13 @@ def paint_host_with_state(row):
         state = row["host_state"]
     else:
         state = "p"
-    return "state hstate hstate%s" % state, row["host_name"]
+    if state != 0:
+        return "state hstate hstate%s" % state, row["host_name"]
+    else:
+        return "", row["host_name"]
 
 multisite_painters["host_with_state"] = {
-    "title"   : _("Hostname colored with state"),
+    "title"   : _("Hostname, marked red if down"),
     "short"   : _("Host"),
     "columns" : ["site", "host_name", "host_state", "host_has_been_checked" ],
     "paint"   : paint_host_with_state,
@@ -1031,20 +1108,6 @@ multisite_painters["host_services"] = {
     "paint"   : lambda row: paint_service_list(row, "host_services_with_state"),
 }
 
-def paint_host_list(site, hosts):
-    h = ""
-    first = True
-    for host in hosts:
-        if first:
-            first = False
-        else:
-            h += ", "
-        link = "view.py?view_name=hoststatus&site=%s&host=%s" % (htmllib.urlencode(site), htmllib.urlencode(host))
-        if html.var("display_options"):
-            link += "&display_options=%s" % html.var("display_options")
-        h += "<a href=\"%s\">%s</a></div>" % (link, host)
-    return "", h
-
 multisite_painters["host_parents"] = {
     "title"   : _("Host's parents"),
     "short"   : _("Parents"),
@@ -1090,9 +1153,9 @@ multisite_painters["host_contact_groups"] = {
 }
 
 multisite_painters["host_custom_notes"] = {
-    "title"   : _("Custom host notes"), 
+    "title"   : _("Custom host notes"),
     "short"   : _("Notes"),
-    "columns" : [ "host_name", "host_address" ],
+    "columns" : [ "host_name", "host_address", "host_plugin_output" ],
     "paint"   : paint_custom_notes,
 }
 
@@ -1339,7 +1402,7 @@ multisite_painters["comment_author"] = {
 multisite_painters["comment_comment"] = {
     "title"   : _("Comment text"),
     "columns" : ["comment_comment"],
-    "paint"   : lambda row: (None, row["comment_comment"]),
+    "paint"   : lambda row: (None, htmllib.attrencode(row["comment_comment"])),
 }
 
 multisite_painters["comment_what"] = {
@@ -1418,28 +1481,28 @@ multisite_painters["downtime_comment"] = {
     "title"   : _("Downtime comment"),
     "short"   : _("Comment"),
     "columns" : ["downtime_comment"],
-    "paint"   : lambda row: (None, row["downtime_comment"]),
+    "paint"   : lambda row: (None, htmllib.attrencode(row["downtime_comment"])),
 }
 
 multisite_painters["downtime_fixed"] = {
     "title"   : _("Downtime is fixed"),
     "short"   : _("Fixed"),
     "columns" : ["downtime_fixed"],
-    "paint"   : lambda row: (None, row["downtime_fixed"] == 0 and "flexible" or "fixed"),
+    "paint"   : lambda row: (None, row["downtime_fixed"] == 0 and _("flexible") or _("fixed")),
 }
 
 multisite_painters["downtime_what"] = {
     "title"   : _("Downtime type (host/service)"),
     "short"   : _("Type"),
     "columns" : ["is_service"],
-    "paint"   : lambda row: (None, row["is_service"] and "Service" or "Host"),
+    "paint"   : lambda row: (None, row["is_service"] and _("Service") or _("Host")),
 }
 
 multisite_painters["downtime_type"] = {
     "title"   : _("Downtime active or pending"),
     "short"   : _("act/pend"),
     "columns" : ["downtime_type"],
-    "paint"   : lambda row: (None, row["downtime_type"] == 0 and "active" or "pending"),
+    "paint"   : lambda row: (None, row["downtime_type"] == 0 and _("active") or _("pending")),
 }
 
 multisite_painters["downtime_entry_time"] = {
@@ -1490,14 +1553,34 @@ multisite_painters["log_message"] = {
     "title"   : _("Log: complete message"),
     "short"   : _("Message"),
     "columns" : ["log_message"],
-    "paint"   : lambda row: ("", row["log_message"]),
+    "paint"   : lambda row: ("", htmllib.attrencode(row["log_message"])),
 }
 
+def paint_log_plugin_output(row):
+    output = row["log_plugin_output"]
+    if output:
+        return "", format_plugin_output(output, row)
+    else:
+        log_type = row["log_type"]
+        lst = row["log_state_type"]
+        if "FLAPPING" in log_type:
+            if "HOST" in log_type:
+                what = _("host")
+            else:
+                what = _("service")
+            if lst == "STOPPED":
+                return "", _("The %s stopped flapping") % what
+            else:
+                return "", _("The %s started flapping") % what
+
+        return "", (lst + " - " + log_type)
+
+
 multisite_painters["log_plugin_output"] = {
     "title"   : _("Log: output of check plugin"),
     "short"   : _("Check output"),
-    "columns" : ["log_plugin_output"],
-    "paint"   : lambda row: ("", format_plugin_output(row["log_plugin_output"])),
+    "columns" : ["log_plugin_output", "log_type", "log_state_type" ],
+    "paint"   : paint_log_plugin_output,
 }
 
 multisite_painters["log_attempt"] = {
@@ -1546,6 +1629,8 @@ def paint_log_icon(row):
         img = "start"
     elif "shutdown..." in log_type:
         img = "stop"
+    elif " FLAPPING " in log_type:
+        img = "flapping"
 
     if img:
         return "icon", '<img src="images/alert_%s.png">' % img
@@ -1563,7 +1648,21 @@ multisite_painters["log_options"] = {
     "title"   : _("Log: informational part of message"),
     "short"   : _("Info"),
     "columns" : ["log_options"],
-    "paint"   : lambda row: ("", row["log_options"]),
+    "paint"   : lambda row: ("", htmllib.attrencode(row["log_options"])),
+}
+
+def paint_log_comment(msg):
+    if ';' in msg:
+        parts = msg.split(';')
+        if len(parts) > 6:
+          return ("", htmllib.attrencode(parts[-1]))
+    return ("", "")
+
+multisite_painters["log_comment"] = {
+    "title"   : _("Log: comment"),
+    "short"   : _("Comment"),
+    "columns" : ["log_options"],
+    "paint"   : lambda row: paint_log_comment(row['log_options']),
 }
 
 multisite_painters["log_time"] = {
