@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -31,7 +31,8 @@ perfometers = {}
 
 # Helper functions for perfometers
 def perfometer_td(perc, color):
-    return '<td class="inner" style="background-color: %s; width: %d%%;"></td>' % (color, int(float(perc)))
+    return '<td class="inner" style="background-color: %s; ' \
+           'width: %d%%;"></td>' % (color, int(float(perc)))
 
 # Paint linear performeter with one value
 def perfometer_linear(perc, color):
@@ -64,8 +65,8 @@ def perfometer_logarithmic(value, half_value, base, color):
 # Dual logarithmic Perf-O-Meter
 def perfometer_logarithmic_dual(value_left, color_left, value_right, color_right, half_value, base):
     result = '<table><tr>'
-    for where, value, color in [ 
-        ("left", value_left, color_left), 
+    for where, value, color in [
+        ("left", value_left, color_left),
         ("right", value_right, color_right) ]:
         value = float(value)
         if value == 0.0:
@@ -89,16 +90,29 @@ def perfometer_logarithmic_dual(value_left, color_left, value_right, color_right
     return result + '</tr></table>'
 
 
-def number_human_readable(n):
+def number_human_readable(n, precision=1, unit="B"):
     n = float(n)
+    f = "%." + str(precision) + "f"
     if abs(n) > 1024 * 1024 * 1024:
-        return "%.1fG" % (n / (1024.0 * 1024 * 1024))
+        return (f + "G%s") % (n / (1024.0 * 1024 * 1024), unit)
     elif abs(n) > 1024 * 1024:
-        return "%.1fM" % (n / (1024.0 * 1024))
+        return (f + "M%s") % (n / (1024.0 * 1024), unit)
     elif abs(n) > 1024:
-        return "%.fK" % (n / 1024.0)
+        return (f + "k%s") % (n / 1024.0, unit)
     else:
-        return "%.1fB" % n
+        return (f + "%s") % (n, unit)
+
+def age_human_readable(secs):
+    if secs < 240:
+        return "%d sec" % secs
+    mins = secs / 60
+    if mins < 240:
+        return "%d min" % mins
+    hours = mins / 60
+    if hours < 48:
+        return "%d hours" % hours
+    days = hours / 24
+    return "%d days" % days
 
 
 def paint_perfometer(row):
@@ -171,8 +185,8 @@ def paint_perfometer(row):
 multisite_painters["perfometer"] = {
     "title" : _("Service Perf-O-Meter"),
     "short" : _("Perf-O-Meter"),
-    "columns" : [ "service_perf_data", "service_state", 
-                  "service_check_command", "service_pnpgraph_present" ],
+    "columns" : [ "service_perf_data", "service_state",
+                  "service_check_command", "service_pnpgraph_present", "service_plugin_output" ],
     "paint" : paint_perfometer
 }
 
