@@ -7,7 +7,7 @@
 # |           | |___| | | |  __/ (__|   <    | |  | | . \            |
 # |            \____|_| |_|\___|\___|_|\_\___|_|  |_|_|\_\           |
 # |                                                                  |
-# | Copyright Mathias Kettner 2010             mk@mathias-kettner.de |
+# | Copyright Mathias Kettner 2013             mk@mathias-kettner.de |
 # +------------------------------------------------------------------+
 #
 # This file is part of Check_MK.
@@ -65,7 +65,20 @@ def encode_string_json(s):
 
 def render_json(rows, view, group_painters, painters, num_columns, show_checkboxes):
     html.write("[\n")
-    html.write(repr([p[0]["name"] for p in painters]))
+
+    first = True
+    html.write("[")
+    for p in painters:
+        if first:
+            first = False
+        else:
+            html.write(",")
+        content = p[0]["name"]
+        stripped = htmllib.strip_tags(content)
+        utf8 = stripped.encode("utf-8")
+        html.write(encode_string_json(utf8))
+    html.write("]")
+
     for row in rows:
         html.write(",\n[")
         first = True
@@ -75,12 +88,15 @@ def render_json(rows, view, group_painters, painters, num_columns, show_checkbox
             else:
                 html.write(",")
             tdclass, content = p[0]["paint"](row)
+            content = content.replace("<br>","\n")
             stripped = htmllib.strip_tags(content)
             utf8 = stripped.encode("utf-8")
             html.write(encode_string_json(utf8))
         html.write("]")
+
     html.write("\n]\n")
 
+
 multisite_layouts["json"] = {
     "title"  : _("JSON data output"),
     "render" : render_json,
